@@ -1,7 +1,7 @@
 #include <ATen/core/List.h>
 
-namespace c10 {
-namespace detail {
+
+namespace c10::detail {
 bool operator==(const ListImpl& lhs, const ListImpl& rhs) {
   return *lhs.elementType == *rhs.elementType &&
       lhs.list.size() == rhs.list.size() &&
@@ -16,5 +16,4 @@ bool operator==(const ListImpl& lhs, const ListImpl& rhs) {
 ListImpl::ListImpl(list_type list_, TypePtr elementType_)
   : list(std::move(list_))
   , elementType(std::move(elementType_)) {}
-} // namespace detail
-} // namespace c10
+} // namespace c10::detail
