@@ -11,13 +11,6 @@ TORCH_API void RegisterPrivateUse1HooksInterface(at::PrivateUse1HooksInterface*
   privateuse1_hooks = hook_;
 }
 
-TORCH_API at::PrivateUse1HooksInterface* GetPrivateUse1HooksInterface() {
-  TORCH_CHECK(
-      privateuse1_hooks != nullptr,
-      "Please register PrivateUse1HooksInterface by `RegisterPrivateUse1HooksInterface` first.");
-  return privateuse1_hooks;
-}
-
 TORCH_API bool isPrivateUse1HooksRegistered() {
   return privateuse1_hooks != nullptr;
 }
