@@ -5,7 +5,6 @@
 #include <ATen/Parallel.h>
 #include <ATen/native/Pool.h>
 #include <c10/util/irange.h>
-#include <tuple>
 
 #ifndef AT_PER_OPERATOR_HEADERS
 #include <ATen/Functions.h>
@@ -214,15 +213,13 @@ static void avg_pool3d_out_frame(
               continue;
             }
 
-            // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-            int divide_factor;
+            int64_t divide_factor = 0;
             if (divisor_override.has_value()) {
               divide_factor = divisor_override.value();
             } else {
               if(count_include_pad) {
                 divide_factor = pool_size;
               } else {
-                // NOLINTNEXTLINE(cppcoreguidelines-narrowing-conversions,bugprone-narrowing-conversions)
                 divide_factor = (tend - tstart) * (hend - hstart) * (wend - wstart);
               }
             }
@@ -366,21 +363,18 @@ static void avg_pool3d_backward_out_frame(
 {
   at::parallel_for(0, nslices, 0, [&](int64_t start, int64_t end) {
     for (const auto k : c10::irange(start, end)) {
-      // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-      int64_t i, j, ti;
-
       /* local pointers */
       scalar_t *ip = gradInput_p + k * itime * iwidth * iheight;
       const scalar_t *op = gradOutput_p + k * otime * owidth * oheight;
-      for (i = 0; i < itime*iwidth*iheight; i++)
+      for (int64_t i = 0; i < itime*iwidth*iheight; i++)
         *(ip + i) = 0;
 
       /* loop over output */
-      for (ti = 0; ti < otime; ti++)
+      for (int64_t ti = 0; ti < otime; ti++)
       {
-        for (i = 0; i < oheight; i++)
+        for (int64_t i = 0; i < oheight; i++)
         {
-          for (j = 0; j < owidth; j++)
+          for (int64_t j = 0; j < owidth; j++)
           {
             int64_t tstart = ti * dT - padT;
             int64_t hstart = i  * dH - padH;
@@ -396,15 +390,13 @@ static void avg_pool3d_backward_out_frame(
             hend = std::min(hend, iheight);
             wend = std::min(wend, iwidth);
 
-            // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-            int divide_factor;
+            int64_t divide_factor = 0;
             if (divisor_override.has_value()) {
               divide_factor = divisor_override.value();
             } else {
               if(count_include_pad) {
                 divide_factor = pool_size;
               } else {
-                // NOLINTNEXTLINE(cppcoreguidelines-narrowing-conversions,bugprone-narrowing-conversions)
                 divide_factor = (tend - tstart) * (hend - hstart) * (wend - wstart);
               }
             }
@@ -412,13 +404,11 @@ static void avg_pool3d_backward_out_frame(
             /* scatter gradients out to footprint: */
             scalar_t val  = *op++;
 
-            // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-            int64_t x,y,z;
-            for (z = tstart; z < tend; z++)
+            for (auto z = tstart; z < tend; z++)
             {
-              for (y = hstart; y < hend; y++)
+              for (auto y = hstart; y < hend; y++)
               {
-                for (x = wstart; x < wend; x++)
+                for (auto x = wstart; x < wend; x++)
                 {
                   *(ip + z * iheight * iwidth + y * iwidth + x) += val / divide_factor;
                 }
