@@ -733,9 +733,10 @@ struct HelperInterpBase {
     auto new_shape = std::vector<int64_t>(ndims, 1);
     new_shape[reshape_dim] = output_size;
 
-    for (const auto j C10_UNUSED : c10::irange(interp_size)) {
-      output.emplace_back(empty(new_shape, CPU(c10::CppTypeToScalarType<int64_t>())));
-      output.emplace_back(empty(new_shape, CPU(output_type)));
+    for ([[maybe_unused]] const auto j : c10::irange(interp_size)) {
+      output.emplace_back(
+          empty(new_shape, at::device(kCPU).dtype(c10::CppTypeToScalarType<int64_t>())));
+      output.emplace_back(empty(new_shape, at::device(kCPU).dtype(output_type)));
     }
   }
 
@@ -877,16 +878,16 @@ struct HelperInterpBase {
 
     // Bounds approach as in PIL: xmin/xmax
     output.emplace_back(
-        empty(new_shape, CPU(c10::CppTypeToScalarType<int64_t>())));
+        empty(new_shape, at::device(kCPU).dtype(c10::CppTypeToScalarType<int64_t>())));
     output.emplace_back(
-        empty(new_shape, CPU(c10::CppTypeToScalarType<int64_t>())));
+        empty(new_shape, at::device(kCPU).dtype(c10::CppTypeToScalarType<int64_t>())));
     output.emplace_back(
-        empty(new_shape, CPU(c10::CppTypeToScalarType<int64_t>())));
+        empty(new_shape, at::device(kCPU).dtype(c10::CppTypeToScalarType<int64_t>())));
 
     {
       // Weights
       new_shape[reshape_dim] = output_size * max_interp_size;
-      auto wts = empty(new_shape, CPU(c10::CppTypeToScalarType<scalar_t>()));
+      auto wts = empty(new_shape, at::device(kCPU).dtype(c10::CppTypeToScalarType<scalar_t>()));
       auto strides = wts.strides().vec();
       strides[reshape_dim] = 0;
       new_shape[reshape_dim] = output_size;
@@ -894,7 +895,7 @@ struct HelperInterpBase {
       output.emplace_back(wts);
       // Weights indices
       output.emplace_back(
-          empty(new_shape, CPU(c10::CppTypeToScalarType<int64_t>())));
+          empty(new_shape, at::device(kCPU).dtype(c10::CppTypeToScalarType<int64_t>())));
     }
 
     int64_t* idx_ptr_xmin = output[0].data_ptr<int64_t>();
@@ -1047,10 +1048,11 @@ struct HelperInterpNearest : public HelperInterpBase {
     auto new_shape = std::vector<int64_t>(ndims, 1);
     new_shape[reshape_dim] = output_size;
 
-    for (const auto j C10_UNUSED : c10::irange(interp_size)) {
-      output.emplace_back(empty(new_shape, CPU(c10::CppTypeToScalarType<int64_t>())));
+    for ([[maybe_unused]] const auto j : c10::irange(interp_size)) {
+      output.emplace_back(
+          empty(new_shape, at::device(kCPU).dtype(c10::CppTypeToScalarType<int64_t>())));
       // Defines weights for consistency, but not used
-      output.emplace_back(at::ones(new_shape, CPU(output_type)));
+      output.emplace_back(at::ones(new_shape, at::device(kCPU).dtype(output_type)));
     }
   }
 
@@ -2058,20 +2060,20 @@ void upsample_bicubic2d_aa_backward_kernel_impl(
 
 } // anonymous namespace
 
-REGISTER_DISPATCH(upsample_nearest1d_kernel, &upsample_nearest1d_kernel_impl);
-REGISTER_DISPATCH(_upsample_nearest_exact1d_kernel, &_upsample_nearest_exact1d_kernel_impl);
-REGISTER_DISPATCH(upsample_nearest2d_kernel, &upsample_nearest2d_kernel_impl);
-REGISTER_DISPATCH(_upsample_nearest_exact2d_kernel, &_upsample_nearest_exact2d_kernel_impl);
-REGISTER_DISPATCH(upsample_nearest3d_kernel, &upsample_nearest3d_kernel_impl);
-REGISTER_DISPATCH(_upsample_nearest_exact3d_kernel, &_upsample_nearest_exact3d_kernel_impl);
+REGISTER_DISPATCH(upsample_nearest1d_kernel, &upsample_nearest1d_kernel_impl)
+REGISTER_DISPATCH(_upsample_nearest_exact1d_kernel, &_upsample_nearest_exact1d_kernel_impl)
+REGISTER_DISPATCH(upsample_nearest2d_kernel, &upsample_nearest2d_kernel_impl)
+REGISTER_DISPATCH(_upsample_nearest_exact2d_kernel, &_upsample_nearest_exact2d_kernel_impl)
+REGISTER_DISPATCH(upsample_nearest3d_kernel, &upsample_nearest3d_kernel_impl)
+REGISTER_DISPATCH(_upsample_nearest_exact3d_kernel, &_upsample_nearest_exact3d_kernel_impl)
 
-REGISTER_DISPATCH(upsample_linear1d_kernel, &upsample_linear1d_kernel_impl);
-REGISTER_DISPATCH(upsample_bilinear2d_kernel, &upsample_bilinear2d_kernel_impl);
-REGISTER_DISPATCH(_upsample_bilinear2d_aa_kernel, &upsample_bilinear2d_aa_kernel_impl);
-REGISTER_DISPATCH(_upsample_bilinear2d_aa_backward_kernel, &upsample_bilinear2d_aa_backward_kernel_impl);
-REGISTER_DISPATCH(upsample_trilinear3d_kernel, &upsample_trilinear3d_kernel_impl);
+REGISTER_DISPATCH(upsample_linear1d_kernel, &upsample_linear1d_kernel_impl)
+REGISTER_DISPATCH(upsample_bilinear2d_kernel, &upsample_bilinear2d_kernel_impl)
+REGISTER_DISPATCH(_upsample_bilinear2d_aa_kernel, &upsample_bilinear2d_aa_kernel_impl)
+REGISTER_DISPATCH(_upsample_bilinear2d_aa_backward_kernel, &upsample_bilinear2d_aa_backward_kernel_impl)
+REGISTER_DISPATCH(upsample_trilinear3d_kernel, &upsample_trilinear3d_kernel_impl)
 
-REGISTER_DISPATCH(upsample_bicubic2d_kernel, &upsample_bicubic2d_kernel_impl);
-REGISTER_DISPATCH(_upsample_bicubic2d_aa_kernel, &upsample_bicubic2d_aa_kernel_impl);
-REGISTER_DISPATCH(_upsample_bicubic2d_aa_backward_kernel, &upsample_bicubic2d_aa_backward_kernel_impl);
+REGISTER_DISPATCH(upsample_bicubic2d_kernel, &upsample_bicubic2d_kernel_impl)
+REGISTER_DISPATCH(_upsample_bicubic2d_aa_kernel, &upsample_bicubic2d_aa_kernel_impl)
+REGISTER_DISPATCH(_upsample_bicubic2d_aa_backward_kernel, &upsample_bicubic2d_aa_backward_kernel_impl)
 } // namespace at::native
