@@ -2,8 +2,7 @@
 #include <ATen/core/Tensor.h>
 #include <ATen/SparseCsrTensorUtils.h>
 
-namespace at {
-namespace sparse_csr {
+namespace at::sparse_csr {
 Tensor& _sparse_mm_mkl_(
     Tensor& self,
     const SparseCsrTensor& sparse_,
@@ -11,5 +10,4 @@ Tensor& _sparse_mm_mkl_(
     const Tensor& t,
     const Scalar& alpha,
     const Scalar& beta);
-} // namespace native
 } // namespace at
