@@ -28,7 +28,7 @@ namespace mps {
 static Tensor& pad_out_template(Tensor& output,
                                 const Tensor& input_,
                                 IntArrayRef padding,
-                                const c10::optional<Tensor>& grad_output_opt,
+                                const std::optional<Tensor>& grad_output_opt,
                                 MPSGraphPaddingMode mode,
                                 double constantValue,
                                 const string op_name) {
@@ -232,13 +232,6 @@ static Tensor& pad_out_template(Tensor& output,
         endsVec[padIdx] = @(input.size(padIdx) + padding[rightIdx]);
         endMask &= ~(1U << padIdx);
       }
-      // workaround for the right padding bug in Monterey
-    } else if (!is_macos_13_or_newer()) {
-      if (padding[rightIdx] == 1 && padding[leftIdx] == 0) {
-        rightPadVec[padIdx] = @(2);
-        endsVec[padIdx] = @(input.size(padIdx) + 2);
-        endMask &= ~(1U << padIdx);
-      }
     }
   }
   MPSShape* leftPadding = [NSArray arrayWithObjects:leftPadVec.data() count:ndims];
@@ -317,9 +310,7 @@ static Tensor& pad_out_template(Tensor& output,
     if (is_backward_pass) {
       feeds[gradOutputPlaceholder.getMPSGraphTensor()] = gradOutputPlaceholder.getMPSGraphTensorData();
     }
-    NSDictionary<MPSGraphTensor*, MPSGraphTensorData*>* results =
-        @{outputPlaceholder.getMPSGraphTensor() : outputPlaceholder.getMPSGraphTensorData()};
-    runMPSGraph(getCurrentMPSStream(), cachedGraph->graph(), feeds, results);
+    runMPSGraph(getCurrentMPSStream(), cachedGraph->graph(), feeds, outputPlaceholder);
   }
   return output;
 }
@@ -331,7 +322,7 @@ TORCH_IMPL_FUNC(reflection_pad1d_out_mps)
   mps::pad_out_template(const_cast<Tensor&>(output),
                         input,
                         padding,
-                        c10::nullopt,
+                        std::nullopt,
                         MPSGraphPaddingModeReflect,
                         0.0,
                         "reflection_pad1d_out_mps");
@@ -354,7 +345,7 @@ TORCH_IMPL_FUNC(replication_pad1d_out_mps)
   mps::pad_out_template(const_cast<Tensor&>(output),
                         input,
                         padding,
-                        c10::nullopt,
+                        std::nullopt,
                         MPSGraphPaddingModeClampToEdge,
                         0.0,
                         "replication_pad1d_out_mps");
@@ -374,12 +365,12 @@ TORCH_IMPL_FUNC(replication_pad1d_backward_out_mps)
 
 // 2D Reflection and Replication Padding
 Tensor& reflection_pad2d_out_mps(const Tensor& input, IntArrayRef padding, Tensor& output) {
-  return mps::pad_out_template(output, input, padding, c10::nullopt, MPSGraphPaddingModeReflect, 0.0, __func__);
+  return mps::pad_out_template(output, input, padding, std::nullopt, MPSGraphPaddingModeReflect, 0.0, __func__);
 }
 
 Tensor reflection_pad2d_mps(const Tensor& input, IntArrayRef padding) {
   Tensor output = at::empty({0}, input.options());
-  return mps::pad_out_template(output, input, padding, c10::nullopt, MPSGraphPaddingModeReflect, 0.0, __func__);
+  return mps::pad_out_template(output, input, padding, std::nullopt, MPSGraphPaddingModeReflect, 0.0, __func__);
 }
 
 Tensor& reflection_pad2d_backward_out_mps(const Tensor& grad_output,
@@ -400,7 +391,7 @@ TORCH_IMPL_FUNC(replication_pad2d_out_mps)
   mps::pad_out_template(const_cast<Tensor&>(output),
                         input,
                         padding,
-                        c10::nullopt,
+                        std::nullopt,
                         MPSGraphPaddingModeClampToEdge,
                         0.0,
                         "replication_pad2d_out_mps");
@@ -425,7 +416,7 @@ TORCH_IMPL_FUNC(reflection_pad3d_out_mps)
   mps::pad_out_template(const_cast<Tensor&>(output),
                         input,
                         padding,
-                        c10::nullopt,
+                        std::nullopt,
                         MPSGraphPaddingModeReflect,
                         0.0,
                         "reflection_pad3d_out_mps");
@@ -448,7 +439,7 @@ TORCH_IMPL_FUNC(replication_pad3d_out_mps)
   mps::pad_out_template(const_cast<Tensor&>(output),
                         input,
                         padding,
-                        c10::nullopt,
+                        std::nullopt,
                         MPSGraphPaddingModeClampToEdge,
                         0.0,
                         "replication_pad3d_out_mps");
@@ -467,7 +458,7 @@ Tensor replication_pad3d_backward_mps(const Tensor& grad_output, const Tensor& i
   return mps::pad_out_template(grad_input, input, padding, grad_output, MPSGraphPaddingModeClampToEdge, 0.0, __func__);
 }
 
-// backward pass is exlicitly handled in autograd by negating the "pad" argument
+// backward pass is explicitly handled in autograd by negating the "pad" argument
 Tensor constant_pad_nd_mps(const Tensor& self, IntArrayRef pad, const Scalar& value) {
   if (pad.size() > 6) {
     TORCH_WARN_ONCE("MPS: The constant padding of more than 3 dimensions is not currently supported natively. ",
@@ -476,7 +467,7 @@ Tensor constant_pad_nd_mps(const Tensor& self, IntArrayRef pad, const Scalar& va
   }
   Tensor output = at::empty({0}, self.options());
   return mps::pad_out_template(
-      output, self, pad, c10::nullopt, MPSGraphPaddingModeConstant, value.toDouble(), __func__);
+      output, self, pad, std::nullopt, MPSGraphPaddingModeConstant, value.toDouble(), __func__);
 }
 
 } // namespace at::native
