@@ -5,8 +5,7 @@ from pathlib import Path
 
 import torch
 import torch._dynamo as torchdynamo
-
-from torch._export.db.case import ExportCase, normalize_inputs
+from torch._export.db.case import ExportCase
 from torch._export.db.examples import all_examples
 from torch.export import export
 
@@ -33,8 +32,6 @@ def generate_example_rst(example_case: ExportCase):
     )
     with open(source_file) as file:
         source_code = file.read()
-    source_code = re.sub(r"from torch\._export\.db\.case import .*\n", "", source_code)
-    source_code = re.sub(r"@export_case\((.|\n)*?\)\n", "", source_code)
     source_code = source_code.replace("\n", "\n    ")
     splitted_source_code = re.split(r"@export_rewrite_case.*\n", source_code)
 
@@ -43,6 +40,12 @@ def generate_example_rst(example_case: ExportCase):
         2,
     }, f"more than one @export_rewrite_case decorator in {source_code}"
 
+    more_arguments = ""
+    if example_case.example_kwargs:
+        more_arguments += ", example_kwargs"
+    if example_case.dynamic_shapes:
+        more_arguments += ", dynamic_shapes=dynamic_shapes"
+
     # Generate contents of the .rst file
     title = f"{example_case.name}"
     doc_contents = f"""{title}
@@ -60,6 +63,8 @@ Original source code:
 
     {splitted_source_code[0]}
 
+    torch.export.export(model, example_args{more_arguments})
+
 Result:
 
 .. code-block::
@@ -68,11 +73,10 @@ Result:
 
     # Get resulting graph from dynamo trace
     try:
-        inputs = normalize_inputs(example_case.example_inputs)
         exported_program = export(
             model,
-            inputs.args,
-            inputs.kwargs,
+            example_case.example_args,
+            example_case.example_kwargs,
             dynamic_shapes=example_case.dynamic_shapes,
         )
         graph_output = str(exported_program)
