@@ -888,7 +888,7 @@ TEST_F(Kernel, SumAllAxes) {
     parseIR(graph_string, &*graph);
 
     auto o = at::empty({}, TensorOptions(kCPU));
-    c10::optional<c10::ScalarType> dtype;
+    std::optional<c10::ScalarType> dtype;
     if (scalar_type != ScalarType::Undefined) {
       dtype = static_cast<c10::ScalarType>(scalar_type);
     }
@@ -947,7 +947,7 @@ TEST_F(Kernel, SumOneAxis) {
         env.d("dim", dim);
         env.d("keepdim", keepdim);
         env.s("dtype", dtypeConstant(scalar_type));
-        c10::optional<c10::ScalarType> dtype;
+        std::optional<c10::ScalarType> dtype;
         if (scalar_type != ScalarType::Undefined) {
           dtype = static_cast<c10::ScalarType>(scalar_type);
         }
@@ -1113,7 +1113,7 @@ TEST_F(Kernel, Softmax2D) {
         const auto verification_pattern =
             format(verification_template, ver_env);
 
-        // verication sting temporarily disabled until
+        // verification sting temporarily disabled until
         // inlining of exp() is benchmarked and determined
         // torch::jit::testing::FileCheck().run(verification_pattern,
         // oss.str());
@@ -1192,7 +1192,7 @@ TEST_F(Kernel, Softmax3D) {
       ver_env.d("softmax_dim_size", softmax_dim_size);
       const auto verification_pattern = format(verification_template, ver_env);
 
-      // verication sting temporarily disabled until
+      // verification sting temporarily disabled until
       // inlining of exp() is benchmarked and determined
       // torch::jit::testing::FileCheck().run(verification_pattern, oss.str());
 
@@ -1275,7 +1275,7 @@ TEST_F(Kernel, Softmax4D) {
       ver_env.d("softmax_dim_size", softmax_dim_size);
       const auto verification_pattern = format(verification_template, ver_env);
 
-      // verication sting temporarily disabled until
+      // verification sting temporarily disabled until
       // inlining of exp() is benchmarked and determined
       // torch::jit::testing::FileCheck().run(verification_pattern, oss.str());
 
@@ -1548,7 +1548,7 @@ TEST_F(Kernel, ConstantTensorsNonContiguous) {
   auto graph = std::make_shared<Graph>();
   parseIR(graph_string, &*graph);
   // IRParser doesn't support tensor constants, so we generate several aten
-  // calls to produce non-contiguos constant tensor and then const-prop it
+  // calls to produce non-contiguous constant tensor and then const-prop it
   ConstantPropagation(graph);
 
   TensorExprKernel k(graph);
@@ -1637,7 +1637,7 @@ TEST_F(Kernel, CodegenInspection) {
   auto graph = std::make_shared<Graph>();
   parseIR(graph_string, &*graph);
   // IRParser doesn't support tensor constants, so we generate several aten
-  // calls to produce non-contiguos constant tensor and then const-prop it
+  // calls to produce non-contiguous constant tensor and then const-prop it
   ConstantPropagation(graph);
 
   TensorExprKernel k(graph);
@@ -1665,9 +1665,9 @@ Tensor lowerNanToNum(
     const std::vector<ArgValue>& inputs,
     const std::vector<ExprHandle>& outputShape,
     const std::vector<ExprHandle>& outputStrides,
-    const c10::optional<ScalarType>& outputType,
+    const std::optional<ScalarType>& outputType,
     at::Device device) {
-  auto input_buf = c10::get<BufHandle>(inputs[0]);
+  auto input_buf = std::get<BufHandle>(inputs[0]);
   auto e = Compute(
       "custom_nan_to_num",
       outputShape,
