@@ -4,11 +4,8 @@ import itertools
 
 import torch
 from torch.distributed._tensor import distribute_tensor, DTensor
-from torch.distributed._tensor._utils import (
-    compute_local_shape,
-    compute_local_shape_and_global_offset,
-)
-from torch.distributed.device_mesh import DeviceMesh, init_device_mesh
+from torch.distributed._tensor._utils import compute_local_shape_and_global_offset
+from torch.distributed.device_mesh import init_device_mesh
 from torch.distributed.tensor._dtensor_spec import DTensorSpec, TensorMeta
 from torch.distributed.tensor.debug import CommDebugMode
 from torch.distributed.tensor.placement_types import _StridedShard, Replicate, Shard
@@ -27,48 +24,23 @@ class UtilTest(DTensorTestBase):
     def world_size(self):
         return 8
 
-    @with_comms
-    def test_compute_local_shape_2d_uneven(self):
-        # mesh: 4 * 2
-        mesh_tensor = torch.arange(self.world_size).reshape(4, 2)
-        mesh = DeviceMesh(self.device_type, mesh_tensor)
-        size = torch.Size([7, 7])
-        rank_coordinates = mesh.get_coordinate()
-
-        # replicate, shard
-        placements2 = [Replicate(), Shard(0)]
-        local_size2 = compute_local_shape(size, mesh, placements2)
-        if rank_coordinates[1] < 1:
-            self.assertEqual(local_size2, torch.Size([4, 7]))
-        else:
-            self.assertEqual(local_size2, torch.Size([3, 7]))
-
-        # shard, shard
-        placements3 = [Shard(0), Shard(1)]
-        local_size3 = compute_local_shape(size, mesh, placements3)
-        # first dim
-        if rank_coordinates[0] < 3:
-            self.assertEqual(local_size3[0], 2)
-        else:
-            self.assertEqual(local_size3[0], 1)
-        # second dim
-        if rank_coordinates[1] < 1:
-            self.assertEqual(local_size3[1], 4)
-        else:
-            self.assertEqual(local_size3[1], 3)
+    def _compute_start_end_offsets(self, global_offset, local_size, n_dim):
+        offset = []
+        for i in range(n_dim):
+            offset.append(((global_offset[i]), (global_offset[i] + local_size[i])))
+        return offset
 
     @with_comms
     def test_compute_local_shape_and_global_offset_1D(self):
         one_d_placements = [[Shard(0)], [Replicate()]]
 
+        device_mesh = init_device_mesh(self.device_type, (self.world_size,))
         for placements in one_d_placements:
             # When the placements is [Shard(0)], we test for three different scenarios:
             # 1) sharding resulting in empty shards on all or some of the ranks
             # 2) sharding resulting in shards of different size across different ranks
             # 3) sharding resulting in non-empty shards of same size across all ranks
             for size in range(self.world_size * 2 + 1):
-                mesh_tensor = torch.arange(self.world_size)
-                device_mesh = DeviceMesh(self.device_type, mesh_tensor)
                 global_tensor = torch.arange(size)
                 global_shape = global_tensor.size()
 
@@ -76,10 +48,8 @@ class UtilTest(DTensorTestBase):
                 local_size, global_offset = compute_local_shape_and_global_offset(
                     global_shape, device_mesh, placements
                 )
-
-                # TODO: make this test cleaner and work for nD
-                dim0_start = global_offset[0]
-                dim0_end = global_offset[0] + local_size[0]
+                dim = self._compute_start_end_offsets(global_offset, local_size, 1)
+                dim0_start, dim0_end = dim[0][0], dim[0][1]
 
                 # Check the local tensor of dtensor is exactly the same
                 # if we slice the global_tensor with local_size and global_offset
@@ -96,12 +66,12 @@ class UtilTest(DTensorTestBase):
             itertools.combinations_with_replacement(two_d_placements_options, 2)
         )
 
+        # mesh: 2 * 4
+        device_mesh = init_device_mesh(self.device_type, (2, 4))
         for placements in two_d_placements:
-            for dim_0_size in (1, 2, 4, 8):
-                # mesh: 2 * 4
-                mesh_tensor = torch.arange(self.world_size).reshape(2, 4)
-                device_mesh = DeviceMesh(self.device_type, mesh_tensor)
-                global_tensor = torch.arange(64).view(dim_0_size, -1)
+            for dim_0_size in range(1, 9):
+                nelem = 64 // dim_0_size * dim_0_size
+                global_tensor = torch.arange(nelem).view(dim_0_size, -1)
                 global_shape = global_tensor.size()
 
                 dtensor = distribute_tensor(global_tensor, device_mesh, placements)
@@ -109,11 +79,9 @@ class UtilTest(DTensorTestBase):
                     global_shape, device_mesh, placements
                 )
 
-                # TODO: make this test cleaner and work for nD
-                dim0_start = global_offset[0]
-                dim0_end = global_offset[0] + local_size[0]
-                dim1_start = global_offset[1]
-                dim1_end = global_offset[1] + local_size[1]
+                dim = self._compute_start_end_offsets(global_offset, local_size, 2)
+                dim0_start, dim0_end = dim[0][0], dim[0][1]
+                dim1_start, dim1_end = dim[1][0], dim[1][1]
 
                 # Check the local tensor of dtensor is exactly the same
                 # if we slice the global_tensor with local_size and global_offset
