@@ -6,14 +6,20 @@ import torch
 import torch.nn as nn
 from torch import distributed as dist
 from torch.distributed.fsdp import FullyShardedDataParallel as FSDP
-from torch.distributed.fsdp.flat_param import (
+from torch.distributed.fsdp._flat_param import (
     FlatParamHandle,
     FlatParamShardMetadata,
     HandleShardingStrategy,
 )
 from torch.testing._internal.common_distributed import skip_if_lt_x_gpu
 from torch.testing._internal.common_fsdp import FSDPTest
-from torch.testing._internal.common_utils import run_tests, TEST_WITH_DEV_DBG_ASAN
+from torch.testing._internal.common_utils import (
+    instantiate_parametrized_tests,
+    parametrize,
+    run_tests,
+    TEST_WITH_DEV_DBG_ASAN,
+)
+
 
 if not dist.is_available():
     print("Distributed not available, skipping tests", file=sys.stderr)
@@ -47,6 +53,7 @@ class TestFlattenParams(FSDPTest):
             "keep_low_precision_grads": False,
             "process_group": self.process_group,
             "use_orig_params": False,
+            "fsdp_extension": None,
         }
 
     def _get_transformer(self, seed=0):
@@ -58,7 +65,7 @@ class TestFlattenParams(FSDPTest):
             dim_feedforward=128,
             dropout=0.1,
         )
-        module.register_buffer("dummy_buffer", torch.tensor(1.0))
+        module.dummy_buffer = nn.Buffer(torch.tensor(1.0))
 
         def get_input(device, dtype):
             torch.manual_seed(1)  # keep everything deterministic
@@ -333,6 +340,8 @@ class TestFlattenParams(FSDPTest):
             expected=FlatParamShardMetadata(
                 param_names=["0.weight"],
                 param_shapes=[(10, 10)],
+                param_strides=[(10, 1)],
+                param_contiguities=[True],
                 param_numels=[100],
                 param_offsets=[(0, 0)],
             ),
@@ -344,6 +353,8 @@ class TestFlattenParams(FSDPTest):
             expected=FlatParamShardMetadata(
                 param_names=["0.weight"],
                 param_shapes=[(10, 10)],
+                param_strides=[(10, 1)],
+                param_contiguities=[True],
                 param_numels=[100],
                 param_offsets=[(0, 50)],
             ),
@@ -355,6 +366,8 @@ class TestFlattenParams(FSDPTest):
             expected=FlatParamShardMetadata(
                 param_names=["0.weight"],
                 param_shapes=[(10, 10)],
+                param_strides=[(10, 1)],
+                param_contiguities=[True],
                 param_numels=[100],
                 param_offsets=[(0, 99)],
             ),
@@ -366,6 +379,8 @@ class TestFlattenParams(FSDPTest):
             expected=FlatParamShardMetadata(
                 param_names=["0.weight", "2.weight"],
                 param_shapes=[(10, 10), (10, 10)],
+                param_strides=[(10, 1), (10, 1)],
+                param_contiguities=[True, True],
                 param_numels=[100, 100],
                 param_offsets=[(50, 99), (0, 49)],
             ),
@@ -377,6 +392,8 @@ class TestFlattenParams(FSDPTest):
             expected=FlatParamShardMetadata(
                 param_names=["0.weight", "2.weight"],
                 param_shapes=[(10, 10), (10, 10)],
+                param_strides=[(10, 1), (10, 1)],
+                param_contiguities=[True, True],
                 param_numels=[100, 100],
                 param_offsets=[(50, 99), (0, 99)],
             ),
@@ -388,6 +405,8 @@ class TestFlattenParams(FSDPTest):
             expected=FlatParamShardMetadata(
                 param_names=["0.weight", "2.weight"],
                 param_shapes=[(10, 10), (10, 10)],
+                param_strides=[(10, 1), (10, 1)],
+                param_contiguities=[True, True],
                 param_numels=[100, 100],
                 param_offsets=[(99, 99), (0, 99)],
             ),
@@ -399,6 +418,8 @@ class TestFlattenParams(FSDPTest):
             expected=FlatParamShardMetadata(
                 param_names=["2.weight"],
                 param_shapes=[(10, 10)],
+                param_strides=[(10, 1)],
+                param_contiguities=[True],
                 param_numels=[100],
                 param_offsets=[(0, 99)],
             ),
@@ -410,6 +431,8 @@ class TestFlattenParams(FSDPTest):
             expected=FlatParamShardMetadata(
                 param_names=["2.weight", "4.weight"],
                 param_shapes=[(10, 10), (10, 10)],
+                param_strides=[(10, 1), (10, 1)],
+                param_contiguities=[True, True],
                 param_numels=[100, 100],
                 param_offsets=[(0, 99), (0, 99)],
             ),
@@ -421,6 +444,8 @@ class TestFlattenParams(FSDPTest):
             expected=FlatParamShardMetadata(
                 param_names=["2.weight", "4.weight"],
                 param_shapes=[(10, 10), (10, 10)],
+                param_strides=[(10, 1), (10, 1)],
+                param_contiguities=[True, True],
                 param_numels=[100, 100],
                 param_offsets=[(0, 99), (0, 99)],
             ),
@@ -432,6 +457,8 @@ class TestFlattenParams(FSDPTest):
             expected=FlatParamShardMetadata(
                 param_names=["4.weight"],
                 param_shapes=[(10, 10)],
+                param_strides=[(10, 1)],
+                param_contiguities=[True],
                 param_numels=[100],
                 param_offsets=[(99, 99)],
             ),
@@ -467,6 +494,8 @@ class TestFlattenParams(FSDPTest):
             expected=FlatParamShardMetadata(
                 param_names=["0.weight", "1.weight"],
                 param_shapes=[(7, 3), (5, 7)],
+                param_strides=[(3, 1), (7, 1)],
+                param_contiguities=[True, True],
                 param_numels=[21, 35],
                 # 21 + (3) + 19 = 43
                 param_offsets=[(0, 20), (0, 18)],
@@ -480,6 +509,8 @@ class TestFlattenParams(FSDPTest):
             expected=FlatParamShardMetadata(
                 param_names=["1.weight", "2.weight"],
                 param_shapes=[(5, 7), (5, 5)],
+                param_strides=[(7, 1), (5, 1)],
+                param_contiguities=[True, True],
                 param_numels=[35, 25],
                 # 16 + (1) + 25 = 42
                 param_offsets=[(19, 34), (0, 24)],
@@ -517,6 +548,8 @@ class TestFlattenParams(FSDPTest):
             expected=FlatParamShardMetadata(
                 param_names=["0.weight", "1.weight"],
                 param_shapes=[(5, 2), (5, 5)],
+                param_strides=[(2, 1), (5, 1)],
+                param_contiguities=[True, True],
                 param_numels=[10, 25],
                 # 10 + (6) + 16 = 32
                 param_offsets=[(0, 9), (0, 15)],
@@ -530,6 +563,8 @@ class TestFlattenParams(FSDPTest):
             expected=FlatParamShardMetadata(
                 param_names=["1.weight", "2.weight"],
                 param_shapes=[(5, 5), (3, 5)],
+                param_strides=[(5, 1), (5, 1)],
+                param_contiguities=[True, True],
                 param_numels=[25, 15],
                 # 9 + (7) + 15 = 31
                 param_offsets=[(16, 24), (0, 14)],
@@ -563,6 +598,57 @@ class TestFlattenParams(FSDPTest):
             msg=f"{handle.shard_metadata()}, {expected}",
         )
 
+    @parametrize("memory_format", [torch.contiguous_format, torch.channels_last])
+    def test_flat_param_shard_metadata_with_memory_format(self, memory_format):
+        """
+        Tests that ``FlatParameter`` shard metadata are computed as expected
+        with alignment padding and parameter full precision.
+        """
+        module = torch.nn.Sequential(
+            torch.nn.Conv2d(10, 20, 3, bias=False),  # 0.weight, 1800 params
+            torch.nn.Conv2d(20, 10, 5, bias=False),  # 1.weight, 5000 params
+            torch.nn.Conv2d(10, 10, 1, bias=False),  # 2.weight, 100 params
+        ).to(memory_format=memory_format)
+        params_to_flatten = list(module.parameters())
+        handle_kwargs = self._get_default_config()
+        handle_kwargs["use_orig_params"] = True
+        handle = FlatParamHandle(params_to_flatten, module, **handle_kwargs)
+        contiguous_tensors = memory_format == torch.contiguous_format
+        self._test_flat_param_shard_metadata(
+            handle,
+            # Emulate rank 0 of 2 ranks
+            start=0,
+            end=2999,
+            expected=FlatParamShardMetadata(
+                param_names=["0.weight", "1.weight"],
+                param_shapes=[(20, 10, 3, 3), (10, 20, 5, 5)],
+                param_strides=[(90, 9, 3, 1), (500, 25, 5, 1)]
+                if contiguous_tensors
+                else [(90, 1, 30, 10), (500, 1, 100, 20)],
+                param_contiguities=[contiguous_tensors, contiguous_tensors],
+                param_numels=[1800, 5000],
+                param_offsets=[(0, 1799), (0, 1199)],
+            ),
+        )
+        self._test_flat_param_shard_metadata(
+            handle,
+            # Emulate rank 1 of 2 ranks
+            start=3000,
+            end=6899,
+            expected=FlatParamShardMetadata(
+                param_names=["1.weight", "2.weight"],
+                param_shapes=[(10, 20, 5, 5), (10, 10, 1, 1)],
+                param_strides=[(500, 25, 5, 1), (10, 1, 1, 1)]
+                if contiguous_tensors
+                else [(500, 1, 100, 20), (10, 1, 10, 10)],
+                param_contiguities=[contiguous_tensors, contiguous_tensors],
+                param_numels=[5000, 100],
+                param_offsets=[(1200, 4999), (0, 99)],
+            ),
+        )
+
+
+instantiate_parametrized_tests(TestFlattenParams)
 
 if __name__ == "__main__":
     run_tests()
