@@ -8,20 +8,19 @@ import os
 import random
 import sys
 import tempfile
+from datetime import timedelta
 from functools import reduce
 from itertools import groupby
 
 import torch
 import torch.distributed as c10d
 
+
 if not c10d.is_available() or not c10d.is_gloo_available():
     print("c10d GLOO not available, skipping tests", file=sys.stderr)
     sys.exit(0)
 
 import test_c10d_common
-import torch.distributed as dist
-import torch.nn.functional as F
-import torch.testing._internal.common_utils as common
 from test_c10d_common import (
     gpus_for_rank,
     LOOPBACK,
@@ -29,6 +28,10 @@ from test_c10d_common import (
     SparseGradientModule,
     Task,
 )
+
+import torch.distributed as dist
+import torch.nn.functional as F
+import torch.testing._internal.common_utils as common
 from torch import nn
 from torch.distributed._shard.sharded_tensor import (
     init_from_local_shards,
@@ -43,7 +46,6 @@ from torch.testing._internal.common_distributed import (
     requires_gloo,
     simple_sparse_reduce_tests,
     skip_if_lt_x_gpu,
-    skip_if_win32,
     verify_ddp_error_logged,
 )
 from torch.testing._internal.common_utils import (
@@ -338,8 +340,12 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
                 ]
 
                 output = broadcast(xs, i, j)
-                self.assertEqual(torch.tensor([i * num + j], dtype=torch.float32), output[0])
-                self.assertEqual(torch.tensor([i * num + j], dtype=torch.float32), output[1])
+                self.assertEqual(
+                    torch.tensor([i * num + j], dtype=torch.float32), output[0]
+                )
+                self.assertEqual(
+                    torch.tensor([i * num + j], dtype=torch.float32), output[1]
+                )
 
         # Test overloaded convenience function
         x = torch.tensor([self.rank + 1.0])
@@ -418,7 +424,7 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
 
         # Single input tests
         tests = simple_reduce_tests(self.rank, self.world_size)
-        for (op, input, expected) in tests:
+        for op, input, expected in tests:
             opts = c10d.AllreduceOptions()
             opts.reduceOp = op
             tensor = fn(input)
@@ -429,7 +435,7 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
 
         # Multi input tests
         tests = simple_multi_input_reduce_tests(self.rank, self.world_size)
-        for (op, inputs, output) in tests:
+        for op, inputs, output in tests:
             opts = c10d.AllreduceOptions()
             opts.reduceOp = op
             tensors = [fn(input) for input in inputs]
@@ -458,56 +464,6 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
     def test_allreduce_basics_cuda(self):
         self._test_allreduce_basics(lambda t: t.clone().cuda())
 
-    # _using_work_api tests are to make sure we still properly support work API.
-    # This should go away as we deprecate it.
-    def _test_allreduce_basics_using_work_api(self, fn):
-        store = c10d.FileStore(self.file_name, self.world_size)
-        pg = self._create_process_group_gloo(
-            store, self.rank, self.world_size, self.opts()
-        )
-
-        # Single input tests
-        tests = simple_reduce_tests(self.rank, self.world_size)
-        for (op, input, expected) in tests:
-            opts = c10d.AllreduceOptions()
-            opts.reduceOp = op
-            tensor = fn(input)
-            work = pg.allreduce([tensor], opts)
-            work.wait()
-            result = work.result()
-            self.assertEqual(expected, result[0])
-
-        # Multi input tests
-        tests = simple_multi_input_reduce_tests(self.rank, self.world_size)
-        for (op, inputs, output) in tests:
-            opts = c10d.AllreduceOptions()
-            opts.reduceOp = op
-            tensors = [fn(input) for input in inputs]
-            work = pg.allreduce(tensors, opts)
-            work.wait()
-            result = work.result()
-            for tensor in result:
-                self.assertEqual(output, tensor)
-
-        # Test overloaded convenience function (defaults to using sum)
-        x = fn(torch.tensor([self.rank + 1.0]))
-        work = pg.allreduce(x)
-        work.wait()
-        result = work.result()
-        self.assertEqual(
-            torch.tensor([float(self.world_size * (self.world_size + 1) / 2)]),
-            result[0],
-        )
-
-    @requires_gloo()
-    def test_allreduce_basics_using_work_api(self):
-        self._test_allreduce_basics_using_work_api(lambda t: t.clone())
-
-    @skip_if_lt_x_gpu(2)
-    @requires_gloo()
-    def test_allreduce_basics_cuda_using_work_api(self):
-        self._test_allreduce_basics_using_work_api(lambda t: t.clone().cuda())
-
     def _test_allreduce_stress(self, inputs):
         store = c10d.FileStore(self.file_name, self.world_size)
         pg = self._create_process_group_gloo(
@@ -555,7 +511,9 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
             opts = c10d.AllreduceCoalescedOptions()
             pg.allreduce_coalesced([], opts)
 
-        with self.assertRaisesRegex(RuntimeError, "tensors must all have the same type"):
+        with self.assertRaisesRegex(
+            RuntimeError, "tensors must all have the same type"
+        ):
             opts = c10d.AllreduceCoalescedOptions()
             pg.allreduce_coalesced([t1, t2], opts)
 
@@ -670,7 +628,9 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
 
         # Sparse allreduce only works with c10d.ReduceOp.SUM.
         for op in [c10d.ReduceOp.PRODUCT, c10d.ReduceOp.MIN, c10d.ReduceOp.MAX]:
-            with self.assertRaisesRegex(RuntimeError, "unsupported reduction operation"):
+            with self.assertRaisesRegex(
+                RuntimeError, "unsupported reduction operation"
+            ):
                 opts = c10d.AllreduceOptions()
                 opts.reduceOp = op
                 pg.allreduce([t3], opts)
@@ -685,7 +645,7 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
             tests = simple_sparse_reduce_tests(
                 self.rank, self.world_size, num_inputs=num_inputs_per_rank
             )
-            for (inputs, outputs) in tests:
+            for inputs, outputs in tests:
                 tensors = [fn(input) for input in inputs]
                 fut = pg.allreduce(tensors).get_future()
                 fut.wait()
@@ -702,6 +662,93 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
     def test_sparse_allreduce_basics_cuda(self):
         self._test_sparse_allreduce_basics(lambda t: t.clone().cuda())
 
+    @skip_if_lt_x_gpu(2)
+    @requires_gloo()
+    def test_sparse_allreduce_cuda_dispatched(self):
+        store = c10d.FileStore(self.file_name, self.world_size)
+        dist.init_process_group(
+            backend="gloo", store=store, rank=self.rank, world_size=self.world_size
+        )
+        tests = simple_sparse_reduce_tests(self.rank, self.world_size, num_inputs=1)
+        for inputs, outputs in tests:
+            tensors = inputs[-1].clone().cuda()
+            work = dist.all_reduce(tensors, async_op=True)
+            work.wait()
+            self.assertEqual([tensors], outputs)
+
+    @requires_gloo()
+    def test_allgather_into_tensor_coalesced(self):
+        store = c10d.FileStore(self.file_name, self.world_size)
+        dist.init_process_group(
+            backend="gloo",
+            store=store,
+            rank=self.rank,
+            world_size=self.world_size,
+        )
+        torch.manual_seed(42)
+        in_shapes = [(5, 5), (10, 10), (15, 15)]
+        out_shapes = [(s[0] * self.world_size,) + s[1:] for s in in_shapes]
+
+        outputs = [torch.empty(s) for s in out_shapes]
+        inputs = [torch.rand(s) for s in in_shapes]
+        work = dist.group.WORLD.allgather_into_tensor_coalesced(outputs, inputs)
+        work.wait()
+
+        for output, input in zip(outputs, inputs):
+            expect = torch.cat([input] * self.world_size)
+            self.assertTrue(torch.allclose(output, expect))
+
+    @requires_gloo()
+    def test_reduce_scatter_tensor(self):
+        store = c10d.FileStore(self.file_name, self.world_size)
+        dist.init_process_group(
+            backend="gloo",
+            store=store,
+            rank=self.rank,
+            world_size=self.world_size,
+        )
+        torch.manual_seed(42)
+        out_shape = (20, 20)
+        in_shape = (out_shape[0] * self.world_size,) + out_shape[1:]
+
+        output = torch.empty(out_shape)
+        input = torch.rand(in_shape)
+        work = dist.reduce_scatter_tensor(output, input, async_op=True)
+        work.wait()
+
+        expect = (
+            input.view(self.world_size, *out_shape).chunk(self.world_size)[self.rank]
+            * self.world_size
+        )
+        self.assertTrue(torch.allclose(output, expect))
+
+    @requires_gloo()
+    def test_reduce_scatter_tensor_coalesced(self):
+        store = c10d.FileStore(self.file_name, self.world_size)
+        dist.init_process_group(
+            backend="gloo",
+            store=store,
+            rank=self.rank,
+            world_size=self.world_size,
+        )
+        torch.manual_seed(42)
+        out_shapes = [(5, 5), (10, 10), (15, 15)]
+        in_shapes = [(s[0] * self.world_size,) + s[1:] for s in out_shapes]
+
+        outputs = [torch.empty(s) for s in out_shapes]
+        inputs = [torch.rand(s) for s in in_shapes]
+        work = dist.group.WORLD.reduce_scatter_tensor_coalesced(outputs, inputs)
+        work.wait()
+
+        for output, input in zip(outputs, inputs):
+            expect = (
+                input.view(self.world_size, *output.shape).chunk(self.world_size)[
+                    self.rank
+                ]
+                * self.world_size
+            )
+            self.assertTrue(torch.allclose(output, expect))
+
     @requires_gloo()
     def test_scatter_checks(self):
         store = c10d.FileStore(self.file_name, self.world_size)
@@ -737,12 +784,16 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
             opts.rootRank = 0
             pg.scatter([t1, t1], [], opts)
 
-        with self.assertRaisesRegex(RuntimeError, "requires a single-element input list"):
+        with self.assertRaisesRegex(
+            RuntimeError, "requires a single-element input list"
+        ):
             opts = c10d.ScatterOptions()
             opts.rootRank = self.rank
             pg.scatter([t1], [], opts)
 
-        with self.assertRaisesRegex(RuntimeError, "requires a single-element input list"):
+        with self.assertRaisesRegex(
+            RuntimeError, "requires a single-element input list"
+        ):
             opts = c10d.ScatterOptions()
             opts.rootRank = self.rank
             pg.scatter([t1], [[t1] * self.world_size, [t1] * self.world_size], opts)
@@ -849,6 +900,17 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
                 msg=("Mismatch in iteration %d for rank %d" % (iter, root)),
             )
 
+    @requires_gloo()
+    def test_set_gloo_pg_timeout(self):
+        store = c10d.FileStore(self.file_name, self.world_size)
+        pg = self._create_process_group_gloo(
+            store, self.rank, self.world_size, self.opts()
+        )
+        pg.allreduce(torch.rand(10))
+        self.assertEqual(pg.options._timeout, timedelta(seconds=50))
+        pg._set_default_timeout(timedelta(seconds=23))
+        self.assertEqual(pg.options._timeout, timedelta(seconds=23))
+
     @requires_gloo()
     def test_scatter_stress(self):
         inputs = [
@@ -1051,7 +1113,9 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
         t2 = torch.zeros([1], dtype=torch.float64)
         t3 = torch.zeros([2], dtype=torch.float32)
 
-        with self.assertRaisesRegex(RuntimeError, "requires non-empty input tensor list"):
+        with self.assertRaisesRegex(
+            RuntimeError, "requires non-empty input tensor list"
+        ):
             pg.allgather([], [])
 
         with self.assertRaisesRegex(
@@ -1124,6 +1188,11 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
         # Take a column of 2D tensor, such that memory is not dense
         self._test_allgather_basics(lambda t: t.expand(2, 2).contiguous()[:, 0])
 
+    @requires_gloo()
+    def test_allgather_inference_mode(self):
+        with torch.inference_mode():
+            self._test_allgather_basics(lambda t: t.clone())
+
     def _test_allgather_stress(self, inputs, fn):
         store = c10d.FileStore(self.file_name, self.world_size)
         pg = self._create_process_group_gloo(
@@ -1203,7 +1272,7 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
         # Output is not a list of lists.
         dummy_output_lists = [torch.zeros([0], dtype=torch.float32)]
         with self.assertRaisesRegex(
-            RuntimeError, "Invalid function argument.*output_tensor_lists"
+            TypeError, "Invalid function argument.*output_tensor_lists"
         ):
             c10d.all_gather_coalesced(dummy_output_lists, dummy_input, pg)
 
@@ -1215,11 +1284,19 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
         )
 
         xxs = [2 * [torch.tensor([i + self.rank])] for i in range(2)]
-        yys = [[[torch.zeros_like(x) for x in xx] for _ in range(self.world_size)] for xx in xxs]
-        futs = [c10d.all_gather_coalesced(yy, xx, async_op=True) for xx, yy in zip(xxs, yys)]
+        yys = [
+            [[torch.zeros_like(x) for x in xx] for _ in range(self.world_size)]
+            for xx in xxs
+        ]
+        futs = [
+            c10d.all_gather_coalesced(yy, xx, async_op=True) for xx, yy in zip(xxs, yys)
+        ]
 
         # expected outputs
-        zzs = [[2 * [torch.tensor([i + r])] for r in range(self.world_size)] for i in range(2)]
+        zzs = [
+            [2 * [torch.tensor([i + r])] for r in range(self.world_size)]
+            for i in range(2)
+        ]
 
         torch.futures.wait_all(futs)
         for yy, zz in zip(yys, zzs):
@@ -1281,7 +1358,7 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
         pg = self._create_process_group_gloo(
             store, self.rank, self.world_size, self.opts()
         )
-        for (op, input, output) in simple_reduce_tests(self.rank, self.world_size):
+        for op, input, output in simple_reduce_tests(self.rank, self.world_size):
             for root in range(self.world_size):
                 opts = c10d.ReduceOptions()
                 opts.reduceOp = op
@@ -1408,49 +1485,6 @@ class ProcessGroupGlooTest(MultiProcessTestCase):
         for i, tensor in enumerate(tensors):
             self.assertEqual(torch.full(size, float(i * self.world_size)), tensor)
 
-    @skip_if_win32()
-    @requires_gloo()
-    def test_round_robin(self):
-        num_process_groups = 2
-        store = c10d.FileStore(self.file_name, self.world_size)
-        c10d.init_process_group(backend="gloo", store=store, rank=self.rank, world_size=self.world_size)
-        pg = c10d._round_robin_process_groups(
-            [
-                c10d.new_group(pg_options=self.opts())
-                for i in range(num_process_groups)
-            ]
-        )
-
-        # Run a few collectives so that we have called each process group
-        for _ in range(num_process_groups + 1):
-            tensor = torch.full([100, 100], float(self.rank))
-            pg.broadcast(tensor, root=0).wait()
-            self.assertEqual(torch.full([100, 100], 0.0), tensor)
-
-    @skip_if_win32()
-    @requires_gloo()
-    def test_round_robin_create_destroy(self):
-        store = c10d.FileStore(self.file_name, self.world_size)
-        c10d.init_process_group(backend="gloo", store=store, rank=self.rank, world_size=self.world_size)
-
-        def create(num, prefix):
-            return c10d._round_robin_process_groups(
-                [
-                    c10d.new_group(pg_options=self.opts())
-                    for i in range(num)
-                ]
-            )
-
-        # Run create/use/destroy twice
-        for i in range(2):
-            num_process_groups = 2
-            pg = create(num=num_process_groups, prefix=i)
-            for _ in range(3):
-                tensor = torch.ones([10, 10])
-                pg.allreduce(tensor).wait()
-                self.assertEqual(torch.full([10, 10], float(self.world_size)), tensor)
-            del pg
-
 
 class DistributedDataParallelTest(
     test_c10d_common.CommonDistributedDataParallelTest, MultiProcessTestCase
@@ -1461,14 +1495,18 @@ class DistributedDataParallelTest(
 
     def _get_process_group(self):
         store = self._get_store()
-        c10d.init_process_group(backend="gloo", store=store, rank=self.rank, world_size=self.world_size)
+        c10d.init_process_group(
+            backend="gloo", store=store, rank=self.rank, world_size=self.world_size
+        )
         return c10d.distributed_c10d._get_default_group()
 
     def _test_gloo_backend(
         self, devices, device_ids, multi_device=False, gradient_as_bucket_view=False
     ):
         store = c10d.FileStore(self.file_name, self.world_size)
-        c10d.init_process_group(backend="gloo", store=store, rank=self.rank, world_size=self.world_size)
+        c10d.init_process_group(
+            backend="gloo", store=store, rank=self.rank, world_size=self.world_size
+        )
         process_group = c10d.distributed_c10d._get_default_group()
         device = devices[-1]
         backend = process_group._get_backend(device)
@@ -1525,7 +1563,7 @@ class DistributedDataParallelTest(
         """
 
         class GlobalLocalUnusedParamModule(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.t0 = Task()
                 self.t1 = Task()
@@ -1607,7 +1645,7 @@ class DistributedDataParallelTest(
         """
 
         class FindUnusedParamModule(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.t0 = Task()
                 self.t1 = Task()
@@ -1660,7 +1698,7 @@ class DistributedDataParallelTest(
         process_group = self._get_process_group()
 
         class IgnoredOutput(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.fc1 = nn.Linear(2, 10, bias=False)
                 self.fc2 = nn.Linear(10, 4, bias=False)
@@ -1702,7 +1740,7 @@ class DistributedDataParallelTest(
         process_group = self._get_process_group()
 
         class IgnoredOutputWithUnusedParameters(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.fc1 = nn.Linear(2, 10, bias=False)
                 self.fc2 = nn.Linear(10, 4, bias=False)
@@ -1749,6 +1787,7 @@ class DistributedDataParallelTest(
             def forward(self, x):
                 x = self.relu(self.fc1(x))
                 return F.softmax(x, dim=1)
+
         pg = dist.init_process_group(
             "gloo",
             init_method=f"file://{self.file_name}",
@@ -1759,14 +1798,13 @@ class DistributedDataParallelTest(
         local_shard_metadata = ShardMetadata(
             shard_offsets=[(self.rank % 2) * 5, 0],
             shard_sizes=[5, 10],
-            placement=f"rank:{self.rank}/cuda:{self.rank}"
+            placement=f"rank:{self.rank}/cuda:{self.rank}",
         )
         local_shards = [Shard(torch.randn(5, 10, device=device), local_shard_metadata)]
         st = init_from_local_shards(local_shards, [10, 10])
         m = MyModule(st)
         DistributedDataParallel._set_params_and_buffers_to_ignore_for_model(
-            module=m,
-            params_and_buffers_to_ignore={'st'}
+            module=m, params_and_buffers_to_ignore={"st"}
         )
         # test to make DDP constructor will not fail when module includes a ShardedTensor when ignored
         DistributedDataParallel(
@@ -1796,7 +1834,9 @@ class DistributedDataParallelTest(
         # Check that the gradients are sparse and identical
         vanilla_parameter = next(vanilla_model.parameters())
         ddp_parameter = next(ddp_model.parameters())
-        self.assertEqual(vanilla_parameter.grad.coalesce(), ddp_parameter.grad.coalesce())
+        self.assertEqual(
+            vanilla_parameter.grad.coalesce(), ddp_parameter.grad.coalesce()
+        )
 
     @requires_gloo()
     @skip_if_lt_x_gpu(2)
@@ -1809,7 +1849,7 @@ class DistributedDataParallelTest(
         )
 
         class TestModel(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.fc1 = nn.Linear(2, 10, bias=False)
                 self.fc2 = nn.Linear(10, 4, bias=False)
@@ -2021,7 +2061,9 @@ class DistributedDataParallelTest(
             ModuleForDdpCommHook(), process_group=process_group
         )
 
-        expected_err = "Communication hook: return annotation should be torch.futures.Future"
+        expected_err = (
+            "Communication hook: return annotation should be torch.futures.Future"
+        )
         with self.assertRaisesRegex(
             ValueError,
             expected_err,
@@ -2109,7 +2151,7 @@ class DistributedDataParallelTest(
 
 
 class ReducerModule(nn.Module):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         self.fc1 = nn.Linear(2, 10, bias=False)
         self.fc2 = nn.Linear(10, 4, bias=False)
@@ -2129,7 +2171,9 @@ class ReducerTest(TestCase):
         self.file = tempfile.NamedTemporaryFile(delete=False)
         world_size = 1
         self.store = c10d.FileStore(self.file.name, world_size)
-        c10d.init_process_group(backend="gloo", store=self.store, rank=0, world_size=world_size)
+        c10d.init_process_group(
+            backend="gloo", store=self.store, rank=0, world_size=world_size
+        )
         self.process_group = c10d.distributed_c10d._get_default_group()
 
     def tearDown(self):
@@ -2138,14 +2182,15 @@ class ReducerTest(TestCase):
             os.remove(self.file.name)
         except OSError as e:
             print(str(e))
-            pass
 
     @requires_gloo()
     def test_single_dtype_single_bucket(self):
         model = ReducerModule()
         parameters = list(model.parameters())
         buckets = [list(range(len(parameters)))]
-        dist.Reducer(parameters, buckets, [dist._DEFAULT_FIRST_BUCKET_BYTES], self.process_group)
+        dist.Reducer(
+            parameters, buckets, [dist._DEFAULT_FIRST_BUCKET_BYTES], self.process_group
+        )
 
     def _create_mixed_precision_model(self):
         model = ReducerModule()
@@ -2166,7 +2211,7 @@ class ReducerTest(TestCase):
                 parameters,
                 buckets,
                 [dist._DEFAULT_FIRST_BUCKET_BYTES],
-                self.process_group
+                self.process_group,
             )
 
     @requires_gloo()
@@ -2181,7 +2226,7 @@ class ReducerTest(TestCase):
             parameters,
             buckets,
             [dist._DEFAULT_FIRST_BUCKET_BYTES for _ in buckets],
-            self.process_group
+            self.process_group,
         )
 
     def _create_reducer_for_models(self, models, find_unused_parameters=False):
@@ -2264,7 +2309,6 @@ class CommTest(test_c10d_common.AbstractCommTest, MultiProcessTestCase):
     def device(self):
         return "cpu"
 
-
     def setUp(self):
         super().setUp()
         self._spawn_processes()
@@ -2311,7 +2355,9 @@ class CommTest(test_c10d_common.AbstractCommTest, MultiProcessTestCase):
     @skip_if_lt_x_gpu(2)
     def test_broadcast_coalesced_gloo_cuda(self):
         store = c10d.FileStore(self.file_name, self.world_size)
-        c10d.init_process_group(backend="gloo", store=store, rank=self.rank, world_size=self.world_size)
+        c10d.init_process_group(
+            backend="gloo", store=store, rank=self.rank, world_size=self.world_size
+        )
         process_group = c10d.distributed_c10d._get_default_group()
         device = torch.device("cuda:%d" % self.rank)
         backend = process_group._get_backend(device)
@@ -2323,7 +2369,9 @@ class CommTest(test_c10d_common.AbstractCommTest, MultiProcessTestCase):
     @requires_gloo()
     def test_broadcast_coalesced_gloo_cpu(self):
         store = c10d.FileStore(self.file_name, self.world_size)
-        c10d.init_process_group(backend="gloo", store=store, rank=self.rank, world_size=self.world_size)
+        c10d.init_process_group(
+            backend="gloo", store=store, rank=self.rank, world_size=self.world_size
+        )
         process_group = c10d.distributed_c10d._get_default_group()
         device = torch.device("cpu")
         backend = process_group._get_backend(device)
@@ -2378,7 +2426,10 @@ class CommTest(test_c10d_common.AbstractCommTest, MultiProcessTestCase):
     def test_bool_tensors(self):
         self._test_bool_tensors(backend="gloo")
 
-class GlooProcessGroupWithDispatchedCollectivesTests(test_c10d_common.ProcessGroupWithDispatchedCollectivesTests):
+
+class GlooProcessGroupWithDispatchedCollectivesTests(
+    test_c10d_common.ProcessGroupWithDispatchedCollectivesTests
+):
     @requires_gloo()
     def test_collectives(self):
         self._test_collectives(backend="gloo")
@@ -2416,69 +2467,6 @@ class GlooProcessGroupWithDispatchedCollectivesTests(test_c10d_common.ProcessGro
         )
         dist.monitored_barrier()
 
-class CompilerTest(test_c10d_common.CompilerTest):
-
-    @property
-    def world_size(self):
-        return 2
-
-    def _get_default_group(self):
-        store = c10d.FileStore(self.file_name, self.world_size)
-        dist.init_process_group(
-            backend="gloo",
-            rank=self.rank,
-            world_size=self.world_size,
-            store=store,
-        )
-        return dist.distributed_c10d._get_default_group()
-
-    def test_allreduce_work_wait_cpu(self):
-        self._test_allreduce_work_wait(torch.ones(2, 2) * self.rank)
-
-    @skip_if_lt_x_gpu(2)
-    def test_allreduce_work_wait_gpu(self):
-        self._test_allreduce_work_wait(
-            torch.ones(2, 2, device=self.rank) * self.rank
-        )
-
-    def test_allgather_work_wait_cpu(self):
-        self._test_allgather_work_wait(torch.ones(2, 2) * self.rank)
-
-    @skip_if_lt_x_gpu(2)
-    def test_allgather_work_wait_gpu(self):
-        self._test_allgather_work_wait(
-            torch.ones(2, 2, device=self.rank) * self.rank
-        )
-
-    def test_broadcast_work_wait_cpu(self):
-        self._test_broadcast_work_wait(torch.ones(2, 2) * self.rank)
-
-    @skip_if_lt_x_gpu(2)
-    def test_broadcast_work_wait_gpu(self):
-        self._test_broadcast_work_wait(
-            torch.ones(2, 2, device=self.rank) * self.rank
-        )
-
-    def test_scatter_work_wait_cpu(self):
-        self._test_scatter_work_wait(torch.ones(2, 2) * self.rank)
-
-    @skip_if_lt_x_gpu(2)
-    def test_scatter_work_wait_gpu(self):
-        self._test_scatter_work_wait(
-            torch.ones(2, 2, device=self.rank) * self.rank
-        )
-
-    def test_nested_comm_tensor_wrapping(self):
-        self._test_nested_comm_tensor_wrapping(torch.ones(2, 2) * self.rank)
-
-    def test_consecutive_comm_work_wait_cpu(self):
-        self._test_consecutive_comm_work_wait(torch.ones(2, 2) * self.rank)
-
-    @skip_if_lt_x_gpu(2)
-    def test_consecutive_comm_work_wait_gpu(self):
-        self._test_consecutive_comm_work_wait(
-            torch.ones(2, 2, device=self.rank) * self.rank
-        )
 
 class LargeCommTest(test_c10d_common.AbstractLargeCommTest, MultiProcessTestCase):
     def setUp(self):
@@ -2508,6 +2496,7 @@ class LargeCommTest(test_c10d_common.AbstractLargeCommTest, MultiProcessTestCase
     def test_new_group_local_sync_duplicate_pg(self):
         self._test_new_group_local_sync_duplicate_pg(backend="gloo")
 
+
 if __name__ == "__main__":
     assert (
         not torch.cuda._initialized
