@@ -3,9 +3,9 @@ from torch._inductor.codegen.common import DeviceOpOverrides
 from torch._inductor.scheduler import BaseScheduling
 
 
-class ExtensionWrapperCodegen(wrapper.WrapperCodeGen):
-    def __init__(self):
-        super().__init__()
+class ExtensionWrapperCodegen(wrapper.PythonWrapperCodegen):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
 
 
 class ExtensionScheduling(BaseScheduling):
