@@ -1,21 +1,27 @@
 # Owner(s): ["module: inductor"]
 import copy
-
 import functools
 import os
 import unittest
+from typing import Tuple
 
 import torch
 from torch import nn, Tensor
 from torch._dynamo.convert_frame import maybe_cprofile
+from torch._dynamo.device_interface import get_interface_for_device
 from torch._dynamo.test_case import run_tests, TestCase
 from torch._dynamo.testing import rand_strided, reduce_to_scalar_loss
 from torch._inductor import config, ir, metrics
 from torch._inductor.fx_passes import pad_mm as pad_mm_pass
-from torch._inductor.runtime.runtime_utils import do_bench
-from torch._inductor.utils import run_and_get_code
-from torch.testing._internal.common_utils import serialTest
-from torch.testing._internal.inductor_utils import HAS_CUDA
+from torch._inductor.runtime.benchmarking import benchmarker
+from torch._inductor.utils import ceildiv, run_and_get_code
+from torch.testing._internal.common_utils import (
+    instantiate_parametrized_tests,
+    parametrize,
+    serialTest,
+)
+from torch.testing._internal.inductor_utils import GPU_TYPE, HAS_GPU, requires_gpu
+
 
 DO_PERF_TEST = os.environ.get("DO_PERF_TEST") == "1"
 DO_ACC_TEST = os.environ.get("DO_ACC_TEST", "1") == "1"
@@ -85,7 +91,25 @@ def forward_and_backward_pass(m, inputs):
         "triton.cudagraphs": USE_CUDA_GRAPHS,
     }
 )
+@requires_gpu()
 class TestCaseBase(TestCase):
+    @classmethod
+    def setUpClass(cls):
+        if HAS_GPU:
+            cls.prior_float32_matmul_precision = torch.get_float32_matmul_precision()
+            cls.prior_default_device = torch.get_default_device()
+            torch.set_float32_matmul_precision("high")
+            torch.set_default_device(GPU_TYPE)
+
+    @classmethod
+    def tearDownClass(cls):
+        if HAS_GPU:
+            torch.set_float32_matmul_precision(cls.prior_float32_matmul_precision)
+            torch.set_default_device(cls.prior_default_device)
+
+            cls.prior_float32_matmul_precision = None
+            cls.prior_default_device = None
+
     def check_close(self, ref, act, tol=1e-3):
         if type(ref).__name__ == "LongformerMaskedLMOutput":
             ref = ref.loss
@@ -117,7 +141,8 @@ class TestCaseBase(TestCase):
     ):
         if kwargs is None:
             kwargs = {}
-        torch.cuda.synchronize()
+        device_interface = get_interface_for_device(GPU_TYPE)
+        device_interface.synchronize()
         with torch.profiler.profile(with_stack=WITH_STACK) as p:
             niter = 3
             for _ in range(niter):
@@ -126,7 +151,7 @@ class TestCaseBase(TestCase):
 
                 with torch.profiler.record_function(tag_rhs):
                     f_rhs(*args, **kwargs)
-            torch.cuda.synchronize()
+            device_interface.synchronize()
 
         profile_path = "/tmp/chrome.json"
         p.export_chrome_trace(profile_path)
@@ -151,10 +176,10 @@ class PerfTestBetweenGoodAndBadShape(TestCaseBase):
         m_bad_shape_opt = torch.compile(m_bad_shape)
         m_good_shape_opt = torch.compile(m_good_shape)
 
-        latency_good_shape = do_bench(
+        latency_good_shape = benchmarker.benchmark_gpu(
             lambda: forward_and_backward_pass(m_good_shape_opt, inputs_good_shape)
         )
-        latency_bad_shape = do_bench(
+        latency_bad_shape = benchmarker.benchmark_gpu(
             lambda: forward_and_backward_pass(m_bad_shape_opt, inptus_bad_shape)
         )
         print(
@@ -183,7 +208,7 @@ class PerfTestBetweenGoodAndBadShape(TestCaseBase):
 
             def f(**inputs):
                 optim.zero_grad(True)
-                with torch.cuda.amp.autocast():
+                with torch.autocast(GPU_TYPE):
                     pred = model(**inputs)
                     loss = pred[0]
                 loss.backward()
@@ -195,9 +220,13 @@ class PerfTestBetweenGoodAndBadShape(TestCaseBase):
         f_bad_shape, inputs_bad_shape = create_model(30522)
 
         print("benchmark for good shape")
-        latency_good_shape = do_bench(lambda: f_good_shape(**inputs_good_shape))
+        latency_good_shape = benchmarker.benchmark_gpu(
+            lambda: f_good_shape(**inputs_good_shape)
+        )
         print("benchmark for bad shape")
-        latency_bad_shape = do_bench(lambda: f_bad_shape(**inputs_bad_shape))
+        latency_bad_shape = benchmarker.benchmark_gpu(
+            lambda: f_bad_shape(**inputs_bad_shape)
+        )
         print(
             f"Latency with good and bad shape: {latency_good_shape:.3f} v.s. {latency_bad_shape:.3f}"
         )
@@ -251,7 +280,7 @@ class PerfTestWithAndWithoutPadding(TestCaseBase):
             def get_f(m, optim):
                 def f(*args, **kwargs):
                     optim.zero_grad(True)
-                    with torch.cuda.amp.autocast():
+                    with torch.autocast(GPU_TYPE):
                         pred = m(*args, **kwargs)
                         loss = reduce_to_scalar_loss(pred)
                     loss.backward()
@@ -267,7 +296,7 @@ class PerfTestWithAndWithoutPadding(TestCaseBase):
                 opt_f_with_padding = torch.compile(
                     get_f(m_copy_with_padding, optim_with_padding)
                 )
-                latency_with_padding = do_bench(
+                latency_with_padding = benchmarker.benchmark_gpu(
                     lambda: opt_f_with_padding(*perf_args, **perf_kwargs)
                 )
             latency_without_padding = None
@@ -278,7 +307,7 @@ class PerfTestWithAndWithoutPadding(TestCaseBase):
                 opt_f_without_padding = torch.compile(
                     get_f(m_copy_without_padding, optim_without_padding)
                 )
-                latency_without_padding = do_bench(
+                latency_without_padding = benchmarker.benchmark_gpu(
                     lambda: opt_f_without_padding(*perf_args, **perf_kwargs)
                 )
             print(
@@ -301,7 +330,7 @@ class PerfTestWithAndWithoutPadding(TestCaseBase):
         x = torch.randn(4, layer_sizes[0])
 
         class Model(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 mod_list = []
                 for i in range(len(layer_sizes) - 1):
@@ -340,6 +369,7 @@ class PerfTestWithAndWithoutPadding(TestCaseBase):
         self.test_longformer(bs=2)
 
 
+@instantiate_parametrized_tests
 class PaddingTest(TestCaseBase):
     @unittest.skipIf(not DO_PERF_TEST, "Perf test not enabled")
     def test_mm_padding_perf(self):
@@ -369,7 +399,7 @@ class PaddingTest(TestCaseBase):
         ):
             a = torch.randn(M, K)
             b = torch.randn(K, N)
-            ms = do_bench(lambda: f(a, b))
+            ms = benchmarker.benchmark_gpu(lambda: f(a, b))
             print(f"MxKxN {M}x{K}x{N} {f.__name__}: {ms:.3f}ms")
 
     @unittest.skipIf(not DO_PERF_TEST, "Perf test not enabled")
@@ -395,8 +425,8 @@ class PaddingTest(TestCaseBase):
             mat2 = pad_dim(mat2, 6, 0)
             return torch.ops.aten.mm(mat1, mat2)
 
-        ori_time = do_bench(f)
-        pad_time = do_bench(g)
+        ori_time = benchmarker.benchmark_gpu(f)
+        pad_time = benchmarker.benchmark_gpu(g)
 
         print(
             f"Latency between origional matmul and padded matmul: {ori_time:.3f} v.s. {pad_time:.3f}"
@@ -414,7 +444,7 @@ class PaddingTest(TestCaseBase):
 
         # Using stride (30522, 1) does not make a difference here.
         x_bad_shape = rand_strided(
-            (8192, 30522), (30528, 1), device="cuda", dtype=torch.float16
+            (8192, 30522), (30528, 1), device=GPU_TYPE, dtype=torch.float16
         )
         weight_bad_shape = torch.randn(30522, 768, dtype=torch.float16)
         out_bad_shape = torch.randn(8192, 768, dtype=torch.float16)
@@ -429,8 +459,8 @@ class PaddingTest(TestCaseBase):
         f2 = torch.compile(
             functools.partial(f, x_bad_shape, weight_bad_shape, out_bad_shape)
         )
-        latency_good_shape = do_bench(f1)
-        latency_bad_shape = do_bench(f2)
+        latency_good_shape = benchmarker.benchmark_gpu(f1)
+        latency_bad_shape = benchmarker.benchmark_gpu(f2)
         print(
             f"Latency with good and bad shapes: {latency_good_shape:.3f} v.s. {latency_bad_shape:.3f}"
         )
@@ -449,22 +479,20 @@ class PaddingTest(TestCaseBase):
             forward_and_backward_pass, m_bad_shape_opt, inputs_bad_shape
         )
         forward_and_backward_pass(m_bad_shape, inputs_bad_shape)
-        self.assertTrue(
-            torch.allclose(
-                m_bad_shape.linear.weight.grad, m_bad_shape_opt.linear.weight.grad
-            )
+        self.assertEqual(
+            m_bad_shape.linear.weight.grad, m_bad_shape_opt.linear.weight.grad
         )
         self.assertTrue(len(wrapper_codes) == 2)  # one for forward and oen for backward
         forward_wrapper = wrapper_codes[0]
 
         # make sure the load for softmax is aligned
         self.assertTrue(
-            "tl.load(in_ptr0 + (r1 + (30528*x0))" in forward_wrapper,
+            "tl.load(in_ptr0 + (r1 + 30528*x0)" in forward_wrapper,
             f"forward_wrapper: {forward_wrapper}",
         )
 
         if DO_PERF_TEST:
-            latency = do_bench(
+            latency = benchmarker.benchmark_gpu(
                 lambda: forward_and_backward_pass(m_bad_shape_opt, inputs_bad_shape)
             )
             print(f"latency: {latency:.3f}ms")
@@ -475,7 +503,7 @@ class PaddingTest(TestCaseBase):
         inv_scale = (num_heads / hidden_size) ** 0.5
 
         class Attention(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.query = nn.Linear(hidden_size, hidden_size)
                 self.key = nn.Linear(hidden_size, hidden_size)
@@ -565,7 +593,7 @@ class PaddingTest(TestCaseBase):
         x1 = torch.randn(*x_shape)
 
         padded_stride = ir.Layout._pad_strides(x1.stride(), x1.shape, torch.float32)
-        x2 = rand_strided(x_shape, padded_stride, device="cuda")
+        x2 = rand_strided(x_shape, padded_stride, device=GPU_TYPE)
         x2.copy_(x1)
 
         weight = torch.randn(64, 128, 3, 3)
@@ -587,8 +615,8 @@ class PaddingTest(TestCaseBase):
         act = fun(x2, weight)
         self.check_close(ref, act)
         if DO_PERF_TEST:
-            latency_with_padding = do_bench(lambda: fun(x2, weight))
-            latency_without_padding = do_bench(lambda: fun(x1, weight))
+            latency_with_padding = benchmarker.benchmark_gpu(lambda: fun(x2, weight))
+            latency_without_padding = benchmarker.benchmark_gpu(lambda: fun(x1, weight))
             print(
                 f"Latency with and without padding: {latency_with_padding:.3f} v.s. {latency_without_padding:.3f}"
             )
@@ -616,8 +644,8 @@ class PaddingTest(TestCaseBase):
         with config.patch("triton.cudagraphs", False):
             opt_f = torch.compile(f)
             opt_f(x)
-        eager_time = do_bench(lambda: f(x))
-        opt_time = do_bench(lambda: opt_f(x))
+        eager_time = benchmarker.benchmark_gpu(lambda: f(x))
+        opt_time = benchmarker.benchmark_gpu(lambda: opt_f(x))
         print(
             f"Latency between eager and compiled: {eager_time:.3f} v.s. {opt_time:.3f}"
         )
@@ -634,9 +662,54 @@ class PaddingTest(TestCaseBase):
         out_strides = ir.Layout._pad_strides(in_strides, t.shape, torch.float32)
         self.assertTrue(in_strides == out_strides)
 
+    @parametrize("alignment_bytes", (32, 128))
+    @parametrize("shape", [(21, 19), (3, 5, 71)])
+    @parametrize("dtype", (torch.float16, torch.float32))
+    def test_pad_outputs(
+        self, dtype: torch.dtype, shape: Tuple[int], alignment_bytes: int
+    ):
+        """
+        Tests padding output tensors to a specific alignment.
+        This is enabled by a config flag.
+        """
+        func = torch.add
+        inputs = tuple(torch.randn(*shape, dtype=dtype) for input_idx in range(2))
+
+        # Compile and run
+        with config.patch(
+            {
+                "comprehensive_padding": True,
+                "padding_alignment_bytes": alignment_bytes,
+                "padding_stride_threshold": 0,
+                "pad_outputs": True,
+            }
+        ):
+            compiled_func = torch.compile(func)
+            compiled_out = compiled_func(*inputs)
+
+        # Check numerics
+        eager_out = func(*inputs)
+        self.check_close(eager_out, compiled_out)
+
+        # Compute the expected padding
+        element_size = torch.tensor([], dtype=dtype).element_size()
+        self.assertGreater(alignment_bytes, element_size)
+        self.assertEqual(alignment_bytes % element_size, 0)
+        alignment_elements = alignment_bytes // element_size
+        contiguous_stride = inputs[0].stride()
+        expected_stride = [1]
+        for dim in reversed(shape[1:]):
+            slice_size = dim * expected_stride[0]
+            new_stride = alignment_elements * ceildiv(slice_size, alignment_elements)
+            expected_stride.insert(0, new_stride)
+        expected_stride = tuple(expected_stride)
+        self.assertNotEqual(expected_stride, contiguous_stride)
+
+        # Check strides
+        self.assertFalse(compiled_out.is_contiguous())
+        self.assertEqual(compiled_out.stride(), expected_stride)
+
 
 if __name__ == "__main__":
-    if HAS_CUDA:
-        torch.set_float32_matmul_precision("high")
-        torch.set_default_device("cuda")
+    if HAS_GPU:
         run_tests()
