@@ -3,13 +3,13 @@
 import os
 import sys
 from collections import OrderedDict
-
 from typing import Any, List, Tuple
 
 import torch
 import torch.nn as nn
 from torch.testing._internal.jit_utils import JitTestCase
 
+
 # Make the helper files in test/ importable
 pytorch_test_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 sys.path.append(pytorch_test_dir)
@@ -33,7 +33,7 @@ class TestModuleContainers(JitTestCase):
                 return {"1": x}
 
         class C(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.foo = torch.nn.Sequential(A(), B())
 
@@ -56,7 +56,7 @@ class TestModuleContainers(JitTestCase):
                 return (x - 4) * 3
 
         class M(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 modules = OrderedDict(
                     [
@@ -132,7 +132,7 @@ class TestModuleContainers(JitTestCase):
                 return x + 10
 
         class CustomSequential(nn.Sequential):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__(nn.ReLU(), Inner())
 
             def forward(self, x):
@@ -144,7 +144,7 @@ class TestModuleContainers(JitTestCase):
         self.checkModule(CustomSequential(), (torch.tensor(0.5),))
 
         class CustomModuleList(nn.ModuleList):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__([nn.ReLU(), Inner()])
 
             def forward(self, x):
@@ -156,7 +156,7 @@ class TestModuleContainers(JitTestCase):
         self.checkModule(CustomModuleList(), (torch.tensor(0.5),))
 
         class CustomModuleDict(nn.ModuleDict):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__(
                     OrderedDict(
                         [
@@ -203,7 +203,7 @@ class TestModuleContainers(JitTestCase):
                 return thing - self.i
 
         class M(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.mods = nn.ModuleList([Sub(i) for i in range(10)])
 
@@ -217,7 +217,7 @@ class TestModuleContainers(JitTestCase):
         self.checkModule(M(), (x,))
 
         class MForward(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.mods = nn.ModuleList([Sub(i) for i in range(10)])
 
@@ -276,7 +276,7 @@ class TestModuleContainers(JitTestCase):
                 torch.nn.ModuleDict.__init__(self, modules)
 
         class MyModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 # work around aliasing issue for 'is' operator by scripting ReLU up front
                 self.submod = torch.jit.script(torch.nn.ReLU())
@@ -343,7 +343,7 @@ class TestModuleContainers(JitTestCase):
                 return 2
 
         class MyModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 # work around aliasing issue for 'is' operator by scripting ReLU up front
                 self.submod = torch.jit.script(torch.nn.ReLU())
@@ -359,7 +359,7 @@ class TestModuleContainers(JitTestCase):
 
     def test_moduledict_getitem(self):
         class MyModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.relu = torch.jit.script(torch.nn.ReLU())
                 self.tanh = torch.jit.script(torch.nn.Tanh())
@@ -377,7 +377,7 @@ class TestModuleContainers(JitTestCase):
 
     def test_moduledict_keyerror(self):
         class BadModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.moduledict = torch.nn.ModuleDict({"foo": None, "bar": None})
 
@@ -391,7 +391,7 @@ class TestModuleContainers(JitTestCase):
             torch.jit.script(b)
 
         class AnotherBadModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.moduledict = torch.nn.ModuleDict({"foo": None, "bar": None})
 
@@ -417,7 +417,7 @@ class TestModuleContainers(JitTestCase):
         """
 
         class Mod(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.a = [torch.nn.ReLU(), torch.nn.ReLU()]
 
@@ -438,7 +438,7 @@ class TestModuleContainers(JitTestCase):
                 torch.nn.ModuleDict.__init__(self, modules)
 
         class MyModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 # work around aliasing issue for 'is' operator by scripting ReLU up front
                 self.submod = torch.jit.script(torch.nn.ReLU())
@@ -477,7 +477,7 @@ class TestModuleContainers(JitTestCase):
 
         # Test annotation of submodule.
         class Mod(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.d = torch.nn.ModuleDict({"module": ImplementsInterface()})
 
@@ -490,7 +490,7 @@ class TestModuleContainers(JitTestCase):
 
         # Test annotation of self.
         class ModDict(torch.nn.ModuleDict):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__({"module": ImplementsInterface()})
 
             def forward(self, x: torch.Tensor, key: str) -> Any:
@@ -503,7 +503,7 @@ class TestModuleContainers(JitTestCase):
         # Test error message thrown when annotated attribute does not comply with the
         # annotation.
         class ModWithWrongAnnotation(torch.nn.ModuleDict):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.d = torch.nn.ModuleDict({"module": DoesNotImplementInterface()})
 
@@ -540,7 +540,7 @@ class TestModuleContainers(JitTestCase):
 
         # Test annotation of submodule.
         class Mod(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.l = torch.nn.ModuleList([ImplementsInterface()])
 
@@ -553,7 +553,7 @@ class TestModuleContainers(JitTestCase):
 
         # Test annotation of self.
         class ModList(torch.nn.ModuleList):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__([ImplementsInterface()])
 
             def forward(self, x: torch.Tensor, idx: int) -> Any:
@@ -566,7 +566,7 @@ class TestModuleContainers(JitTestCase):
         # Test error message thrown when annotated attribute does not comply with the
         # annotation.
         class ModWithWrongAnnotation(torch.nn.ModuleList):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.l = torch.nn.ModuleList([DoesNotImplementInterface()])
 
@@ -681,7 +681,7 @@ class TestModuleContainers(JitTestCase):
                 return self.linear(self.linear(x))
 
         class N(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.linear = nn.Linear(4, 4)
 
@@ -710,7 +710,7 @@ class TestModuleContainers(JitTestCase):
 
     def test_parameterlist_script_getitem(self):
         class MyModule(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.module_list = nn.ModuleList([nn.Linear(1, 1) for _ in range(10)])
                 self.parameter_list = nn.ParameterList(
@@ -726,7 +726,7 @@ class TestModuleContainers(JitTestCase):
 
     def test_parameterlist_script_iter(self):
         class MyModule(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.module_list = nn.ModuleList([nn.Linear(1, 1) for _ in range(10)])
                 self.parameter_list = nn.ParameterList(
@@ -743,7 +743,7 @@ class TestModuleContainers(JitTestCase):
 
     def test_parameterdict_script_getitem(self):
         class MyModule(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.parameter_dict = nn.ParameterDict(
                     {k: nn.Parameter(torch.zeros(1)) for k in ["a", "b", "c"]}
