@@ -2,9 +2,11 @@ import sys
 
 import torch
 
+
 if __name__ == "__main__":
     script_mod = torch.jit.load(sys.argv[1])
-    mod = torch.load(sys.argv[1] + ".orig")
+    # weights_only=False as this is loading a sharded model
+    mod = torch.load(sys.argv[1] + ".orig", weights_only=False)
     print(script_mod)
     inp = torch.rand(2, 28 * 28)
     _ = mod(inp)
