@@ -81,7 +81,7 @@ class TestQuantizeEagerOps(QuantizationTestCase):
                                     extra_module_kwargs,
                                     input_size):
         class M(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.conv = float_module_class(**extra_module_kwargs)
                 self.quant = QuantStub()
@@ -94,7 +94,7 @@ class TestQuantizeEagerOps(QuantizationTestCase):
                 return x
 
         class RefM(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.conv = float_module_class(**extra_module_kwargs)
                 self.quant1 = QuantStub()
@@ -203,7 +203,7 @@ class TestQuantizeEagerOps(QuantizationTestCase):
     def test_int16_reference_module(self):
 
         class RefM(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.conv = nn.ConvTranspose2d(1, 1, 1)
                 self.quant1 = QuantStub()
@@ -277,7 +277,7 @@ class TestQuantizeEagerOps(QuantizationTestCase):
             extra_module_kwargs: keyword args to instantiate the float module
         """
         class M(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.activation_op = float_module_class(**extra_module_kwargs)
                 self.quant = QuantStub()
@@ -327,6 +327,7 @@ class TestQuantizeEagerOps(QuantizationTestCase):
             self.assertEqual(type(model.myadd), torch.ao.nn.quantized.QFunctional)
             self.assertEqual(type(model.mycat), torch.ao.nn.quantized.QFunctional)
             self.assertEqual(type(model.myadd_relu), torch.ao.nn.quantized.QFunctional)
+            self.assertEqual(type(model.mymatmul), torch.ao.nn.quantized.QFunctional)
             self.checkNoQconfig(model)
 
         checkQuantized(model)
@@ -838,7 +839,7 @@ class TestQuantizeEagerPTQStatic(QuantizationTestCase):
             self.checkScriptable(quantized_model, [[indices, offsets, per_sample_weights]], check_save_load=True)
 
             class EmbeddingBagWithLinear(torch.nn.Module):
-                def __init__(self):
+                def __init__(self) -> None:
                     super().__init__()
                     self.emb = torch.nn.EmbeddingBag(num_embeddings=10, embedding_dim=12,
                                                      include_last_offset=True, scale_grad_by_freq=False, mode='sum')
@@ -860,7 +861,7 @@ class TestQuantizeEagerPTQStatic(QuantizationTestCase):
     @skipIfNoFBGEMM
     def test_custom_module_class(self):
         class CustomModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.conv = torch.nn.Conv2d(1, 1, 1)
 
@@ -900,7 +901,7 @@ class TestQuantizeEagerPTQStatic(QuantizationTestCase):
                 return quantized
 
         class Sub(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.custom = CustomModule()
 
@@ -908,7 +909,7 @@ class TestQuantizeEagerPTQStatic(QuantizationTestCase):
                 return self.custom(x)
 
         class M(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.quant = QuantStub()
                 self.conv = torch.nn.Conv2d(1, 1, 1)
@@ -923,7 +924,7 @@ class TestQuantizeEagerPTQStatic(QuantizationTestCase):
                 return x
 
         class RefM(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.quant = QuantStub()
                 self.conv1 = torch.nn.Conv2d(1, 1, 1)
@@ -1030,7 +1031,7 @@ class TestQuantizeEagerPTQStatic(QuantizationTestCase):
         `non_leaf_module_list`.
         """
         class MyModel(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.quant = QuantStub()
                 self.sigmoid = torch.nn.Sigmoid()
@@ -1474,9 +1475,9 @@ class TestQuantizeEagerPTQDynamic(QuantizationTestCase):
             checkHooksIsPresent(model)
 
     @skipIfNoFBGEMM
-    def test_embedding_ops_dynamic(self):
+    def test_embedding_bag_dynamic(self):
         class EmbeddingBagWithLinear(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.emb = torch.nn.EmbeddingBag(num_embeddings=10, embedding_dim=12,
                                                  include_last_offset=True, scale_grad_by_freq=False, mode='sum')
@@ -1495,9 +1496,30 @@ class TestQuantizeEagerPTQDynamic(QuantizationTestCase):
         q_model = quantize_dynamic(model, qconfig_dict)
 
         q_model(indices, offsets, torch.randn(5, 5))
-        self.assertTrue('QuantizedEmbedding' in str(q_model))
-        self.assertTrue('DynamicQuantizedLinear' in str(q_model))
+        self.assertTrue('QuantizedEmbeddingBag' in str(q_model.emb))
+        self.assertTrue('DynamicQuantizedLinear' in str(q_model.fc))
 
+    @skipIfNoFBGEMM
+    def test_embedding_ops_dynamic(self):
+        class EmbeddingWithLinear(torch.nn.Module):
+            def __init__(self) -> None:
+                super().__init__()
+                self.emb = torch.nn.Embedding(
+                    num_embeddings=10, embedding_dim=12, scale_grad_by_freq=False)
+                self.fc = torch.nn.Linear(5, 5)
+
+            def forward(self, indices, linear_in):
+                return self.emb(indices), self.fc(linear_in)
+        model = EmbeddingWithLinear().eval()
+        qconfig_dict = {
+            torch.nn.Embedding : float_qparams_weight_only_qconfig,
+            torch.nn.Linear: default_dynamic_qconfig
+        }
+        indices = torch.tensor([9, 6, 5, 7, 8, 8, 9, 2, 8, 6, 6, 9, 1, 6, 8, 8, 3, 2, 3, 6, 3, 6, 5, 7, 0, 8, 4, 6, 5, 8, 2, 3])
+        q_model = quantize_dynamic(model, qconfig_dict)
+        self.assertTrue('QuantizedEmbedding' in str(q_model.emb))
+        self.assertTrue('DynamicQuantizedLinear' in str(q_model.fc))
+        q_model(indices, torch.randn(5, 5))
 
 if __name__ == '__main__':
     raise RuntimeError("This test file is not meant to be run directly, use:\n\n"
