@@ -1,6 +1,5 @@
 # Owner(s): ["oncall: quantization"]
 import copy
-
 import unittest
 from typing import List
 
@@ -12,16 +11,14 @@ from torch.ao.quantization.quantizer.xnnpack_quantizer import (
     get_symmetric_quantization_config,
 )
 from torch.ao.quantization.quantizer.xnnpack_quantizer_utils import OP_TO_ANNOTATOR
-
 from torch.fx import Node
-
 from torch.testing._internal.common_quantization import QuantizationTestCase
-from torch.testing._internal.common_utils import IS_WINDOWS
+from torch.testing._internal.common_utils import IS_WINDOWS, skipIfCrossRef
 
 
 class TestHelperModules:
     class Conv2dWithObsSharingOps(torch.nn.Module):
-        def __init__(self):
+        def __init__(self) -> None:
             super().__init__()
             self.conv = torch.nn.Conv2d(3, 3, 3)
             self.hardtanh = torch.nn.Hardtanh()
@@ -77,8 +74,8 @@ class TestMetaDataPorting(QuantizationTestCase):
                         f"from_node metadata is of type {type(from_node_meta)}, but expected list"
                     )
                 for meta in from_node_meta:
-                    node_target = meta[1]
-                    if node_target == from_node:
+                    node_target = meta.target
+                    if node_target == str(from_node):
                         node_tag = n.meta.get("quantization_tag", None)
                         if node_tag is None or tag != node_tag:
                             not_found_nodes += str(n.target) + ", "
@@ -102,10 +99,10 @@ class TestMetaDataPorting(QuantizationTestCase):
 
         # program capture
         m = copy.deepcopy(m_eager)
-        m = torch._export.capture_pre_autograd_graph(
+        m = torch.export.export_for_training(
             m,
             example_inputs,
-        )
+        ).module()
 
         m = prepare_pt2e(m, quantizer)
         # Calibrate
@@ -142,6 +139,8 @@ class TestMetaDataPorting(QuantizationTestCase):
             self.assertEqual(v, node_tags[k])
         return m
 
+    @skipIfCrossRef  # mlazos: retracing FX graph with torch function mode doesn't propagate metadata, because the stack
+    # trace of the mode torch function impl doesn't match the traced graph stored lineno.
     def test_simple_metadata_porting(self):
         """
         Model under test
@@ -466,7 +465,7 @@ class TestMetaDataPorting(QuantizationTestCase):
         """
 
         class MatmulWithConstInput(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.register_parameter("w", torch.nn.Parameter(torch.rand(8, 16)))
 
