@@ -53,7 +53,7 @@ class TestOptimizer(TestCase):
         linear_weight_shape = (weight_output_dim, linear_input_shape)
 
         class MyTestModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.conv_weight = torch.nn.Parameter(torch.rand(conv_weight_shape))
                 self.conv_bias = torch.nn.Parameter(torch.rand(conv_bias_shape))
@@ -85,7 +85,7 @@ class TestOptimizer(TestCase):
 
 
         class BNTestModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.conv = torch.nn.Conv2d(1, 20, 5, 1)
                 self.bn = torch.nn.BatchNorm2d(num_features=20)
@@ -166,7 +166,7 @@ class TestOptimizer(TestCase):
         torch.testing.assert_close(bn_scripted_module(bn_input), no_bn_fold_scripted_module(bn_input), rtol=1e-2, atol=1e-3)
 
         class MyMobileOptimizedTagTest(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.linear_weight = torch.nn.Parameter(torch.rand(linear_weight_shape))
                 self.linear_bias = torch.nn.Parameter(torch.rand(weight_output_dim))
@@ -183,7 +183,7 @@ class TestOptimizer(TestCase):
         self.assertTrue(tag)
 
         class MyPreserveMethodsTest(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.linear_weight = torch.nn.Parameter(torch.rand(linear_weight_shape))
                 self.linear_bias = torch.nn.Parameter(torch.rand(weight_output_dim))
@@ -207,7 +207,7 @@ class TestOptimizer(TestCase):
         self.assertNotEqual(preserveThis, None)
 
         class OptimizeNoForwardTest(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.l = nn.Linear(10, 100)
                 self.l2 = nn.Linear(100, 1)
@@ -233,7 +233,7 @@ class TestOptimizer(TestCase):
         torch.testing.assert_close(initial_result, optimized_result, rtol=1e-2, atol=1e-3)
 
         class BNTestNoForwardModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.conv = torch.nn.Conv2d(1, 20, 5, 1)
                 self.bn = torch.nn.BatchNorm2d(num_features=20)
@@ -272,7 +272,7 @@ class TestOptimizer(TestCase):
             return
 
         class Child(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.conv2 = nn.Conv2d(1, 1, 1)
 
@@ -281,7 +281,7 @@ class TestOptimizer(TestCase):
                 return x
 
         class Parent(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.quant = torch.ao.quantization.QuantStub()
                 self.conv1 = nn.Conv2d(1, 1, 1)
@@ -307,7 +307,7 @@ class TestOptimizer(TestCase):
 
     def test_generate_mobile_module_lints(self):
         class MyTestModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.fc = torch.nn.Linear(4, 4)
                 self.dropout = torch.nn.Dropout(p=0.5)
@@ -318,7 +318,7 @@ class TestOptimizer(TestCase):
                 return out
 
         class MyBNModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.bn = torch.nn.BatchNorm2d(4, affine=True)
 
@@ -409,7 +409,7 @@ class TestOptimizer(TestCase):
             return
 
         class Standalone(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.quant = torch.ao.quantization.QuantStub()
                 self.conv1 = nn.Conv2d(1, 1, 1)
@@ -427,10 +427,9 @@ class TestOptimizer(TestCase):
 
             def fuse_model(self):
                 torch.ao.quantization.fuse_modules(self, [['conv2', 'relu']], inplace=True)
-                pass
 
         class Child(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.conv1 = nn.Conv2d(1, 1, 1)
 
@@ -439,7 +438,7 @@ class TestOptimizer(TestCase):
                 return x
 
         class Parent(nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.quant = torch.ao.quantization.QuantStub()
                 self.conv1 = nn.Conv2d(1, 1, 1)
@@ -511,7 +510,7 @@ class TestOptimizer(TestCase):
 
     def test_clone_module_with_class(self):
         class MyInnerTestModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.pqr = torch.Tensor([10., 20., 30.])
 
@@ -523,7 +522,7 @@ class TestOptimizer(TestCase):
                 return 20
 
         class MyTestModule(torch.nn.Module):
-            def __init__(self):
+            def __init__(self) -> None:
                 super().__init__()
                 self.abc = 23
                 self.pqr = torch.Tensor([1., 2., 3.])
