@@ -1,11 +1,9 @@
 # Owner(s): ["module: dynamo"]
 
 import functools
-
 from unittest import expectedFailure as xfail, skipIf as skipif
 
 import numpy
-
 import pytest
 from pytest import raises as assert_raises
 
@@ -15,9 +13,10 @@ from torch.testing._internal.common_utils import (
     run_tests,
     TEST_WITH_TORCHDYNAMO,
     TestCase,
-    xpassIfTorchDynamo,
+    xpassIfTorchDynamo_np,
 )
 
+
 # If we are going to trace through these, we should use NumPy
 # If testing on eager mode, we use torch._numpy
 if TEST_WITH_TORCHDYNAMO:
@@ -27,13 +26,18 @@ if TEST_WITH_TORCHDYNAMO:
         atleast_1d,
         atleast_2d,
         atleast_3d,
-        AxisError,
         concatenate,
         hstack,
         newaxis,
         stack,
         vstack,
     )
+
+    if int(numpy.__version__[0]) >= 2:
+        from numpy.exceptions import AxisError
+    else:
+        from numpy import AxisError
+
     from numpy.testing import assert_, assert_array_equal, assert_equal
 else:
     import torch._numpy as np
@@ -359,7 +363,7 @@ class TestConcatenate(TestCase):
         assert out is rout
         assert np.all(r == rout)
 
-    @xpassIfTorchDynamo  # (reason="concatenate(x, axis=None) relies on x being a sequence")
+    @xpassIfTorchDynamo_np  # (reason="concatenate(x, axis=None) relies on x being a sequence")
     def test_large_concatenate_axis_None(self):
         # When no axis is given, concatenate uses flattened versions.
         # This also had a bug with many arrays (see gh-5979).
