@@ -1,12 +1,11 @@
 # Owner(s): ["module: dynamo"]
 
-# from numpy.testing._private.utils import requires_memory
 import functools
-
 from unittest import expectedFailure as xfail, skipIf
 
 from pytest import raises as assert_raises
 
+
 skip = functools.partial(skipIf, True)
 
 from torch.testing._internal.common_utils import (
@@ -16,9 +15,10 @@ from torch.testing._internal.common_utils import (
     slowTest as slow,
     TEST_WITH_TORCHDYNAMO,
     TestCase,
-    xpassIfTorchDynamo,
+    xpassIfTorchDynamo_np,
 )
 
+
 if TEST_WITH_TORCHDYNAMO:
     import numpy as np
     from numpy import histogram, histogram_bin_edges, histogramdd
@@ -29,7 +29,6 @@ if TEST_WITH_TORCHDYNAMO:
         assert_array_almost_equal,
         assert_array_equal,
         assert_equal,
-        # assert_array_max_ulp, #assert_raises_regex, suppress_warnings,
     )
 else:
     import torch._numpy as np
@@ -41,7 +40,6 @@ else:
         assert_array_almost_equal,
         assert_array_equal,
         assert_equal,
-        # assert_array_max_ulp, #assert_raises_regex, suppress_warnings,
     )
 
 
@@ -203,7 +201,7 @@ class TestHistogram(TestCase):
         )
         assert_almost_equal(a, [0.2, 0.1, 0.1, 0.075])
 
-    @xpassIfTorchDynamo  # (reason="histogram complex weights")
+    @xpassIfTorchDynamo_np  # (reason="histogram complex weights")
     def test_exotic_weights(self):
         # Test the use of weights that are not integer or floats, but e.g.
         # complex numbers or object types.
@@ -289,7 +287,7 @@ class TestHistogram(TestCase):
         with assert_raises((RuntimeError, ValueError)):
             np.histogram(vals, bins=bins)
 
-    @xpassIfTorchDynamo  # (reason="no uint64")
+    @xpassIfTorchDynamo_np  # (reason="no uint64")
     def test_unsigned_monotonicity_check(self):
         # Ensures ValueError is raised if bins not increasing monotonically
         # when bins contain unsigned values (see #9222)
@@ -315,7 +313,7 @@ class TestHistogram(TestCase):
         np.histogram([np.array(0.5) for i in range(10)] + [0.500000000000001])
         np.histogram([np.array(0.5) for i in range(10)] + [0.5])
 
-    @xpassIfTorchDynamo  # (reason="bins='auto'")
+    @xpassIfTorchDynamo_np  # (reason="bins='auto'")
     def test_some_nan_values(self):
         # gh-7503
         one_nan = np.array([0, 1, np.nan])
@@ -396,14 +394,14 @@ class TestHistogram(TestCase):
         self.do_precision_lower_bound(float_small, float_large)
         self.do_precision_upper_bound(float_small, float_large)
 
-    @xpassIfTorchDynamo  # (reason="mixed dtypes")
+    @xpassIfTorchDynamo_np  # (reason="mixed dtypes")
     def test_precision(self):
         # not looping results in a useful stack trace upon failure
         self.do_precision(np.half, np.single)
         self.do_precision(np.half, np.double)
         self.do_precision(np.single, np.double)
 
-    @xpassIfTorchDynamo  # (reason="histogram_bin_edges")
+    @xpassIfTorchDynamo_np  # (reason="histogram_bin_edges")
     def test_histogram_bin_edges(self):
         hist, e = histogram([1, 2, 3, 4], [1, 2])
         edges = histogram_bin_edges([1, 2, 3, 4], [1, 2])
@@ -419,7 +417,7 @@ class TestHistogram(TestCase):
         assert_allclose(edges, e, atol=2e-15)
 
     # @requires_memory(free_bytes=1e10)
-    @xpassIfTorchDynamo  # (reason="pytorch does not support bins = [int, int, array]")
+    @xpassIfTorchDynamo_np  # (reason="pytorch does not support bins = [int, int, array]")
     @slow
     def test_big_arrays(self):
         sample = np.zeros([100000000, 3])
@@ -430,7 +428,7 @@ class TestHistogram(TestCase):
         assert_equal(type(hist), type((1, 2)))
 
 
-@xpassIfTorchDynamo  # (reason="TODO")
+@xpassIfTorchDynamo_np  # (reason="TODO")
 @instantiate_parametrized_tests
 class TestHistogramOptimBinNums(TestCase):
     """
@@ -856,13 +854,13 @@ class TestHistogramdd(TestCase):
             (RuntimeError, ValueError), np.histogramdd, x, bins=[1, 1, 1, [1, 2, 3, -3]]
         )
 
-    @xpassIfTorchDynamo  # (reason="pytorch does not support bins = [int, int, array]")
+    @xpassIfTorchDynamo_np  # (reason="pytorch does not support bins = [int, int, array]")
     def test_bins_error_2(self):
         # mixing scalar (# of bins) and explicit bin arrays, ugh
         x = np.arange(8).reshape(2, 4)
         assert_(np.histogramdd(x, bins=[1, 1, 1, [1, 2, 3, 4]]))
 
-    @xpassIfTorchDynamo  # (reason="pytorch does not support bins = [int, int, array]")
+    @xpassIfTorchDynamo_np  # (reason="pytorch does not support bins = [int, int, array]")
     def test_inf_edges(self):
         # Test using +/-inf bin edges works. See #1788.
         x = np.arange(6).reshape(3, 2)
@@ -913,7 +911,7 @@ class TestHistogramdd(TestCase):
             range=[[0.0, 1.0], [np.nan, 0.75], [0.25, 0.5]],
         )
 
-    @xpassIfTorchDynamo  # (reason="pytorch does not allow equal entries")
+    @xpassIfTorchDynamo_np  # (reason="pytorch does not allow equal entries")
     def test_equal_edges(self):
         """Test that adjacent entries in an edge array can be equal"""
         x = np.array([0, 1, 2])
