@@ -12,7 +12,7 @@ from torch.testing._internal.common_utils import (
     run_tests,
     TEST_WITH_TORCHDYNAMO,
     TestCase,
-    xpassIfTorchDynamo,
+    xpassIfTorchDynamo_np,
 )
 
 
@@ -127,7 +127,7 @@ class TestMean(TestCase):
         # of float32.
         assert np.mean(np.ones(100000, dtype="float16")) == 1
 
-    @xpassIfTorchDynamo  # (reason="XXX: mean(..., where=...) not implemented")
+    @xpassIfTorchDynamo_np  # (reason="XXX: mean(..., where=...) not implemented")
     def test_mean_where(self):
         a = np.arange(16).reshape((4, 4))
         wh_full = np.array(
@@ -194,7 +194,7 @@ class TestSum(TestCase):
         assert res_float.dtype == "float64"
 
     @skipIf(numpy.__version__ < "1.24", reason="NP_VER: fails on NumPy 1.23.x")
-    @xpassIfTorchDynamo  # (reason="sum: does not warn on overflow")
+    @xpassIfTorchDynamo_np  # (reason="sum: does not warn on overflow")
     def test_sum_dtypes_warnings(self):
         for dt in (int, np.float16, np.float32, np.float64):
             for v in (0, 1, 2, 7, 8, 9, 15, 16, 19, 127, 128, 1024, 1235):
@@ -261,7 +261,7 @@ class TestSum(TestCase):
         d += d
         assert_allclose(d, 2.0 + 2j, atol=1.5e-7)
 
-    @xpassIfTorchDynamo  # (reason="initial=... need implementing")
+    @xpassIfTorchDynamo_np  # (reason="initial=... need implementing")
     def test_sum_initial(self):
         # Integer, single axis
         assert_equal(np.sum([3], initial=2), 5)
@@ -275,7 +275,7 @@ class TestSum(TestCase):
             [12, 12, 12],
         )
 
-    @xpassIfTorchDynamo  # (reason="where=... need implementing")
+    @xpassIfTorchDynamo_np  # (reason="where=... need implementing")
     def test_sum_where(self):
         # More extensive tests done in test_reduction_with_where.
         assert_equal(np.sum([[1.0, 2.0], [3.0, 4.0]], where=[True, False]), 4.0)
