@@ -155,11 +155,12 @@ class TestParsePrevTests(TestTD):
 
 class TestFilePath(TestTD):
     def test_get_keywords(self) -> None:
-        self.assertEqual(get_keywords("test/test_car.py"), [])
-        self.assertEqual(get_keywords("test/nn/test_amp.py"), ["nn"])
-        self.assertEqual(get_keywords("torch/nn/test_amp.py"), ["nn"])
+        self.assertEqual(get_keywords("test/test_car.py"), ["car"])
+        self.assertEqual(get_keywords("test/nn/test_amp.py"), ["nn", "amp"])
+        self.assertEqual(get_keywords("torch/nn/test_amp.py"), ["nn", "amp"])
         self.assertEqual(
-            get_keywords("torch/nn/mixed_precision/test_amp.py"), ["nn", "amp"]
+            get_keywords("torch/nn/mixed_precision/test_something.py"),
+            ["nn", "amp", "something"],
         )
 
     def test_match_keywords(self) -> None:
