@@ -1,27 +1,25 @@
-# mypy: allow-untyped-defs
-# mypy: disable-error-code="method-assign"
-
 import functools
 import weakref
+from typing import Any, List, Type
 
 import torch.nn
 from torch.nn import Module
 
 from . import config
-from .utils import ExactWeakKeyDictionary, is_lazy_module, nn_module_has_global_hooks
+from .utils import ExactWeakKeyDictionary, nn_module_has_global_hooks
 
 
 unpatched_nn_module_init = torch.nn.Module.__init__
 
 
 class MutationTracker:
-    db = ExactWeakKeyDictionary()
+    db: ExactWeakKeyDictionary = ExactWeakKeyDictionary()
 
-    def __init__(self):
-        self.mutation_count = 0
-        self.watchers = []
+    def __init__(self) -> None:
+        self.mutation_count: int = 0
+        self.watchers: List[weakref.ReferenceType[Any]] = []
 
-    def on_mutation(self, name):
+    def on_mutation(self, name: str) -> None:
         self.mutation_count += 1
         tmp = self.watchers
         self.watchers = []
@@ -30,11 +28,11 @@ class MutationTracker:
             if guarded is not None:
                 guarded.invalidate(ref)
 
-    def track(self, guarded_code):
+    def track(self, guarded_code: Any) -> None:
         self.watchers.append(weakref.ref(guarded_code))
 
 
-def watch(obj, guarded_code):
+def watch(obj: Any, guarded_code: Any) -> None:
     """invalidate guarded_code when obj is mutated"""
     ensure_patched(type(obj))
 
@@ -44,13 +42,13 @@ def watch(obj, guarded_code):
     tracker.track(guarded_code)
 
 
-def ensure_patched(cls):
+def ensure_patched(cls: Any) -> None:
     if getattr(cls, "___needs_mutation_patch", True):
         cls.___needs_mutation_patch = False
         original_setattr = cls.__setattr__
 
         @functools.wraps(original_setattr)
-        def custom_setattr(self, key, value):
+        def custom_setattr(self: Any, key: str, value: Any) -> None:
             try:
                 MutationTracker.db[self].on_mutation(key)
             except KeyError:
@@ -61,48 +59,46 @@ def ensure_patched(cls):
 
 
 class GenerationTracker:
-    generation = 0
-    dynamic_classes = ExactWeakKeyDictionary()
-    generation_values = ExactWeakKeyDictionary()
+    generation: int = 0
+    dynamic_classes: ExactWeakKeyDictionary = ExactWeakKeyDictionary()
+    generation_values: ExactWeakKeyDictionary = ExactWeakKeyDictionary()
 
     @classmethod
-    def tag(cls, obj):
+    def tag(cls, obj: Any) -> None:
         cls.generation_values[obj] = cls.generation
 
     @staticmethod
-    def mark_class_dynamic(cls):
+    def mark_class_dynamic(cls: Type[torch.nn.Module]) -> None:
         assert issubclass(cls, torch.nn.Module)
         GenerationTracker.dynamic_classes[cls] = True
 
     @classmethod
-    def get_generation_value(cls, obj):
+    def get_generation_value(cls, obj: Any) -> int:
         if obj not in cls.generation_values:
             return -1
         return cls.generation_values[obj]
 
     @classmethod
-    def check(cls, obj):
+    def check(cls, obj: Any) -> bool:
         return (
             obj in cls.generation_values
             and cls.generation_values[obj] == cls.generation
         )
 
     @classmethod
-    def clear(cls):
+    def clear(cls) -> None:
         cls.generation = 0
         cls.dynamic_classes = ExactWeakKeyDictionary()
         cls.generation_values = ExactWeakKeyDictionary()
 
 
-def is_dynamic_nn_module(obj, is_export):
+def is_dynamic_nn_module(obj: Any, is_export: bool) -> bool:
     """Check for nn.Modules() created dynamically or mutated"""
     if isinstance(obj, torch.nn.Module) and "forward" in obj.__dict__:
         # A monkey patched `.forward` indicates something wacky is going on
         return True
     if hasattr(obj, "torchdynamo_force_dynamic"):
         return obj.torchdynamo_force_dynamic
-    if is_lazy_module(obj):
-        return False
     # For export, we will have to fix
     # 1) Input signature problem because params are lifted as inputs
     # 2) nn module stack info changes
@@ -122,7 +118,7 @@ def is_dynamic_nn_module(obj, is_export):
     return dyn
 
 
-def install_generation_tagging_init():
+def install_generation_tagging_init() -> None:
     """
     Monkey patch torch.nn.Module.__init__ and torch.nn.Module.__setstate__
     so we can detect nn.Module instances created dynamically inside forward methods.
@@ -131,19 +127,19 @@ def install_generation_tagging_init():
     if getattr(Module, "___needs_generation_tag_patch", True):
         init = Module.__init__
 
-        def patched_init(self, *args, **kwargs):
+        def patched_init(self: Module, *args: Any, **kwargs: Any) -> None:
             init(self, *args, **kwargs)
             GenerationTracker.tag(self)
 
-        Module.__init__ = patched_init
+        Module.__init__ = patched_init  # type: ignore[method-assign]
 
         setstate = Module.__setstate__
 
-        def patched_setstate(self, state):
+        def patched_setstate(self: Module, state: Any) -> None:
             setstate(self, state)
             GenerationTracker.tag(self)
 
-        Module.__setstate__ = patched_setstate
+        Module.__setstate__ = patched_setstate  # type: ignore[method-assign]
 
         Module.___needs_generation_tag_patch = False  # type: ignore[attr-defined]
 
