@@ -1,16 +1,28 @@
 # mypy: allow-untyped-defs
 from __future__ import annotations
 
+import collections
 import functools
 import itertools
 import re
 from enum import auto, Enum
-from typing import Any, Callable, Dict, List, NamedTuple, Optional, Sequence, Tuple
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    NamedTuple,
+    Optional,
+    Sequence,
+    Tuple,
+    TypeVar,
+)
 
 import sympy
 
 import torch.fx
 from torch._dynamo.utils import identity
+from torch.fx.proxy import Scope, TracerBase
 from torch.utils._sympy.symbol import SymT
 
 from . import config, dependencies
@@ -19,6 +31,9 @@ from .utils import cache_on_self, sympy_index_symbol_with_prefix, sympy_subs
 from .virtualized import ops, V
 
 
+T = TypeVar("T")
+
+
 class InterpreterShim(torch.fx.Interpreter):
     @staticmethod
     @functools.lru_cache(None)
@@ -45,6 +60,16 @@ class InterpreterShim(torch.fx.Interpreter):
             return super().run(*args, **kwargs)
 
 
+# We don't need the nn.Module and constant handling in Tracer
+class LightTracer(TracerBase):
+    def __init__(self):
+        super().__init__()
+        self.graph = torch.fx.Graph(tracer_cls=self.__class__)  # type: ignore[arg-type]
+        self.scope = Scope("", None)
+        self.module_stack = {}  # type: ignore[assignment]
+        self.node_name_to_scope = {}
+
+
 class MemoryEntry(NamedTuple):
     index_name: str  # LoopBody.indexing_exprs[index_name]
     buffer_name: Optional[str]
@@ -72,10 +97,11 @@ class LoopBody:
     indexing_exprs_name: Dict[sympy.Expr, str]
     submodules: Dict[str, Any]
     subblocks: Dict[str, LoopBodyBlock]
-    indirect_vars: List[str]
+    indirect_vars: List[sympy.Symbol]
     indirect_var_ranges: Dict[sympy.Symbol, sympy.Expr]
     root_block: LoopBodyBlock
     memory_usage: Dict[MemoryUsageType, List[MemoryEntry]]
+    op_counts: collections.Counter[str]
 
     def __init__(self, fn, args, var_ranges, iter_vars, reduce_vars):
         super().__init__()
@@ -106,6 +132,7 @@ class LoopBody:
         self.indirect_vars = []
         self.indirect_var_ranges: Dict[sympy.Symbol, sympy.Expr] = {}
         self.memory_usage = {t: [] for t in MemoryUsageType}
+        self.op_counts = collections.Counter()
         self.root_block = LoopBodyBlock(self, fn, args)  # traces
         del self.indexing_exprs_name  # not used after _init_with_tracing
 
@@ -124,6 +151,7 @@ class LoopBody:
         self.indirect_vars = other.indirect_vars
         self.indirect_var_ranges = other.indirect_var_ranges
         self.memory_usage = other.memory_usage
+        self.op_counts = other.op_counts
         self.root_block = other.root_block.clone(self)
 
         submodules = {**other.submodules}
@@ -133,6 +161,9 @@ class LoopBody:
             **{k: v.clone(self) for k, v in submodules.items()},  # type: ignore[attr-defined]
         }
 
+    def has_op(self, name: str):
+        return self.op_counts.get(name, 0) > 0
+
     def merge_loops(self) -> LoopBody:
         """
         Merge both iteration and reduction loops and return a new LoopBody.
@@ -184,7 +215,7 @@ class LoopBody:
         # use the original symbol prefix
         # Can try to optimize if this is a bottleneck for compilation time
         (iter_vars2, reduce_vars2), var_ranges2 = dependencies.index_vars_no_squeeze(
-            iter_sizes, reduce_sizes, prefix="z"
+            iter_sizes, reduce_sizes, prefix="p"
         )
         new_body2 = LoopBody(
             new_body, (iter_vars2, reduce_vars2), var_ranges2, iter_vars2, reduce_vars2
@@ -228,7 +259,7 @@ class LoopBody:
 
         # use the original symbol prefix so we can do multiple round of reordering
         (iter_vars2, reduce_vars2), var_ranges2 = dependencies.index_vars_no_squeeze(
-            *new_sizes, prefix="z"  # type: ignore[arg-type]
+            *new_sizes, prefix="p"  # type: ignore[arg-type]
         )
         new_body = LoopBody(
             loop_body, (iter_vars2, reduce_vars2), var_ranges2, iter_vars2, reduce_vars2
@@ -468,17 +499,51 @@ class LoopBodyBlock:
 
             def bucketize(
                 self,
-                values,
-                offsets_name: str,
-                offsets_size: sympy.Expr,
+                values: T,
+                boundaries: Tuple[str, sympy.Expr, sympy.Expr, sympy.Expr],
+                boundary_indices: T,
                 indexing_dtype: torch.dtype,
                 right: bool,
-            ):
-                offsets_size = add_index(
-                    offsets_size, MemoryUsageType.BUCKETIZE, buffer_name=offsets_name
+                sorter: Optional[Tuple[str, sympy.Expr]] = None,
+                sorter_indices: Optional[T] = None,
+            ) -> T:
+                """
+                See [Note: Inductor bucketize op]
+                """
+                boundaries = (
+                    boundaries[0],
+                    add_index(
+                        boundaries[1],
+                        MemoryUsageType.BUCKETIZE,
+                        buffer_name=boundaries[0],
+                    ),
+                    add_index(
+                        boundaries[2],
+                        MemoryUsageType.BUCKETIZE,
+                        buffer_name=boundaries[0],
+                    ),
+                    add_index(
+                        boundaries[3],
+                        MemoryUsageType.BUCKETIZE,
+                        buffer_name=boundaries[0],
+                    ),
                 )
+                if sorter is not None:
+                    sorter = (
+                        sorter[0],
+                        add_index(
+                            sorter[1], MemoryUsageType.BUCKETIZE, buffer_name=sorter[0]
+                        ),
+                    )
+
                 return self._inner.bucketize(
-                    values, offsets_name, offsets_size, indexing_dtype, right
+                    values,
+                    boundaries,
+                    boundary_indices,
+                    indexing_dtype,
+                    right,
+                    sorter,
+                    sorter_indices,
                 )
 
             @staticmethod
@@ -545,15 +610,15 @@ class LoopBodyBlock:
             def output(result):
                 tracer.create_proxy("output", "output", (result,), {})
 
-        tracer = torch.fx.Tracer()
-        tracer.graph = torch.fx.Graph(tracer_cls=tracer.__class__)
+        tracer = LightTracer()
         proxy_ops = tracer.create_proxy("placeholder", "ops", (), {})
 
         from .index_propagation import IndexPropagation
         from .sizevars import SimplifyIndexing
 
-        handler: Any = SimplifyIndexing(
-            CaptureIndexing(proxy_ops), self.body.var_ranges
+        handler: Any = CountOps(
+            SimplifyIndexing(CaptureIndexing(proxy_ops), self.body.var_ranges),
+            body.op_counts,
         )
         if config.constant_and_index_propagation:
             handler = IndexPropagation(
@@ -592,3 +657,13 @@ class LoopBodyBlock:
         copy = LoopBodyBlock.__new__(LoopBodyBlock)
         copy.__dict__.update({**self.__dict__, "body": body})
         return copy
+
+
+class CountOps:
+    def __init__(self, inner: Any, counts: collections.Counter[str]):
+        self._inner = inner
+        self._counts = counts
+
+    def __getattr__(self, name):
+        self._counts[name] += 1
+        return getattr(self._inner, name)
