@@ -12,20 +12,25 @@
 #include <c10/util/StringUtil.h>
 
 PyObject *THPException_FatalError, *THPException_LinAlgError,
-    *THPException_OutOfMemoryError, *THPException_DistBackendError;
+    *THPException_OutOfMemoryError, *THPException_DistError,
+    *THPException_DistBackendError, *THPException_DistNetworkError,
+    *THPException_DistStoreError;
 
 #define ASSERT_TRUE(cond) \
   if (!(cond))            \
   return false
 bool THPException_init(PyObject* module) {
+  // NOLINTNEXTLINE(bugprone-assignment-in-if-condition)
   ASSERT_TRUE(
       THPException_FatalError =
           PyErr_NewException("torch.FatalError", nullptr, nullptr));
+  // NOLINTNEXTLINE(bugprone-assignment-in-if-condition)
   ASSERT_TRUE(
       PyModule_AddObject(module, "FatalError", THPException_FatalError) == 0);
 
   // Set the doc string here since _add_docstr throws malloc errors if tp_doc is
   // modified for an error class.
+  // NOLINTNEXTLINE(bugprone-assignment-in-if-condition)
   ASSERT_TRUE(
       THPException_LinAlgError = PyErr_NewExceptionWithDoc(
           "torch._C._LinAlgError",
@@ -52,34 +57,70 @@ could not be completed because the input matrix is singular.",
       PyModule_AddObject(module, "_LinAlgError", THPException_LinAlgError) ==
       0);
 
+  // NOLINTNEXTLINE(bugprone-assignment-in-if-condition)
   ASSERT_TRUE(
       THPException_OutOfMemoryError = PyErr_NewExceptionWithDoc(
-          "torch.cuda.OutOfMemoryError",
-          "Exception raised when CUDA is out of memory",
+          "torch.OutOfMemoryError",
+          "Exception raised when device is out of memory",
+          PyExc_RuntimeError,
+          nullptr));
+  PyTypeObject* type = (PyTypeObject*)THPException_OutOfMemoryError;
+  type->tp_name = "torch.OutOfMemoryError";
+  ASSERT_TRUE(
+      PyModule_AddObject(
+          module, "OutOfMemoryError", THPException_OutOfMemoryError) == 0);
+
+  // NOLINTNEXTLINE(bugprone-assignment-in-if-condition)
+  ASSERT_TRUE(
+      THPException_DistError = PyErr_NewExceptionWithDoc(
+          "torch.distributed.DistError",
+          "Exception raised when an error occurs in the distributed library",
           PyExc_RuntimeError,
           nullptr));
   ASSERT_TRUE(
-      PyModule_AddObject(
-          module, "_OutOfMemoryError", THPException_OutOfMemoryError) == 0);
+      PyModule_AddObject(module, "_DistError", THPException_DistError) == 0);
 
+  // NOLINTNEXTLINE(bugprone-assignment-in-if-condition)
   ASSERT_TRUE(
       THPException_DistBackendError = PyErr_NewExceptionWithDoc(
           "torch.distributed.DistBackendError",
           "Exception raised when a backend error occurs in distributed",
-          PyExc_RuntimeError,
+          THPException_DistError,
           nullptr));
   ASSERT_TRUE(
       PyModule_AddObject(
           module, "_DistBackendError", THPException_DistBackendError) == 0);
 
+  // NOLINTNEXTLINE(bugprone-assignment-in-if-condition)
+  ASSERT_TRUE(
+      THPException_DistNetworkError = PyErr_NewExceptionWithDoc(
+          "torch.distributed.DistNetworkError",
+          "Exception raised when a network error occurs in distributed",
+          THPException_DistError,
+          nullptr));
+  ASSERT_TRUE(
+      PyModule_AddObject(
+          module, "_DistNetworkError", THPException_DistNetworkError) == 0);
+
+  // NOLINTNEXTLINE(bugprone-assignment-in-if-condition)
+  ASSERT_TRUE(
+      THPException_DistStoreError = PyErr_NewExceptionWithDoc(
+          "torch.distributed.DistStoreError",
+          "Exception raised when an error occurs in the distributed store",
+          THPException_DistError,
+          nullptr));
+  ASSERT_TRUE(
+      PyModule_AddObject(
+          module, "_DistStoreError", THPException_DistStoreError) == 0);
+
   return true;
 }
 
 namespace torch {
 
-void processErrorMsgInplace(std::string& str) {
+static void processErrorMsgInplace(std::string& str) {
   // Translate Aten types to their respective pytorch ones
-  constexpr std::array<std::pair<c10::string_view, c10::string_view>, 64>
+  constexpr std::array<std::pair<std::string_view, std::string_view>, 64>
       changes{{
           // TODO: remove torch.(cuda.|)sparse.*Tensor items?
           {"Variable[SparseCUDAByteType]", "torch.cuda.sparse.ByteTensor"},
@@ -163,15 +204,14 @@ std::string processErrorMsg(std::string str) {
 }
 
 static std::string formatMessage(const char* format, va_list fmt_args) {
-  static const size_t ERROR_BUF_SIZE = 1024;
-  // NOLINTNEXTLINE(modernize-avoid-c-arrays,cppcoreguidelines-avoid-c-arrays)
-  char error_buf[ERROR_BUF_SIZE];
-  vsnprintf(error_buf, ERROR_BUF_SIZE, format, fmt_args);
-
-  // Ensure that the string is null terminated
-  error_buf[sizeof(error_buf) / sizeof(*error_buf) - 1] = 0;
-
-  return std::string(error_buf);
+  constexpr size_t ERROR_BUF_SIZE = 1024;
+  std::string error_buf(ERROR_BUF_SIZE, '\0');
+  auto res = vsnprintf(error_buf.data(), ERROR_BUF_SIZE, format, fmt_args);
+  if (res < 0) {
+    res = 0;
+  }
+  error_buf.resize(res);
+  return error_buf;
 }
 
 void translate_exception_to_python(const std::exception_ptr& e_ptr) {
@@ -182,46 +222,18 @@ void translate_exception_to_python(const std::exception_ptr& e_ptr) {
         "called with invalid exception pointer");
     std::rethrow_exception(e_ptr);
   }
-  CATCH_ALL_ERRORS(return )
-}
-
-IndexError::IndexError(const char* format, ...) {
-  va_list fmt_args;
-  va_start(fmt_args, format);
-  msg = formatMessage(format, fmt_args);
-  va_end(fmt_args);
+  CATCH_ALL_ERRORS(return)
 }
 
 TypeError::TypeError(const char* format, ...) {
-  va_list fmt_args;
-  va_start(fmt_args, format);
-  msg = formatMessage(format, fmt_args);
-  va_end(fmt_args);
-}
-
-ValueError::ValueError(const char* format, ...) {
-  va_list fmt_args;
-  va_start(fmt_args, format);
-  msg = formatMessage(format, fmt_args);
-  va_end(fmt_args);
-}
-
-NotImplementedError::NotImplementedError(const char* format, ...) {
-  va_list fmt_args;
+  va_list fmt_args{};
   va_start(fmt_args, format);
   msg = formatMessage(format, fmt_args);
   va_end(fmt_args);
 }
 
 AttributeError::AttributeError(const char* format, ...) {
-  va_list fmt_args;
-  va_start(fmt_args, format);
-  msg = formatMessage(format, fmt_args);
-  va_end(fmt_args);
-}
-
-LinAlgError::LinAlgError(const char* format, ...) {
-  va_list fmt_args;
+  va_list fmt_args{};
   va_start(fmt_args, format);
   msg = formatMessage(format, fmt_args);
   va_end(fmt_args);
@@ -238,7 +250,7 @@ PyWarningHandler::PyWarningHandler() noexcept(true)
 }
 
 // Get the Python warning type for a warning
-PyObject* map_warning_to_python_type(const c10::Warning& warning) {
+static PyObject* map_warning_to_python_type(const c10::Warning& warning) {
   struct Visitor {
     PyObject* operator()(const c10::UserWarning&) const {
       return PyExc_UserWarning;
@@ -247,7 +259,7 @@ PyObject* map_warning_to_python_type(const c10::Warning& warning) {
       return PyExc_DeprecationWarning;
     }
   };
-  return c10::visit(Visitor(), warning.type());
+  return std::visit(Visitor(), warning.type());
 }
 
 /// See NOTE [ Conversion Cpp Python Warning ] for noexcept justification
@@ -257,8 +269,7 @@ PyWarningHandler::~PyWarningHandler() noexcept(false) {
   auto& warning_buffer = internal_handler_.warning_buffer_;
 
   if (!warning_buffer.empty()) {
-    // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-    PyObject *type, *value, *traceback;
+    PyObject *type = nullptr, *value = nullptr, *traceback = nullptr;
     pybind11::gil_scoped_acquire gil;
     auto result = 0;
     if (in_exception_) {
@@ -282,7 +293,7 @@ PyWarningHandler::~PyWarningHandler() noexcept(false) {
             /*category=*/map_warning_to_python_type(warning),
             /*message=*/msg.c_str(),
             /*filename=*/source_location.file,
-            /*lineno=*/source_location.line,
+            /*lineno=*/static_cast<int>(source_location.line),
             /*module=*/nullptr,
             /*registry=*/nullptr);
       } else {
@@ -313,7 +324,6 @@ PyWarningHandler::~PyWarningHandler() noexcept(false) {
       throw python_error();
     }
     if (in_exception_) {
-      // NOLINTNEXTLINE(clang-analyzer-core.CallAndMessage)
       PyErr_Restore(type, value, traceback);
     }
   }
