@@ -7,10 +7,7 @@
 #include <torch/nn/pimpl.h>
 #include <torch/types.h>
 
-#include <cstdint>
-
-namespace torch {
-namespace nn {
+namespace torch::nn {
 
 /// Base class for all (dimension-specialized) batchnorm and instancenorm
 /// modules.
@@ -104,11 +101,8 @@ class BatchNormImplBase : public NormImplBase<D, Derived, BatchNormOptions> {
 
   Tensor forward(const Tensor& input) {
     this->_check_input_dim(input);
-    // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-    double exponential_average_factor;
-    if (this->options.momentum() == std::nullopt) {
-      exponential_average_factor = 0.0;
-    } else {
+    double exponential_average_factor = 0.0;
+    if (this->options.momentum().has_value()) {
       exponential_average_factor = this->options.momentum().value();
     }
 
@@ -246,5 +240,4 @@ class TORCH_API BatchNorm3dImpl : public BatchNormImplBase<3, BatchNorm3dImpl> {
 /// learn about PyTorch's module storage semantics.
 TORCH_MODULE(BatchNorm3d);
 
-} // namespace nn
-} // namespace torch
+} // namespace torch::nn
