@@ -23,7 +23,7 @@ uint64_t SequenceNum::get() const {
 
 void SequenceNum::increment() {
   std::lock_guard<std::mutex> lock(lock_);
-  TORCH_CHECK(num_ != std::nullopt);
+  TORCH_CHECK(num_.has_value());
   num_ = ++(*num_);
 }
 
@@ -32,7 +32,7 @@ void SequenceNum::increment() {
 uint64_t SequenceNum::getAndIncrement() {
   uint64_t curVal = 0;
   std::lock_guard<std::mutex> lock(lock_);
-  TORCH_CHECK(num_ != std::nullopt);
+  TORCH_CHECK(num_.has_value());
   curVal = *num_;
   num_ = ++(*num_);
   return curVal;
@@ -45,7 +45,7 @@ void SequenceNum::set(const uint64_t num) {
 
 bool SequenceNum::isSet() const {
   std::lock_guard<std::mutex> lock(lock_);
-  return num_ != std::nullopt;
+  return num_.has_value();
 }
 
 SequenceNum& SequenceNum::operator=(const SequenceNum& other) {
