@@ -5,17 +5,15 @@
 #include <ATen/core/ivalue.h>
 #include <ATen/core/qualified_name.h>
 #include <c10/util/Exception.h>
-#include <c10/util/Optional.h>
 #include <c10/util/ScopeExit.h>
 #include <c10/util/irange.h>
 #include <caffe2/serialize/in_memory_adapter.h>
 #include <caffe2/serialize/inline_container.h>
-#include <caffe2/serialize/read_adapter_interface.h>
+#include <caffe2/serialize/istream_adapter.h>
 #include <caffe2/serialize/versions.h>
 #include <torch/csrc/jit/api/compilation_unit.h>
 #include <torch/csrc/jit/mobile/file_format.h>
 #include <torch/csrc/jit/mobile/flatbuffer_loader.h>
-#include <torch/csrc/jit/mobile/interpreter.h>
 #include <torch/csrc/jit/mobile/observer.h>
 #include <torch/csrc/jit/mobile/type_parser.h>
 #include <torch/csrc/jit/mobile/upgrader_mobile.h>
@@ -24,8 +22,7 @@
 #include <torch/csrc/jit/serialization/import_export_functions.h>
 #include <torch/csrc/jit/serialization/import_read.h>
 #include <torch/custom_class.h>
-#include <exception>
-#include <fstream>
+#include <optional>
 #include <string>
 #include <vector>
 
@@ -84,8 +81,7 @@
 //  - Argument::{known_length_,kwarg_only_}
 //  - FunctionSchema::{overload_name_, is_vararg_, is_varret_}
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
 using caffe2::serialize::MemoryReadAdapter;
 using caffe2::serialize::PyTorchStreamReader;
 using caffe2::serialize::ReadAdapterInterface;
@@ -94,7 +90,7 @@ OpCode parseOpCode(const char* str);
 
 TypePtr resolveTypeNameMobile(
     const c10::QualifiedName& qn,
-    std::shared_ptr<CompilationUnit> compilation_unit) {
+    const std::shared_ptr<CompilationUnit>& compilation_unit) {
   // HACK: first we check whether the name starts with special prefix to
   // tell if it's a supported pytorch class type. There are two special
   // prefixes. "__torch__" for nn module, and "torch.jit" from to_backend.
@@ -117,7 +113,7 @@ TypePtr resolveTypeNameMobile(
 
 c10::StrongTypePtr typeResolverMobile(
     const c10::QualifiedName& qn,
-    std::shared_ptr<CompilationUnit> compilation_unit) {
+    const std::shared_ptr<CompilationUnit>& compilation_unit) {
   return c10::StrongTypePtr(
       compilation_unit, resolveTypeNameMobile(qn, compilation_unit));
 }
@@ -149,7 +145,7 @@ c10::intrusive_ptr<c10::ivalue::Object> objLoaderMobile(
     custom_class_type->getMethod("__setstate__").run(stack);
     return obj;
   } else {
-    auto dict = std::move(input).toGenericDict();
+    auto dict = input.toGenericDict();
     size_t ndict = dict.size();
     auto obj = c10::ivalue::Object::create(type, ndict);
     auto it = dict.begin();
@@ -194,12 +190,12 @@ class BytecodeDeserializer final {
   explicit BytecodeDeserializer(
       std::unique_ptr<PyTorchStreamReader> reader,
       uint64_t module_load_options = 0);
-  mobile::Module deserialize(c10::optional<at::Device> device);
+  mobile::Module deserialize(std::optional<at::Device> device);
   mobile::Module deserialize(
-      c10::optional<at::Device> device,
+      std::optional<at::Device> device,
       ExtraFilesMap& extra_files);
   void deserialize_only_extra(
-      c10::optional<at::Device> device,
+      std::optional<at::Device> device,
       ExtraFilesMap& extra_files);
 
  private:
@@ -207,7 +203,7 @@ class BytecodeDeserializer final {
   void init_upgrader(mobile::Function* function);
   void parseMethods(
       c10::ivalue::TupleElements&& vals,
-      c10::optional<c10::ivalue::TupleElements>&& debug_handles,
+      std::optional<c10::ivalue::TupleElements>&& debug_handles,
       mobile::CompilationUnit& mcu);
   c10::IValue readArchive(
       const std::string& archive_name,
@@ -220,14 +216,14 @@ class BytecodeDeserializer final {
   std::shared_ptr<CompilationUnit> compilation_unit_;
   std::unordered_set<std::string> imported_libs_;
   std::unique_ptr<PyTorchStreamReader> reader_{};
-  c10::optional<at::Device> device_;
+  std::optional<at::Device> device_;
   uint64_t module_load_options_;
   // From `version` or `.data/version` in model.ptl and it's compute
   // dynamically. It's used for finding the minimum required runtime to run all
   // operators from the given model. If it's less than the current runtime,
   // upgrader will be applied at loading stage.
-  uint64_t operator_version_;
-  uint64_t bytecode_version_;
+  uint64_t operator_version_{0};
+  uint64_t bytecode_version_{0};
 };
 
 BytecodeDeserializer::BytecodeDeserializer(
@@ -270,7 +266,7 @@ void BytecodeDeserializer::parseFunctionSchema(
         args.emplace_back(
             name,
             std::move(type),
-            c10::nullopt /*N*/,
+            std::nullopt /*N*/,
             std::move(default_value));
       }
       tryRegisterMethod(args, *function);
@@ -308,7 +304,7 @@ void BytecodeDeserializer::init_upgrader(mobile::Function* function) {
 
 void BytecodeDeserializer::parseMethods(
     c10::ivalue::TupleElements&& vals,
-    c10::optional<c10::ivalue::TupleElements>&& debug_handles,
+    std::optional<c10::ivalue::TupleElements>&& debug_handles,
     mobile::CompilationUnit& mcu) {
   TORCH_CHECK(!vals.empty(), "Bytecode has no elements. ");
   // Initialized with the version number when kProducedBytecodeVersion was
@@ -322,9 +318,7 @@ void BytecodeDeserializer::parseMethods(
     method_i_start = 1;
   }
   TORCH_CHECK(
-      // NOLINTNEXTLINE(clang-diagnostic-sign-compare)
       caffe2::serialize::kMinSupportedBytecodeVersion <= bytecode_version_ &&
-          // NOLINTNEXTLINE(clang-diagnostic-sign-compare)
           bytecode_version_ <= caffe2::serialize::kMaxSupportedBytecodeVersion,
       "Lite Interpreter version number does not match. ",
       "The model version must be between ",
@@ -345,8 +339,7 @@ void BytecodeDeserializer::parseMethods(
     auto element = std::move(vals[i]);
     auto m_tuple = std::move(element.toTupleRef()).elements();
     const std::string& function_name = m_tuple[0].toStringRef();
-    auto codeTableElements =
-        std::move(std::move(m_tuple[1]).toTupleRef()).elements();
+    auto codeTableElements = std::move(m_tuple[1].toTupleRef()).elements();
     IValue* schemaTable = // older files do not store function schema
         (bytecode_version_ > 0x4L ||
          (bytecode_version_ == 0x4L && m_tuple.size() >= 3))
@@ -420,15 +413,13 @@ void BytecodeDeserializer::parseMethods(
 }
 
 void BytecodeDeserializer::deserialize_only_extra(
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     ExtraFilesMap& extra_files) {
   device_ = device;
   for (const auto& kv : extra_files) {
     const std::string& key = "extra/" + kv.first;
     if (reader_->hasRecord(key)) {
-      at::DataPtr meta_ptr;
-      size_t meta_size = 0;
-      std::tie(meta_ptr, meta_size) = reader_->getRecord(key);
+      auto [meta_ptr, meta_size] = reader_->getRecord(key);
       extra_files[kv.first] =
           std::string(static_cast<char*>(meta_ptr.get()), meta_size);
     }
@@ -436,14 +427,14 @@ void BytecodeDeserializer::deserialize_only_extra(
 }
 
 mobile::Module BytecodeDeserializer::deserialize(
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     ExtraFilesMap& extra_files) {
   deserialize_only_extra(device, extra_files);
   return deserialize(device);
 }
 
 mobile::Module BytecodeDeserializer::deserialize(
-    c10::optional<at::Device> device) {
+    std::optional<at::Device> device) {
   device_ = device;
   auto mcu = std::make_shared<mobile::CompilationUnit>();
 
@@ -458,7 +449,7 @@ mobile::Module BytecodeDeserializer::deserialize(
   //
   auto bvals = std::move(readArchive("bytecode", mcu).toTupleRef()).elements();
 
-  c10::optional<c10::ivalue::TupleElements> debug_handles;
+  std::optional<c10::ivalue::TupleElements> debug_handles;
   bool has_debug_handles{false};
   if (reader_->hasRecord("mobile_debug_handles.pkl")) {
     debug_handles =
@@ -486,13 +477,12 @@ c10::IValue BytecodeDeserializer::readArchive(
     return typeResolverMobile(qn, compilation_unit_);
   };
 
-  auto obj_loader = [&](at::StrongTypePtr type, IValue input) {
+  auto obj_loader = [&](const at::StrongTypePtr& type, const IValue& input) {
     return objLoaderMobile(type, input, *mcu);
   };
 
   bool bytecode_tensor_in_constants_archive =
-      (archive_name == "bytecode" &&
-       !isTensorInBytecodeArchive(*reader_.get()));
+      (archive_name == "bytecode" && !isTensorInBytecodeArchive(*reader_));
 
   auto ivalues = torch::jit::readArchiveAndTensors(
       archive_name,
@@ -502,135 +492,14 @@ c10::IValue BytecodeDeserializer::readArchive(
       type_resolver,
       obj_loader,
       device_,
-      *reader_.get(),
+      *reader_,
       nullptr);
   return ivalues;
 }
 
-} // namespace
-
-// Forward declare so that _load_for_mobile() overloads can
-// call this method directly.
 mobile::Module _load_for_mobile_impl(
     std::unique_ptr<ReadAdapterInterface> rai,
-    c10::optional<c10::Device> device,
-    ExtraFilesMap& extra_files,
-    uint64_t module_load_options);
-
-mobile::Module _load_mobile_from_bytes(
-    std::shared_ptr<char> data,
-    size_t size,
-    c10::optional<c10::Device> device,
-    ExtraFilesMap& extra_files,
-    uint64_t module_load_options);
-
-mobile::Module _load_for_mobile(
-    std::istream& in,
-    c10::optional<at::Device> device) {
-  ExtraFilesMap extra_files;
-  return _load_for_mobile(in, device, extra_files);
-}
-
-mobile::Module _load_for_mobile(
-    const std::string& filename,
-    c10::optional<at::Device> device) {
-  ExtraFilesMap extra_files;
-  return _load_for_mobile(filename, device, extra_files);
-}
-
-mobile::Module _load_for_mobile(
-    std::unique_ptr<ReadAdapterInterface> rai,
-    c10::optional<c10::Device> device) {
-  ExtraFilesMap extra_files;
-  return _load_for_mobile(std::move(rai), device, extra_files);
-}
-
-mobile::Module _load_for_mobile(
-    std::istream& in,
-    c10::optional<at::Device> device,
-    ExtraFilesMap& extra_files) {
-  if (getFileFormat(in) == FileFormat::FlatbufferFileFormat) {
-    std::shared_ptr<char> data;
-    size_t size = 0;
-    std::tie(data, size) = get_stream_content(in);
-    return _load_mobile_from_bytes(
-        data, size, device, extra_files, kDefaultMobileLoadOptions);
-  }
-  std::unique_ptr<IStreamAdapter> rai = std::make_unique<IStreamAdapter>(&in);
-  auto module = _load_for_mobile_impl(
-      std::move(rai), device, extra_files, kDefaultMobileLoadOptions);
-  return module;
-}
-
-mobile::Module _load_for_mobile(
-    const std::string& filename,
-    c10::optional<at::Device> device,
-    ExtraFilesMap& extra_files) {
-  return _load_for_mobile(
-      filename, device, extra_files, kDefaultMobileLoadOptions);
-}
-
-mobile::Module _load_for_mobile(
-    const std::string& filename,
-    c10::optional<at::Device> device,
-    ExtraFilesMap& extra_files,
-    uint64_t module_load_options) {
-  auto format = getFileFormat(filename);
-
-  if (format == FileFormat::FlatbufferFileFormat) {
-    std::shared_ptr<char> data;
-    size_t size = 0;
-    std::tie(data, size) = get_file_content(filename.c_str());
-    return _load_mobile_from_bytes(
-        data, size, device, extra_files, module_load_options);
-  }
-
-  std::unique_ptr<FileAdapter> rai = std::make_unique<FileAdapter>(filename);
-  return _load_for_mobile_impl(
-      std::move(rai), device, extra_files, module_load_options);
-}
-
-TORCH_API mobile::Module _load_for_mobile(
-    std::unique_ptr<ReadAdapterInterface> rai,
-    c10::optional<c10::Device> device,
-    ExtraFilesMap& extra_files,
-    uint64_t module_load_options) {
-  // TODO optimize file read for non-flatbuffer models
-  std::shared_ptr<char> data;
-  size_t size = 0;
-  std::tie(data, size) = get_rai_content(rai.get());
-  return _load_mobile_from_bytes(
-      data, size, device, extra_files, module_load_options);
-}
-
-mobile::Module _load_mobile_from_bytes(
-    std::shared_ptr<char> data,
-    size_t size,
-    c10::optional<c10::Device> device,
-    ExtraFilesMap& extra_files,
-    uint64_t module_load_options) {
-  TORCH_CHECK(size >= kFileFormatHeaderSize, "Format error");
-  auto format = getFileFormat(data.get());
-  switch (format) {
-    case FileFormat::ZipFileFormat: {
-      std::unique_ptr<ReadAdapterInterface> rai =
-          std::make_unique<MemoryReadAdapter>(data.get(), size);
-      return _load_for_mobile_impl(
-          std::move(rai), device, extra_files, module_load_options);
-    }
-    case FileFormat::FlatbufferFileFormat: {
-      return parse_and_initialize_mobile_module(
-          data, size, device, &extra_files);
-    }
-    default: {
-      TORCH_CHECK(false, "Format error");
-    }
-  }
-}
-
-mobile::Module _load_for_mobile_impl(
-    std::unique_ptr<ReadAdapterInterface> rai,
-    c10::optional<c10::Device> device,
+    std::optional<c10::Device> device,
     ExtraFilesMap& extra_files,
     uint64_t module_load_options) {
   auto observer = torch::observerConfig().getModuleObserver();
@@ -648,7 +517,22 @@ mobile::Module _load_for_mobile_impl(
   }
 
   const size_t model_size = rai != nullptr ? rai->size() : 0;
-  auto reader = torch::make_unique<PyTorchStreamReader>(std::move(rai));
+  auto reader = std::make_unique<PyTorchStreamReader>(std::move(rai));
+  if (module_load_options &
+      MobileModuleLoadOptions::PARSE_ALL_EXTRA_FILE_MAPS) {
+    // ExtraFilesMap is serialized with a "extra/", hence it is necessary to
+    // account for when we de-serialize de-serialized filemap key values contain
+    // prefix and we need to remove prior to construct the map. "extra/" string
+    // has a length of 6 characters, hence we need only sub-string 6th position
+    // of a string. Please refer to following link for a detail:
+    // https://www.internalfb.com/code/fbsource/[9996fcb7a6fb]/fbcode/caffe2/torch/csrc/jit/mobile/import.cpp?lines=427-434
+    std::vector<std::string> all_files = reader->getAllRecords();
+    for (auto& file_name : all_files) {
+      if (file_name.find("extra/") == 0) {
+        extra_files[file_name.substr(6)] = "";
+      }
+    }
+  }
   BytecodeDeserializer deserializer(std::move(reader), module_load_options);
 
   std::string error_message;
@@ -672,7 +556,7 @@ mobile::Module _load_for_mobile_impl(
       // Add model_name and model_size to metadata_map
       extra_files.insert(std::make_pair("model_name", result.name()));
       extra_files.insert(
-          std::make_pair("model_size", c10::guts::to_string(model_size)));
+          std::make_pair("model_size", std::to_string(model_size)));
       metadata_map = observer->processMetadataFromExtra(extra_files);
       observer->onExitLoadModel(instance_key, metadata_map);
     }
@@ -685,9 +569,114 @@ mobile::Module _load_for_mobile_impl(
   }
 }
 
+mobile::Module _load_mobile_from_bytes(
+    const std::shared_ptr<char>& data,
+    size_t size,
+    std::optional<c10::Device> device,
+    ExtraFilesMap& extra_files,
+    uint64_t module_load_options) {
+  TORCH_CHECK(size >= kFileFormatHeaderSize, "Format error");
+  auto format = getFileFormat(data.get());
+  switch (format) {
+    case FileFormat::ZipFileFormat: {
+      std::unique_ptr<ReadAdapterInterface> rai =
+          std::make_unique<MemoryReadAdapter>(data.get(), size);
+      return _load_for_mobile_impl(
+          std::move(rai), device, extra_files, module_load_options);
+    }
+    case FileFormat::FlatbufferFileFormat: {
+      return parse_and_initialize_mobile_module(
+          data, size, device, &extra_files);
+    }
+    default: {
+      TORCH_CHECK(false, "Format error");
+    }
+  }
+}
+
+} // namespace
+
+mobile::Module _load_for_mobile(
+    std::istream& in,
+    std::optional<at::Device> device) {
+  ExtraFilesMap extra_files;
+  return _load_for_mobile(in, device, extra_files);
+}
+
+mobile::Module _load_for_mobile(
+    const std::string& filename,
+    std::optional<at::Device> device) {
+  ExtraFilesMap extra_files;
+  return _load_for_mobile(filename, device, extra_files);
+}
+
+mobile::Module _load_for_mobile(
+    std::unique_ptr<ReadAdapterInterface> rai,
+    std::optional<c10::Device> device) {
+  ExtraFilesMap extra_files;
+  return _load_for_mobile(std::move(rai), device, extra_files);
+}
+
+mobile::Module _load_for_mobile(
+    std::istream& in,
+    std::optional<at::Device> device,
+    ExtraFilesMap& extra_files,
+    uint64_t module_load_options) {
+  if (getFileFormat(in) == FileFormat::FlatbufferFileFormat) {
+    auto [data, size] = get_stream_content(in);
+    return _load_mobile_from_bytes(
+        data, size, device, extra_files, module_load_options);
+  }
+  auto rai = std::make_unique<caffe2::serialize::IStreamAdapter>(&in);
+  auto module = _load_for_mobile_impl(
+      std::move(rai), device, extra_files, module_load_options);
+  return module;
+}
+
+mobile::Module _load_for_mobile(
+    const std::string& filename,
+    std::optional<at::Device> device,
+    ExtraFilesMap& extra_files) {
+  return _load_for_mobile(
+      filename, device, extra_files, kDefaultMobileLoadOptions);
+}
+
+mobile::Module _load_for_mobile(
+    const std::string& filename,
+    std::optional<at::Device> device,
+    ExtraFilesMap& extra_files,
+    uint64_t module_load_options) {
+  auto observer = torch::observerConfig().getModuleObserver();
+  if (observer) {
+    extra_files.insert(std::make_pair("model_path", filename));
+  }
+  auto format = getFileFormat(filename);
+
+  if (format == FileFormat::FlatbufferFileFormat) {
+    auto [data, size] = get_file_content(filename.c_str());
+    return _load_mobile_from_bytes(
+        data, size, device, extra_files, module_load_options);
+  }
+
+  auto rai = std::make_unique<caffe2::serialize::FileAdapter>(filename);
+  return _load_for_mobile_impl(
+      std::move(rai), device, extra_files, module_load_options);
+}
+
+TORCH_API mobile::Module _load_for_mobile(
+    std::unique_ptr<ReadAdapterInterface> rai,
+    std::optional<c10::Device> device,
+    ExtraFilesMap& extra_files,
+    uint64_t module_load_options) {
+  // TODO optimize file read for non-flatbuffer models
+  auto [data, size] = get_rai_content(rai.get());
+  return _load_mobile_from_bytes(
+      data, size, device, extra_files, module_load_options);
+}
+
 void _load_extra_only_for_mobile(
     const std::string& filename,
-    c10::optional<at::Device> device,
+    std::optional<at::Device> device,
     ExtraFilesMap& extra_files) {
   auto observer = torch::observerConfig().getModuleObserver();
   // NOLINTNEXTLINE(clang-analyzer-security.insecureAPI.rand)
@@ -699,9 +688,8 @@ void _load_extra_only_for_mobile(
   auto format = getFileFormat(filename);
   switch (format) {
     case FileFormat::ZipFileFormat: {
-      std::unique_ptr<FileAdapter> rai =
-          std::make_unique<FileAdapter>(filename);
-      auto reader = torch::make_unique<PyTorchStreamReader>(std::move(rai));
+      auto rai = std::make_unique<caffe2::serialize::FileAdapter>(filename);
+      auto reader = std::make_unique<PyTorchStreamReader>(std::move(rai));
       BytecodeDeserializer deserializer(std::move(reader));
       deserializer.deserialize_only_extra(device, extra_files);
       break;
@@ -710,7 +698,7 @@ void _load_extra_only_for_mobile(
       // TODO: the current flatbuffers implementation will always load the
       // whole module including the extra files. Ideally it should be
       // possible to just get the extra files given data
-      load_mobile_module_from_file(filename, c10::nullopt, &extra_files);
+      load_mobile_module_from_file(filename, std::nullopt, &extra_files);
       break;
     }
     default: {
@@ -740,5 +728,4 @@ std::set<std::string> _export_operator_list(
 }
 
 } // namespace mobile
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
