@@ -13,20 +13,18 @@
 #include <torch/csrc/jit/passes/remove_mutation.h>
 
 #include <memory>
-#include <regex>
 #include <stack>
 #include <string>
 #include <utility>
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
 
-using ModuleQConfigMap = std::unordered_map<ModulePtr, c10::optional<QConfig>>;
+using ModuleQConfigMap = std::unordered_map<ModulePtr, std::optional<QConfig>>;
 
 namespace {
 
 struct OptionalQConfigHash {
-  inline size_t operator()(const c10::optional<QConfig>& qconfig_opt) const {
+  inline size_t operator()(const std::optional<QConfig>& qconfig_opt) const {
     if (qconfig_opt.has_value()) {
       const auto& m1 = std::get<0>(*qconfig_opt);
       const auto& m2 = std::get<1>(*qconfig_opt);
@@ -37,9 +35,9 @@ struct OptionalQConfigHash {
   }
 };
 using QConfigTypePtrMap =
-    std::unordered_map<c10::optional<QConfig>, TypePtr, OptionalQConfigHash>;
+    std::unordered_map<std::optional<QConfig>, TypePtr, OptionalQConfigHash>;
 using NameModuleVector = std::vector<std::pair<std::string, Module>>;
-using OptionalModuleVector = std::vector<c10::optional<Module>>;
+using OptionalModuleVector = std::vector<std::optional<Module>>;
 using ModuleMethodVector = std::vector<std::pair<Module, std::string>>;
 using graph_rewrite_helper::PatternInfo;
 using graph_rewrite_helper::replaceConvolutionWithAtenConv;
@@ -50,8 +48,8 @@ void fillQConfigMap(
     const QConfigDict& qconfig_dict,
     ModuleQConfigMap& map,
     const std::string& key = "",
-    const c10::optional<QConfig>& parent_qconfig = c10::nullopt) {
-  c10::optional<QConfig> qconfig;
+    const std::optional<QConfig>& parent_qconfig = std::nullopt) {
+  std::optional<QConfig> qconfig;
   if (qconfig_dict.find(key) != qconfig_dict.end()) {
     GRAPH_DEBUG("Got module config for key:", key);
     qconfig = qconfig_dict.at(key);
@@ -93,7 +91,7 @@ class ModuleCloneHelper {
       const ModuleQConfigMap& module_qconfig_map,
       bool inplace = false) {
     std::unordered_map<TypePtr, QConfigTypePtrMap> type_remap;
-    IValue::HashAliasedIValueMap memo;
+    IValue::HashIdentityIValueMap memo;
     return clone_impl(
         module, module_qconfig_map, type_remap, inplace, std::move(memo));
   }
@@ -104,7 +102,7 @@ class ModuleCloneHelper {
       const ModuleQConfigMap& module_qconfig_map,
       std::unordered_map<TypePtr, QConfigTypePtrMap>& type_remap,
       bool inplace,
-      IValue::HashAliasedIValueMap memo) {
+      IValue::HashIdentityIValueMap memo) {
     auto qconfig = module_qconfig_map.at(module._ivalue());
     auto type = module.type();
     // Create a new _ivalue in the same compilation unit.
@@ -188,7 +186,7 @@ class ModuleCloneHelper {
       const Module& source,
       Module& target,
       const ModuleQConfigMap& module_qconfig_map,
-      const std::function<TypePtr(TypePtr, c10::optional<QConfig>)>&
+      const std::function<TypePtr(TypePtr, std::optional<QConfig>)>&
           type_remap_fn) {
     // remap of %self will be done outside of the function
     // and we don't support the case when people pass in
@@ -240,7 +238,7 @@ class ModuleCloneHelper {
       const Module& source,
       Module& target,
       const ModuleQConfigMap& module_qconfig_map,
-      const std::function<TypePtr(TypePtr, c10::optional<QConfig>)>&
+      const std::function<TypePtr(TypePtr, std::optional<QConfig>)>&
           type_remap_fn) {
     remapTypes(
         graph->block(),
@@ -258,7 +256,7 @@ class ModuleCloneHelper {
       const ModuleQConfigMap& module_qconfig_map,
       const std::unordered_map<TypePtr, QConfigTypePtrMap>& type_remap) {
     auto type_remap_fn = [&](TypePtr type_ptr,
-                             const c10::optional<QConfig>& qconfig) {
+                             const std::optional<QConfig>& qconfig) {
       if (type_remap.find(type_ptr) != type_remap.end()) {
         const auto& qconfig_map = type_remap.at(type_ptr);
         if (qconfig_map.find(qconfig) != qconfig_map.end()) {
@@ -316,7 +314,7 @@ class InsertObserversHelper {
    * observe/quantize a value a not, we don't want to observe a value multiple
    * times.
    *
-   * arguemnt: is_entry_point means whether the current method is the forward
+   * argument: is_entry_point means whether the current method is the forward
    * method of the top level module.
    *
    * Since we want to insert observers in the call site instead of in the called
@@ -402,9 +400,9 @@ class InsertObserversHelper {
 
   // Uses the state created by fillBoundaryValueMap and fillValueObserverMap
   // to return an observer configured for a value, if it is needed.
-  c10::optional<Module> getObserverFor(Value* v);
+  std::optional<Module> getObserverFor(Value* v);
 
-  // Uses the state created by fillPassThroughValueMap to propage observed
+  // Uses the state created by fillPassThroughValueMap to propagage observed
   // property which should pass through from inputs to outputs.
   void propagateObservedProperty(
       Value* output,
@@ -954,9 +952,9 @@ void InsertObserversHelper::insertObserverFor(
   }
   GRAPH_DEBUG("Inserting observer for:", v->debugName());
   Module observer = observer_module.deepcopy();
-  std::string observer_name = "_observer_" + c10::to_string(uid_++);
+  std::string observer_name = "_observer_" + std::to_string(uid_++);
   while (module.hasattr(observer_name)) {
-    observer_name = "_observer_" + c10::to_string(uid_++);
+    observer_name = "_observer_" + std::to_string(uid_++);
   }
   module.register_module(observer_name, observer);
   observer_name_and_modules.emplace_back(observer_name, observer);
@@ -1121,8 +1119,7 @@ void InsertObserversHelper::fillBoundaryValueMap(
         // offset of input for the caller node, since the first
         // input of CallFunction is the function node and the graph
         // for CallFunction start with actual input
-        // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-        size_t input_offset;
+        size_t input_offset = 0;
         if (n->kind() == prim::CallMethod) {
           auto m_opt = getInvokedModuleOpt(module, n, self);
           if (!m_opt.has_value()) {
@@ -1313,13 +1310,13 @@ void InsertObserversHelper::fillValueObserverMap(
   }
 }
 
-c10::optional<Module> InsertObserversHelper::getObserverFor(Value* v) {
+std::optional<Module> InsertObserversHelper::getObserverFor(Value* v) {
   if (observer_for_value_.count(v)) {
     auto observer = observer_for_value_.at(v);
     GRAPH_DEBUG("Got observer module config for:", v->debugName());
     return observer;
   }
-  c10::optional<Module> result;
+  std::optional<Module> result;
   if (boundary_value_map_.count(v)) {
     for (Value* next : boundary_value_map_.at(v)) {
       GRAPH_DEBUG(
@@ -1385,9 +1382,9 @@ InsertObserversHelper::insertObserversFor(
   // the graph itself can be shared
   std::unordered_set<Value*> inputs_outputs;
   // list of observer modules for input values
-  std::vector<c10::optional<Module>> block_input_observers;
+  std::vector<std::optional<Module>> block_input_observers;
   // list of observer modules for output values
-  std::vector<c10::optional<Module>> block_output_observers;
+  std::vector<std::optional<Module>> block_output_observers;
 
   // if the current block is the block for entry point graph(the forward graph
   // of the top level module), we can insert observers in the block directly
@@ -1409,13 +1406,13 @@ InsertObserversHelper::insertObserversFor(
     }
 
     for (auto* v : block->outputs()) {
-      // we need explictly skip the values that are already observed
+      // we need explicitly skip the values that are already observed
       // this might happen in subblocks for `if` since
       // these subblock has access to all values before the `if` node
       if (!isObserved(v, block_observed_values)) {
         block_output_observers.emplace_back(getObserverFor(v));
       } else {
-        block_output_observers.emplace_back(c10::nullopt);
+        block_output_observers.emplace_back(std::nullopt);
       }
     }
   }
@@ -1470,8 +1467,7 @@ InsertObserversHelper::insertObserversFor(
       if (n->kind() == prim::CallMethod || userDefinedCallFunction(n)) {
         script::Module m;
         std::shared_ptr<Graph> g;
-        // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-        size_t input_offset;
+        size_t input_offset = 0;
         bool is_udf_for_subblock = is_user_defined_function;
         if (n->kind() == prim::CallMethod) {
           auto m_opt = getInvokedModuleOpt(module, n, self);
@@ -1706,7 +1702,7 @@ Module InsertObserversForOnDevicePTQ(
   // you will have multiple getattrs for the same attribute and thus potentially
   // multiple observers observing the same value. This will also lead to
   // increased size of the packed param struct. I dont expect this to be a
-  // commong pattern but something to be aware fo Note that current quant
+  // common pattern but something to be aware fo Note that current quant
   // workflow does not prevent this anyway since during inset quant dequant
   // things are inlined anyway
   helper.fillBoundaryValueMap(cloned_module, observer_method_name);
@@ -1723,5 +1719,4 @@ Module InsertObserversForOnDevicePTQ(
       cloned_module, observer_method_name, /* is_entry_point */ true);
   return cloned_module;
 }
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
