@@ -131,24 +131,24 @@ void ConcretePythonOp::cloneFrom(Node* other_) {
 // recover the autograd.Function instance, if this PythonOp's function
 // was originally SomeFunction.apply
 // used in ONNX for discovering symbolics
-c10::optional<THPObjectPtr> ConcretePythonOp::autogradFunction() const {
+std::optional<THPObjectPtr> ConcretePythonOp::autogradFunction() const {
   pybind11::gil_scoped_acquire gil;
   // NOLINTNEXTLINE(cppcoreguidelines-pro-type-const-cast)
   py::handle obj = const_cast<PyObject*>(pyobj.get());
 
   auto r = py::getattr(obj, "__self__", py::none());
   if (r.is_none())
-    return c10::nullopt;
+    return std::nullopt;
 
   auto apply = py::getattr(r, "apply", py::none());
   if (apply.is_none())
-    return c10::nullopt;
+    return std::nullopt;
 
   auto c = PyObject_RichCompareBool(apply.ptr(), obj.ptr(), Py_NE);
   if (PyErr_Occurred())
     throw py::error_already_set();
   if (c)
-    return c10::nullopt;
+    return std::nullopt;
 
   return THPObjectPtr(r.release().ptr());
 }
@@ -259,31 +259,26 @@ void initPythonIRBindings(PyObject* module_) {
              const std::string& onnx_file_path,
              const NodeAttrNameMap& node_attr_to_name) {
             std::string graph;
-            std::shared_ptr<::ONNX_NAMESPACE::ModelProto> model_proto;
-            RawDataExportMap export_map;
-            SymbolDimMap symbol_map;
-            bool val_use_external_data_format = false;
-            NodeNameMap onnx_node_names;
-            std::tie(
-                model_proto,
-                export_map,
-                symbol_map,
-                val_use_external_data_format,
-                onnx_node_names) =
-                export_onnx(
-                    g,
-                    initializers,
-                    onnx_opset_version,
-                    dynamic_axes,
-                    defer_weight_export,
-                    operator_export_type,
-                    strip_doc_string,
-                    keep_initializers_as_inputs,
-                    custom_opsets,
-                    add_node_names,
-                    val_use_external_data_format,
-                    onnx_file_path,
-                    node_attr_to_name);
+            auto
+                [model_proto,
+                 export_map,
+                 symbol_map,
+                 val_use_external_data_format,
+                 onnx_node_names] =
+                    export_onnx(
+                        g,
+                        initializers,
+                        onnx_opset_version,
+                        dynamic_axes,
+                        defer_weight_export,
+                        operator_export_type,
+                        strip_doc_string,
+                        keep_initializers_as_inputs,
+                        custom_opsets,
+                        add_node_names,
+                        false,
+                        onnx_file_path,
+                        node_attr_to_name);
             std::unordered_map<std::string, py::bytes>
                 python_serialized_export_map;
             for (auto& kv : export_map) {
@@ -464,20 +459,20 @@ void initPythonIRBindings(PyObject* module_) {
       .def("setInsertPoint", [](Graph& g, Block* n) { g.setInsertPoint(n); })
       .def(
           "insertGraph",
-          [](Graph& g, Graph& callee, std::vector<Value*> inputs) {
+          [](Graph& g, Graph& callee, const std::vector<Value*>& inputs) {
             return insertGraph(g, callee, inputs);
           })
       .def(
           "insertGraph",
           [](Graph& g,
              Graph& callee,
-             std::vector<Value*> inputs,
+             const std::vector<Value*>& inputs,
              std::unordered_map<Value*, Value*> value_map) {
             return insertGraph(g, callee, inputs, value_map);
           })
       .def(
           "insert",
-          [](Graph& g, Symbol opname, std::vector<Value*> args) {
+          [](Graph& g, Symbol opname, const std::vector<Value*>& args) {
             std::vector<NamedValue> args_named;
             args_named.reserve(args.size());
             for (Value* v : args) {
@@ -490,7 +485,9 @@ void initPythonIRBindings(PyObject* module_) {
           [](Graph& g) {
             auto tup = g.createTuple(g.outputs());
             tup->insertBefore(g.return_node());
-            for (int64_t i = g.outputs().size() - 1; i >= 0; i--) {
+            for (int64_t i = static_cast<int64_t>(g.outputs().size()) - 1;
+                 i >= 0;
+                 i--) {
               g.eraseOutput(0);
             }
             g.registerOutput(tup->output());
@@ -870,7 +867,7 @@ void initPythonIRBindings(PyObject* module_) {
           })
       .def(
           "with_sizes",
-          [](Type& t, c10::optional<std::vector<c10::optional<int64_t>>> sizes)
+          [](Type& t, std::optional<std::vector<std::optional<int64_t>>> sizes)
               -> py::object {
             auto ptt = t.expect<TensorType>();
             if (!ptt) {
@@ -927,7 +924,8 @@ void initPythonIRBindings(PyObject* module_) {
       .def(
           "with_device",
           [](Type& t, py::object device) -> py::object {
-            at::Device c_device = python::detail::py_object_to_device(device);
+            at::Device c_device =
+                python::detail::py_object_to_device(std::move(device));
             if (auto ptt = t.expect<TensorType>()) {
               return py::cast(ptt->withDevice(c_device));
             }
@@ -949,7 +947,7 @@ void initPythonIRBindings(PyObject* module_) {
           "with_dtype",
           [](Type& t, py::object dtype) -> py::object {
             at::ScalarType scalar_type =
-                python::detail::py_object_to_dtype(dtype);
+                python::detail::py_object_to_dtype(std::move(dtype));
 
             if (auto ptt = t.expect<TensorType>()) {
               // auto scalar_type = dtype->scalar_type;
@@ -1012,6 +1010,10 @@ void initPythonIRBindings(PyObject* module_) {
       .def_static("get", &StringType::get);
   py::class_<DeviceObjType, Type, DeviceObjTypePtr>(m, "DeviceObjType")
       .def_static("get", &DeviceObjType::get);
+  // TODO(antoniojkim): Add GeneratorType to the public API once its been added
+  //                    to the public documentation
+  py::class_<GeneratorType, Type, GeneratorTypePtr>(m, "_GeneratorType")
+      .def_static("get", &GeneratorType::get);
   py::class_<StreamObjType, Type, StreamObjTypePtr>(m, "StreamObjType")
       .def_static("get", &StreamObjType::get);
   py::class_<PyObjectType, Type, PyObjectTypePtr>(m, "PyObjectType")
@@ -1034,7 +1036,7 @@ void initPythonIRBindings(PyObject* module_) {
       .def(py::init(
           [](const std::vector<TypePtr>& a) { return UnionType::create(a); }));
   py::class_<ListType, Type, ListTypePtr>(m, "ListType")
-      .def(py::init([](TypePtr a) { return ListType::create(a); }))
+      .def(py::init([](const TypePtr& a) { return ListType::create(a); }))
       .def_static("ofInts", &ListType::ofInts)
       .def_static("ofTensors", &ListType::ofTensors)
       .def_static("ofFloats", &ListType::ofFloats)
@@ -1049,8 +1051,7 @@ void initPythonIRBindings(PyObject* module_) {
       .def("getKeyType", &DictType::getKeyType)
       .def("getValueType", &DictType::getValueType);
   py::class_<OptionalType, Type, OptionalTypePtr>(m, "OptionalType")
-      .def(py::init(
-          [](TypePtr a) { return OptionalType::create(std::move(a)); }))
+      .def(py::init([](const TypePtr& a) { return OptionalType::create(a); }))
       .def_static("ofTensor", &OptionalType::ofTensor)
       .def("getElementType", &OptionalType::getElementType);
   py::class_<RRefType, Type, RRefTypePtr>(m, "RRefType")
@@ -1070,8 +1071,15 @@ void initPythonIRBindings(PyObject* module_) {
         return get_python_cu()->get_class(c10::QualifiedName(qualified_name));
       }))
       .def("name", [](ClassType& self) { return self.name()->name(); })
-      .def("qualified_name", [](ClassType& self) {
-        return self.name()->qualifiedName();
+      .def(
+          "qualified_name",
+          [](ClassType& self) { return self.name()->qualifiedName(); })
+      .def("method_names", [](ClassType& self) {
+        std::vector<std::string> method_names;
+        for (const auto* method : self.methods()) {
+          method_names.push_back(method->name());
+        }
+        return method_names;
       });
   py::class_<EnumType, Type, EnumTypePtr>(m, "EnumType")
       .def(py::init([](const std::string& qualified_name,
