@@ -2,6 +2,7 @@
 
 #include <memory>
 #include <unordered_map>
+#include <utility>
 #include <vector>
 
 #include <c10/util/irange.h>
@@ -14,7 +15,7 @@
 #include <torch/csrc/jit/runtime/instruction.h>
 #include <torch/csrc/jit/runtime/interpreter/preprocess_graph.h>
 
-C10_DECLARE_bool(torch_jit_enable_expanded_stacks);
+TORCH_DECLARE_bool(torch_jit_enable_expanded_stacks);
 
 namespace torch::jit {
 
@@ -945,7 +946,11 @@ struct MobileCodeImpl : CodeImpl {
       bool support_default_args_before_out,
       bool emit_promoted_ops,
       size_t remaining_bailout_depth)
-      : CodeImpl(graph, function_name, remaining_bailout_depth, false),
+      : CodeImpl(
+            graph,
+            std::move(function_name),
+            remaining_bailout_depth,
+            false),
         emit_default_input_instructions_(emit_default_input_instructions),
         support_default_args_before_out_(support_default_args_before_out),
         emit_promoted_ops_(emit_promoted_ops) {
