@@ -1,11 +1,14 @@
 #include <torch/csrc/jit/tensorexpr/registerizer.h>
+#include <iostream>
 
 namespace torch::jit::tensorexpr {
 namespace registerizer {
 
 // AccessInfo
 
-void AccessInfo::addStore(StorePtr store, const std::shared_ptr<Scope>& scope) {
+void AccessInfo::addStore(
+    const StorePtr& store,
+    const std::shared_ptr<Scope>& scope) {
   block_ =
       block_ ? Block::getSharedParent(block_, scope->block()) : scope->block();
 
@@ -24,9 +27,9 @@ void AccessInfo::addStore(StorePtr store, const std::shared_ptr<Scope>& scope) {
 }
 
 void AccessInfo::addLoad(
-    LoadPtr load,
+    const LoadPtr& load,
     const std::shared_ptr<Scope>& scope,
-    StmtPtr usage) {
+    const StmtPtr& usage) {
   block_ =
       block_ ? Block::getSharedParent(block_, scope->block()) : scope->block();
   first_usage_ = first_usage_ ? block_->getEnclosedRoot(first_usage_) : usage;
@@ -95,7 +98,7 @@ bool AccessInfo::overlaps(const std::shared_ptr<AccessInfo>& other) {
   return overlap;
 }
 
-bool AccessInfo::dependsOnVar(VarPtr v) {
+bool AccessInfo::dependsOnVar(const VarPtr& v) {
   VarFinder vf;
   for (const auto& i : indices_) {
     i->accept(&vf);
@@ -147,7 +150,7 @@ void Scope::closeAccess(const std::shared_ptr<AccessInfo>& info) {
   closedAccesses_.push_back(info);
 }
 
-AccessHashMap& Scope::getAccessMapByBuf(BufPtr b) {
+AccessHashMap& Scope::getAccessMapByBuf(const BufPtr& b) {
   auto it = openAccesses_.find(b);
   if (it == openAccesses_.end()) {
     // create and return
@@ -187,7 +190,7 @@ void RegisterizerAnalysis::closeAccessIntoScope(
   scope->closeAccess(info);
 }
 
-void RegisterizerAnalysis::visit(ForPtr v) {
+void RegisterizerAnalysis::visit(const ForPtr& v) {
   if (v->loop_options().is_gpu_block_index() ||
       v->loop_options().is_gpu_thread_index()) {
     throw malformed_input(
@@ -269,9 +272,9 @@ void RegisterizerAnalysis::visit(ForPtr v) {
 
   // having hoisted, now we can merge normally.
   mergeCurrentScopeIntoParent();
-};
+}
 
-void RegisterizerAnalysis::visit(CondPtr v) {
+void RegisterizerAnalysis::visit(const CondPtr& v) {
   ExprPtr condition = v->condition();
   BlockPtr true_stmt = v->true_stmt();
   BlockPtr false_stmt = v->false_stmt();
@@ -311,7 +314,7 @@ void RegisterizerAnalysis::visit(CondPtr v) {
 // IfThenElses are just like Conds except they are not Stmts, which means no
 // registerization can occur internally. However, the first reference to an
 // access can occur within one if its visible outside the condition.
-void RegisterizerAnalysis::visit(IfThenElsePtr v) {
+void RegisterizerAnalysis::visit(const IfThenElsePtr& v) {
   ExprPtr condition = v->condition();
   ExprPtr true_value = v->true_value();
   ExprPtr false_value = v->false_value();
@@ -346,7 +349,7 @@ void RegisterizerAnalysis::visit(IfThenElsePtr v) {
   }
 }
 
-void RegisterizerAnalysis::visit(LetPtr v) {
+void RegisterizerAnalysis::visit(const LetPtr& v) {
   currentScope_->addLocalVar(v->var());
 
   stmtStack_.push_front(v);
@@ -354,7 +357,7 @@ void RegisterizerAnalysis::visit(LetPtr v) {
   stmtStack_.pop_front();
 }
 
-void RegisterizerAnalysis::visit(BlockPtr v) {
+void RegisterizerAnalysis::visit(const BlockPtr& v) {
   auto prev_scope = currentScope_;
   if (currentScope_->block() != v) {
     currentScope_ = std::make_shared<Scope>(v, prev_scope);
@@ -382,7 +385,7 @@ void RegisterizerAnalysis::visit(BlockPtr v) {
   }
 }
 
-void RegisterizerAnalysis::visit(StorePtr v) {
+void RegisterizerAnalysis::visit(const StorePtr& v) {
   stmtStack_.push_front(v);
   v->value()->accept(this);
   stmtStack_.pop_front();
@@ -436,7 +439,7 @@ void RegisterizerAnalysis::visit(StorePtr v) {
   }
 }
 
-void RegisterizerAnalysis::visit(LoadPtr v) {
+void RegisterizerAnalysis::visit(const LoadPtr& v) {
   if (v->indices().empty()) {
     // already a scalar.
     return;
@@ -485,7 +488,7 @@ void RegisterizerAnalysis::visit(LoadPtr v) {
 }
 
 // Loop and Conditional scopes are different in that it may or may not be
-// possible to hoist the intializer of a scalar variable outside the block
+// possible to hoist the initializer of a scalar variable outside the block
 // depending on if we can tell that the Buffer access is valid outside. This is
 // tricky because the access that demonstrates this may be later in the tree and
 // we haven't encountered it yet.
@@ -523,7 +526,7 @@ void RegisterizerAnalysis::mergeHiddenScope(bool allowClosed) {
 void RegisterizerAnalysis::mergeCurrentScopeIntoParent() {
   auto parent = currentScope_->parent();
 
-  // copy across current closed accceses, merging / closing as necessary
+  // copy across current closed accesses, merging / closing as necessary
   for (auto& candidate : currentScope_->closedAccesses()) {
     auto& parentAccesses = parent->getAccessMapByBuf(candidate->buf());
 
@@ -648,7 +651,7 @@ std::vector<std::shared_ptr<AccessInfo>> RegisterizerAnalysis::getCandidates() {
 
 // RegisterizerReplacer
 
-ExprPtr RegisterizerReplacer::mutate(LoadPtr v) {
+ExprPtr RegisterizerReplacer::mutate(const LoadPtr& v) {
   auto it = loadToAccess_.find(v);
   if (it == loadToAccess_.end()) {
     // This access cannot be registerized.
@@ -660,9 +663,9 @@ ExprPtr RegisterizerReplacer::mutate(LoadPtr v) {
   return info->replacement().var;
 }
 
-StmtPtr RegisterizerReplacer::mutate(StorePtr v) {
+StmtPtr RegisterizerReplacer::mutate(const StorePtr& v) {
   if (eliminatedIntializers_.count(v) != 0) {
-    // This store is the intializer for a scalar var that is already inserted.
+    // This store is the initializer for a scalar var that is already inserted.
     return nullptr;
   }
 
@@ -682,7 +685,7 @@ StmtPtr RegisterizerReplacer::mutate(StorePtr v) {
   return v;
 }
 
-StmtPtr RegisterizerReplacer::mutate(BlockPtr v) {
+StmtPtr RegisterizerReplacer::mutate(const BlockPtr& v) {
   auto& scope = parentToAccesses_[v];
 
   std::vector<StmtPtr> stmts;
@@ -731,7 +734,7 @@ void RegisterizerReplacer::buildReplacements() {
   for (auto& info : infoSet_) {
     VarPtr v = alloc<Var>(
         info->buf()->name_hint() + "_" +
-            c10::to_string(getBufferAccessCount(info->buf())),
+            std::to_string(getBufferAccessCount(info->buf())),
         info->buf()->dtype());
 
     info->replacement().var = v;
