@@ -20,10 +20,11 @@
 #include <torch/csrc/lazy/ts_backend/ts_eager_fallback.h>
 #include <torch/library.h>
 
+#include <utility>
+
 using at::Tensor;
 
-namespace torch {
-namespace lazy {
+namespace torch::lazy {
 namespace {
 
 at::Tensor CreateLtcTensor(
@@ -267,7 +268,7 @@ at::Tensor LazyNativeFunctions::_to_copy(
             std::move(node), lazy_self->GetDevice()));
     return result;
   }
-};
+}
 
 at::Tensor LazyNativeFunctions::empty_symint(
     at::SymIntArrayRef sym_size,
@@ -422,7 +423,7 @@ at::Tensor LazyNativeFunctions::narrow_copy_symint(
     c10::SymInt start,
     c10::SymInt length) {
   return at::functionalization::functionalize_aten_op_symint<ATEN_OP(
-      narrow_copy)>::call(self, dim, start, length);
+      narrow_copy)>::call(self, dim, std::move(start), std::move(length));
 }
 at::Tensor LazyNativeFunctions::pixel_shuffle(
     const at::Tensor& self,
@@ -442,7 +443,7 @@ at::Tensor LazyNativeFunctions::select_backward_symint(
     int64_t dim,
     c10::SymInt index) {
   return at::functionalization::functionalize_aten_op_symint<ATEN_OP(
-      select_backward)>::call(grad_output, input_sizes, dim, index);
+      select_backward)>::call(grad_output, input_sizes, dim, std::move(index));
 }
 at::Tensor LazyNativeFunctions::_trilinear(
     const at::Tensor& i1,
@@ -518,8 +519,14 @@ at::Tensor LazyNativeFunctions::slice_backward_symint(
     c10::SymInt start,
     c10::SymInt end,
     c10::SymInt step) {
-  return at::functionalization::functionalize_aten_op_symint<ATEN_OP(
-      slice_backward)>::call(grad_output, input_sizes, dim, start, end, step);
+  return at::functionalization::
+      functionalize_aten_op_symint<ATEN_OP(slice_backward)>::call(
+          grad_output,
+          input_sizes,
+          dim,
+          std::move(start),
+          std::move(end),
+          std::move(step));
 }
 
 // re-use the composite kernel from core, that way we don't need to provide a
@@ -537,5 +544,4 @@ std::tuple<Tensor, Tensor, Tensor> LazyNativeFunctions::native_group_norm(
       input, weight, bias, N, C, HxW, group, eps);
 }
 
-} // namespace lazy
-} // namespace torch
+} // namespace torch::lazy
