@@ -1,9 +1,6 @@
 #include <torch/csrc/profiler/orchestration/python_tracer.h>
 
-namespace torch {
-namespace profiler {
-namespace impl {
-namespace python_tracer {
+namespace torch::profiler::impl::python_tracer {
 namespace {
 MakeFn make_fn;
 
@@ -32,7 +29,4 @@ std::unique_ptr<PythonTracerBase> PythonTracerBase::make(RecordQueue* queue) {
   }
   return make_fn(queue);
 }
-} // namespace python_tracer
-} // namespace impl
-} // namespace profiler
-} // namespace torch
+} // namespace torch::profiler::impl::python_tracer
