@@ -1,24 +1,24 @@
 # mypy: allow-untyped-defs
-from dataclasses import dataclass, field
-from collections import defaultdict
 import copy
-import torch
-from torch.fx import (
-    Node,
-    Graph,
-)
-from torch.fx._compatibility import compatibility
-from typing import Dict, List, Set, Any, Union, Tuple
 import logging
 import os
+from collections import defaultdict
+from dataclasses import dataclass, field
+from typing import Any, Dict, List, Set, Tuple, Union
+
+import torch
+from torch.fx import Graph, Node
+from torch.fx._compatibility import compatibility
+
+
+__all__ = ["SubgraphMatcher", "InternalMatch"]
 
-__all__ = ['SubgraphMatcher', 'InternalMatch']
 
 # Set`PYTORCH_MATCHER_LOGLEVEL=INFO` to see debug logs
 def _init_logger():
     logger = logging.getLogger(__name__)
 
-    level = os.environ.get('PYTORCH_MATCHER_LOGLEVEL', 'WARNING').upper()
+    level = os.environ.get("PYTORCH_MATCHER_LOGLEVEL", "WARNING").upper()
     logger.setLevel(level)
     console = logging.StreamHandler()
     formatter = logging.Formatter("%(filename)s > %(message)s")
@@ -29,8 +29,10 @@ def _init_logger():
     logger.propagate = False
     return logger
 
+
 logger = _init_logger()
 
+
 @compatibility(is_backward_compatible=False)
 @dataclass
 class InternalMatch:
@@ -50,17 +52,24 @@ class InternalMatch:
     name_node_map: Dict[str, Node] = field(default_factory=dict)
 
     def __copy__(self):
-        return InternalMatch(anchors=self.anchors, nodes_map=self.nodes_map.copy(),
-                             placeholder_nodes=self.placeholder_nodes.copy(),
-                             returning_nodes=self.returning_nodes.copy())
+        return InternalMatch(
+            anchors=self.anchors,
+            nodes_map=self.nodes_map.copy(),
+            placeholder_nodes=self.placeholder_nodes.copy(),
+            returning_nodes=self.returning_nodes.copy(),
+        )
+
 
 @compatibility(is_backward_compatible=False)
 class SubgraphMatcher:
-    def __init__(self, pattern: Graph,
-                 match_output: bool = False,
-                 match_placeholder: bool = False,
-                 remove_overlapping_matches: bool = True,
-                 ignore_literals: bool = False) -> None:
+    def __init__(
+        self,
+        pattern: Graph,
+        match_output: bool = False,
+        match_placeholder: bool = False,
+        remove_overlapping_matches: bool = True,
+        ignore_literals: bool = False,
+    ) -> None:
         """
         Args:
             pattern: the targeted matching pattern, represented in fx.Graph.
@@ -81,16 +90,21 @@ class SubgraphMatcher:
         self.ignore_literals = ignore_literals
 
         if len(pattern.nodes) == 0:
-            raise ValueError("SubgraphMatcher cannot be initialized with an empty pattern")
+            raise ValueError(
+                "SubgraphMatcher cannot be initialized with an empty pattern"
+            )
 
         for node in pattern.nodes:
             if node.op != "output":
-                assert len(node.users) > 0, \
-                       "SubgraphMatcher cannot be initialized with an pattern with dead code"
+                assert (
+                    len(node.users) > 0
+                ), "SubgraphMatcher cannot be initialized with an pattern with dead code"
 
         # TODO: assert pattern is a connected graph
 
-        self.pattern_placeholder_nodes = [n for n in pattern.nodes if n.op == "placeholder"]
+        self.pattern_placeholder_nodes = [
+            n for n in pattern.nodes if n.op == "placeholder"
+        ]
         output_node = next(iter(reversed(pattern.nodes)))
         # nodes returned by outputs
         self.pattern_returning_nodes: List[Node] = output_node.all_input_nodes
@@ -101,25 +115,17 @@ class SubgraphMatcher:
         else:
             # If a node has output_node as the ONLY user, then this node is a graph sink,
             # and should be matched against as an anchor
-            self.pattern_anchors = [n for n in output_node.all_input_nodes if len(n.users) == 1]
+            self.pattern_anchors = [
+                n for n in output_node.all_input_nodes if len(n.users) == 1
+            ]
 
     def _match_attributes(self, pn: Node, gn: Node) -> bool:
         # Attributes matching is complicated. Right now we only support matching constant tensor
         assert isinstance(pn.target, str), f"pn.target {pn.target} must be a string."
         assert isinstance(gn.target, str), f"gn.target {gn.target} must be a string."
 
-        # TODO(tmanlaibaatar) should probably make this actual API
-        def _getattr(model: torch.fx.GraphModule, attr_name: str):
-            *prefix, field = attr_name.split(".")
-            t = model
-            for item in prefix:
-                t = getattr(t, item, None)  # type: ignore[assignment]
-                assert t is not None
-
-            return getattr(t, field)
-
-        pn_value = _getattr(pn.graph.owning_module, pn.target)
-        gn_value = _getattr(gn.graph.owning_module, gn.target)
+        pn_value = torch.fx.graph_module._get_attr(pn.graph.owning_module, pn.target)
+        gn_value = torch.fx.graph_module._get_attr(gn.graph.owning_module, gn.target)
 
         if type(pn_value) != type(gn_value):
             return False
@@ -149,7 +155,9 @@ class SubgraphMatcher:
         # that are part of `pattern`
 
         # Placeholders can be used by other nodes in the graphs
-        lookup: Dict[Node, Node] = {gn : pn for pn, gn in nodes_map.items() if pn.op != "placeholder"}
+        lookup: Dict[Node, Node] = {
+            gn: pn for pn, gn in nodes_map.items() if pn.op != "placeholder"
+        }
 
         for gn, pn in lookup.items():
             # nodes returned by output are allowed to be used in other areas of the graph
@@ -163,8 +171,10 @@ class SubgraphMatcher:
                     return False
         return True
 
-    def _remove_overlapping_matches(self, matches: List[InternalMatch]) -> List[InternalMatch]:
-        non_overlapping_matches: List[InternalMatch] = list()
+    def _remove_overlapping_matches(
+        self, matches: List[InternalMatch]
+    ) -> List[InternalMatch]:
+        non_overlapping_matches: List[InternalMatch] = []
         nodes_matched: Set[Node] = set()
 
         for match in matches:
@@ -182,7 +192,9 @@ class SubgraphMatcher:
         return non_overlapping_matches
 
     def _match_literals(self, pn: Any, gn: Any, match: InternalMatch) -> bool:
-        assert not (isinstance(pn, Node) and isinstance(gn, Node)), "pn and gn cannot both be Node"
+        assert not (
+            isinstance(pn, Node) and isinstance(gn, Node)
+        ), "pn and gn cannot both be Node"
 
         if isinstance(pn, Node) and not isinstance(gn, Node):
             if pn.op == "placeholder":
@@ -203,7 +215,9 @@ class SubgraphMatcher:
     def _match_nodes(self, pn: Node, gn: Node, match: InternalMatch) -> bool:
         logger.info("  matching %s to %s", pn, gn)
 
-        assert isinstance(pn, Node) and isinstance(gn, Node), str(f"pn and gn must be Node, pn: {pn}, gn: {gn}")
+        assert isinstance(pn, Node) and isinstance(gn, Node), str(
+            f"pn and gn must be Node, pn: {pn}, gn: {gn}"
+        )
 
         # Check if we've already matched these nodes in the current
         # traversal
@@ -240,7 +254,9 @@ class SubgraphMatcher:
                 elif isinstance(a1, (list, tuple)) and isinstance(a2, (list, tuple)):
                     matched = _match_args(a1, a2)
                 else:
-                    matched = self._match_literals(a1, a2, match) or self.ignore_literals
+                    matched = (
+                        self._match_literals(a1, a2, match) or self.ignore_literals
+                    )
 
                 if not matched:
                     return False
@@ -250,9 +266,12 @@ class SubgraphMatcher:
         # Flatten all args/kwargs into 1 list of args
         pn_args, gn_args = None, None
         if (
-            (len(pn.args) != len(gn.args) or list(pn.kwargs.keys()) != list(gn.kwargs.keys())) and
-            pn.op == "call_function" and
-            isinstance(pn.target, torch._ops.OpOverload)
+            (
+                len(pn.args) != len(gn.args)
+                or list(pn.kwargs.keys()) != list(gn.kwargs.keys())
+            )
+            and pn.op == "call_function"
+            and isinstance(pn.target, torch._ops.OpOverload)
         ):
             args_schema = pn.target._schema.arguments
 
@@ -270,7 +289,9 @@ class SubgraphMatcher:
             pn_args = get_all_arguments(pn.args, pn.kwargs)
             gn_args = get_all_arguments(gn.args, gn.kwargs)
 
-        elif len(pn.args) == len(gn.args) and list(pn.kwargs.keys()) == list(gn.kwargs.keys()):
+        elif len(pn.args) == len(gn.args) and list(pn.kwargs.keys()) == list(
+            gn.kwargs.keys()
+        ):
             pn_args = list(pn.args)
             gn_args = list(gn.args)
             pn_args.extend(list(pn.kwargs.values()))
@@ -279,10 +300,10 @@ class SubgraphMatcher:
             match_found = False
 
         match_found = (
-            match_found and
-            pn_args is not None and
-            gn_args is not None and
-            _match_args(pn_args, gn_args)
+            match_found
+            and pn_args is not None
+            and gn_args is not None
+            and _match_args(pn_args, gn_args)
         )
 
         if not match_found:
@@ -344,8 +365,12 @@ class SubgraphMatcher:
 
         def backtracking(anchor_index, match):
             if anchor_index == len(match_candidates_list):
-                match.placeholder_nodes = [match.nodes_map[pn] for pn in self.pattern_placeholder_nodes]
-                match.returning_nodes = [match.nodes_map[pn] for pn in self.pattern_returning_nodes]
+                match.placeholder_nodes = [
+                    match.nodes_map[pn] for pn in self.pattern_placeholder_nodes
+                ]
+                match.returning_nodes = [
+                    match.nodes_map[pn] for pn in self.pattern_returning_nodes
+                ]
                 matches.append(match)
 
                 logger.info("Found a match: %s\n", match)
@@ -362,7 +387,9 @@ class SubgraphMatcher:
                     # match next anchor
                     backtracking(anchor_index + 1, match)
                 else:
-                    logger.info("Failed to match anchor %s to %s\n", pattern_anchor, node)
+                    logger.info(
+                        "Failed to match anchor %s to %s\n", pattern_anchor, node
+                    )
 
                 # revert to saved_match before matching with current anchor
                 match = copy.copy(saved_match)
@@ -376,25 +403,37 @@ class SubgraphMatcher:
         matches = [match for match in matches if self._is_contained(match.nodes_map)]
         after = len(matches)
         if before != after:
-            logger.info("Filtered out %s matches because they are not fully contained", before - after)
+            logger.info(
+                "Filtered out %s matches because they are not fully contained",
+                before - after,
+            )
 
         # filter out the matches that form a cycle if the subgraph is fused
         valid_matches = []
         for match in matches:
-            matched_compute_nodes = \
-                [gn for pn, gn in match.nodes_map.items() if pn.op not in {"placeholder", "output"}]
+            matched_compute_nodes = [
+                gn
+                for pn, gn in match.nodes_map.items()
+                if pn.op not in {"placeholder", "output"}
+            ]
             if validate_partition(matched_compute_nodes):
                 valid_matches.append(match)
         if len(valid_matches) != len(matches):
-            logger.info("Filtered out %s matches because \
-                          matched subgraph would form a cycle if fused", len(matches) - len(valid_matches))
+            logger.info(
+                "Filtered out %s matches because \
+                          matched subgraph would form a cycle if fused",
+                len(matches) - len(valid_matches),
+            )
 
         if self.remove_overlapping_matches:
             before = len(valid_matches)
             matches = self._remove_overlapping_matches(valid_matches)
             after = len(matches)
             if before != after:
-                logger.info("Filtered out %s matches because matched subgraphs are overlapping", before - after)
+                logger.info(
+                    "Filtered out %s matches because matched subgraphs are overlapping",
+                    before - after,
+                )
 
         logger.info("Matches returned: %s", matches)
 
