@@ -1,17 +1,18 @@
+# mypy: allow-untyped-defs
 import ast
 import builtins
 import dis
 import enum
 import inspect
 import re
+import typing
 import warnings
-
 from textwrap import dedent
 from typing import Type
 
 import torch
-
 from torch._C import (
+    _GeneratorType,
     AnyType,
     AwaitType,
     BoolType,
@@ -33,8 +34,7 @@ from torch._C import (
     TupleType,
     UnionType,
 )
-from torch._sources import get_source_lines_and_file
-from .._jit_internal import (  # type: ignore[attr-defined]
+from torch._jit_internal import (  # type: ignore[attr-defined]
     _Await,
     _qualified_name,
     Any,
@@ -56,11 +56,14 @@ from .._jit_internal import (  # type: ignore[attr-defined]
     Tuple,
     Union,
 )
+from torch._sources import get_source_lines_and_file
+
 from ._state import _get_script_class
 
+
 if torch.distributed.rpc.is_available():
     from torch._C import RRefType
-    from .._jit_internal import is_rref, RRef
+    from torch._jit_internal import is_rref, RRef
 
 from torch._ops import OpOverloadPacket
 
@@ -197,7 +200,7 @@ def check_fn(fn, loc):
 
 
 def _eval_no_call(stmt, glob, loc):
-    """Evaluate statement as long as it does not contain any method/function calls"""
+    """Evaluate statement as long as it does not contain any method/function calls."""
     bytecode = compile(stmt, "", mode="eval")
     for insn in dis.get_instructions(bytecode):
         if "CALL" in insn.opname:
@@ -208,7 +211,7 @@ def _eval_no_call(stmt, glob, loc):
 
 
 def parse_type_line(type_line, rcb, loc):
-    """Parses a type annotation specified as a comment.
+    """Parse a type annotation specified as a comment.
 
     Example inputs:
         # type: (Tensor, torch.Tensor) -> Tuple[Tensor]
@@ -238,11 +241,11 @@ def parse_type_line(type_line, rcb, loc):
 
 
 def get_type_line(source):
-    """Tries to find the line containing a comment with the type annotation."""
+    """Try to find the line containing a comment with the type annotation."""
     type_comment = "# type:"
 
     lines = source.split("\n")
-    lines = [(line_num, line) for line_num, line in enumerate(lines)]
+    lines = list(enumerate(lines))
     type_lines = list(filter(lambda line: type_comment in line[1], lines))
     # `type: ignore` comments may be needed in JIT'ed functions for mypy, due
     # to the hack in torch/_VF.py.
@@ -305,7 +308,7 @@ def get_type_line(source):
 
 
 def split_type_line(type_line):
-    """Splits the comment with the type annotation into parts for argument and return types.
+    """Split the comment with the type annotation into parts for argument and return types.
 
     For example, for an input of:
         # type: (Tensor, torch.Tensor) -> Tuple[Tensor, Tensor]
@@ -319,13 +322,13 @@ def split_type_line(type_line):
         arrow_pos = type_line.index("->")
     except ValueError:
         raise RuntimeError(
-            "Syntax error in type annotation (cound't find `->`)"
+            "Syntax error in type annotation (couldn't find `->`)"
         ) from None
     return type_line[start_offset:arrow_pos].strip(), type_line[arrow_pos + 2 :].strip()
 
 
 def try_real_annotations(fn, loc):
-    """Tries to use the Py3.5+ annotation syntax to get the type."""
+    """Try to use the Py3.5+ annotation syntax to get the type."""
     try:
         # Note: anything annotated as `Optional[T]` will automatically
         # be returned as `Union[T, None]` per
@@ -398,6 +401,8 @@ def _fake_rcb(inp):
 
 
 def try_ann_to_type(ann, loc, rcb=None):
+    ann_args = typing.get_args(ann)  # always returns a tuple!
+
     if ann is inspect.Signature.empty:
         return TensorType.getInferred()
     if ann is None:
@@ -406,44 +411,44 @@ def try_ann_to_type(ann, loc, rcb=None):
         return TensorType.get()
     if is_tuple(ann):
         # Special case for the empty Tuple type annotation `Tuple[()]`
-        if len(ann.__args__) == 1 and ann.__args__[0] == ():
+        if len(ann_args) == 1 and ann_args[0] == ():
             return TupleType([])
-        return TupleType([try_ann_to_type(a, loc) for a in ann.__args__])
+        return TupleType([try_ann_to_type(a, loc) for a in ann_args])
     if is_list(ann):
-        elem_type = try_ann_to_type(ann.__args__[0], loc)
+        elem_type = try_ann_to_type(ann_args[0], loc)
         if elem_type:
             return ListType(elem_type)
     if is_dict(ann):
-        key = try_ann_to_type(ann.__args__[0], loc)
-        value = try_ann_to_type(ann.__args__[1], loc)
+        key = try_ann_to_type(ann_args[0], loc)
+        value = try_ann_to_type(ann_args[1], loc)
         # Raise error if key or value is None
         if key is None:
             raise ValueError(
-                f"Unknown type annotation: '{ann.__args__[0]}' at {loc.highlight()}"
+                f"Unknown type annotation: '{ann_args[0]}' at {loc.highlight()}"
             )
         if value is None:
             raise ValueError(
-                f"Unknown type annotation: '{ann.__args__[1]}' at {loc.highlight()}"
+                f"Unknown type annotation: '{ann_args[1]}' at {loc.highlight()}"
             )
         return DictType(key, value)
     if is_optional(ann):
-        if issubclass(ann.__args__[1], type(None)):
-            contained = ann.__args__[0]
+        if issubclass(ann_args[1], type(None)):
+            contained = ann_args[0]
         else:
-            contained = ann.__args__[1]
+            contained = ann_args[1]
         valid_type = try_ann_to_type(contained, loc)
         msg = "Unsupported annotation {} could not be resolved because {} could not be resolved. At\n{}"
         assert valid_type, msg.format(repr(ann), repr(contained), repr(loc))
         return OptionalType(valid_type)
     if is_union(ann):
         # TODO: this is hack to recognize NumberType
-        if set(ann.__args__) == {int, float, complex}:
+        if set(ann_args) == {int, float, complex}:
             return NumberType.get()
         inner: List = []
         # We need these extra checks because both `None` and invalid
         # values will return `None`
         # TODO: Determine if the other cases need to be fixed as well
-        for a in ann.__args__:
+        for a in typing.get_args(ann):
             if a is None:
                 inner.append(NoneType.get())
             maybe_type = try_ann_to_type(a, loc)
@@ -452,21 +457,17 @@ def try_ann_to_type(ann, loc, rcb=None):
             inner.append(maybe_type)
         return UnionType(inner)  # type: ignore[arg-type]
     if torch.distributed.rpc.is_available() and is_rref(ann):
-        return RRefType(try_ann_to_type(ann.__args__[0], loc))
+        return RRefType(try_ann_to_type(ann_args[0], loc))
     if is_future(ann):
-        return FutureType(try_ann_to_type(ann.__args__[0], loc))
+        return FutureType(try_ann_to_type(ann_args[0], loc))
     if is_await(ann):
-        elementType = (
-            try_ann_to_type(ann.__args__[0], loc)
-            if hasattr(ann, "__args__")
-            else AnyType.get()
-        )
+        elementType = try_ann_to_type(ann_args[0], loc) if ann_args else AnyType.get()
         return AwaitType(elementType)
     if ann is float:
         return FloatType.get()
     if ann is complex:
         return ComplexType.get()
-    if ann is int:
+    if ann is int or ann is torch.SymInt:
         return IntType.get()
     if ann is str:
         return StringType.get()
@@ -480,6 +481,8 @@ def try_ann_to_type(ann, loc, rcb=None):
         return InterfaceType(ann.__torch_script_interface__)
     if ann is torch.device:
         return DeviceObjType.get()
+    if ann is torch.Generator:
+        return _GeneratorType.get()
     if ann is torch.Stream:
         return StreamObjType.get()
     if ann is torch.dtype:
