@@ -1,7 +1,9 @@
+from __future__ import annotations
+
 import itertools
 from abc import ABC
 from dataclasses import dataclass
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import Any
 
 import torchgen.api.dispatcher as dispatcher
 from torchgen.api.lazy import (
@@ -109,7 +111,7 @@ def node_ctor_inputs(schema: LazyIrSchema) -> str:
 
 def gen_fallback_code(
     schema: LazyIrSchema,
-    sig: Union[DispatcherSignature, NativeSignature],
+    sig: DispatcherSignature | NativeSignature,
     overload_name: str,
 ) -> str:
     """
@@ -147,9 +149,9 @@ def aten_symbol(schema: LazyIrSchema) -> str:
 # converts  all tensor-like arguments to meta tensors. Returns:
 # (1) a string containing all of the logic that does the conversions.
 # (2) a context, to be used by translate(), with all of the relevant bindings.
-def convert_to_meta_tensors(sig: DispatcherSignature) -> Tuple[str, List[Binding]]:
-    context: List[Binding] = []
-    unwrapped_tensor_args: List[str] = []
+def convert_to_meta_tensors(sig: DispatcherSignature) -> tuple[str, list[Binding]]:
+    context: list[Binding] = []
+    unwrapped_tensor_args: list[str] = []
     for arg in sig.arguments():
         if isinstance(arg.argument, Argument) and arg.argument.type.is_tensor_like():
             unwrapped_name = f"{arg.name}_meta"
@@ -171,7 +173,7 @@ class GenLazyIR(ABC):
     use_lazy_shape: bool
 
     @method_with_native_function
-    def __call__(self, f: Union[NativeFunctionsGroup, NativeFunction]) -> List[str]:
+    def __call__(self, f: NativeFunctionsGroup | NativeFunction) -> list[str]:
         func = f.functional.func if isinstance(f, NativeFunctionsGroup) else f.func
         metadata = self.backend_index.get_kernel(
             f.functional if isinstance(f, NativeFunctionsGroup) else f
@@ -236,7 +238,7 @@ class GenLazyIR(ABC):
               /* num_outputs */ {len(schema.returns)},
               torch::lazy::MHash({scalar_hashes}))"""
 
-    def gen(self, schema: LazyIrSchema) -> List[str]:
+    def gen(self, schema: LazyIrSchema) -> list[str]:
         opkind = schema.opkind or aten_symbol(schema)
 
         # for now, we just want one IR class decl and soon after also the method defs
@@ -413,7 +415,7 @@ class GenLazyNativeFuncDefinition:
     def lazy_tensor_decls(self, func: NativeFunction, schema: LazyIrSchema) -> str:
         value_args = schema.filtered_args(values=True, scalars=False)
         # Generates lazy_{name} variables for LazyTensors wrapping input tensors
-        lazy_tensor_decls: List[str] = []
+        lazy_tensor_decls: list[str] = []
         for arg in value_args:
             if arg.is_wrapped_scalar:
                 if isinstance(arg.lazy_type, OptionalCType):
@@ -460,7 +462,7 @@ class GenLazyNativeFuncDefinition:
         func: NativeFunction,
         schema: LazyIrSchema,
         metadata: BackendMetadata,
-        sig: Union[DispatcherSignature, NativeSignature],
+        sig: DispatcherSignature | NativeSignature,
     ) -> str:
         if self.gen_forced_fallback_code:
             return gen_fallback_code(
@@ -574,7 +576,7 @@ std::vector<torch::lazy::Shape> shapes{torch::lazy::Shape(out_meta.scalar_type()
         }}
         """
 
-    def create_lazy_tensor(self, first_tensor_name: Optional[str] = None) -> str:
+    def create_lazy_tensor(self, first_tensor_name: str | None = None) -> str:
         # xla uses an instance method for tensor creation, for the time being
         if self.create_from_first_tensor:
             # TODO(whc) remove this if XLA switches to using static method for creation
@@ -615,7 +617,7 @@ std::vector<torch::lazy::Shape> shapes{torch::lazy::Shape(out_meta.scalar_type()
         return bridge_str
 
     @method_with_native_function
-    def __call__(self, func: NativeFunction) -> List[str]:
+    def __call__(self, func: NativeFunction) -> list[str]:
         sig = kernel_signature(func, self.backend_index)
         metadata = self.backend_index.get_kernel(func)
         assert metadata is not None
@@ -639,7 +641,7 @@ class ComputeShapeSignature:
     Here we use the base name as the suffix of the signature to avoid generating for in-place variants.
     """
 
-    def __init__(self, kernel_name: str, f: NativeFunction, *, symint: bool):
+    def __init__(self, kernel_name: str, f: NativeFunction, *, symint: bool) -> None:
         self.__schema = LazyIrSchema(f.func, symint=symint)
         self.__dispatch_args = ", ".join(
             [a.decl() for a in dispatcher.arguments(f.func, symint=symint)]
@@ -670,7 +672,7 @@ class GenLazyShapeInferenceDefinition:
     tensor_class: str
 
     @method_with_native_function
-    def __call__(self, f: NativeFunction) -> List[str]:
+    def __call__(self, f: NativeFunction) -> list[str]:
         metadata = self.backend_index.get_kernel(f)
         assert metadata is not None
 
@@ -687,8 +689,8 @@ class GenLazyShapeInferenceDefinition:
 
 
 def generate_non_native_lazy_ir_nodes(
-    non_native: List[Dict[str, Any]], gen_lazy_ir: GenLazyIR
-) -> List[str]:
+    non_native: list[dict[str, Any]], gen_lazy_ir: GenLazyIR
+) -> list[str]:
     """Generate the non-native lazy IR node classes"""
     nodes = []
     for op in non_native:
